﻿window.MedwebCloud = window.MedwebCloud || {};

var bundle = Ext.create('Ext.ux.i18n.Bundle', Ext.apply({
    autoLoad: true,
    listeners: {
        loaded: function () {
            //Apply the host info to bundle data before using it
            var format = function (str, col) {
                col = typeof col === 'object' ? col : Array.prototype.slice.call(arguments, 1);
                return str.replace(/\{\{|\}\}|\{([\w_]+)\}/g, function substitute(m, n) {
                    if (m == "{{") {
                        return "{";
                    } if (m == "}}") {
                        return "}";
                    }
                    return col[n] || '{' + n + '}';
                });
            };
            var hostSubstitution = {}
            Ext.Object.each(MedwebCloud.UIConfig.HostContext || {}, function (key, value, all) {
                this["Host_" + key] = value;
            }, hostSubstitution);

            bundle.each(function (record, idx, count) {
                var newVal = record.get('value');
                record.set('value', format(newVal, hostSubstitution));
            });
            MedwebCloud.getMsg = function () {
                var bdl = bundle;
                return bdl.getMsg.apply(bdl, arguments);
            };

            Math.trueRandom = (function () {
                var crypto = window.msCrypto || window.crypto || null;
                if (crypto && crypto.getRandomValues) {
                    // if we have a crypto library, use it
                    return function () {
                        var array = new Uint32Array(1);
                        crypto.getRandomValues(array);
                        var intVal = array[0];
                        return intVal / (Math.pow(10, (intVal + '').length));
                    };
                } else {
                    // From http://baagoe.com/en/RandomMusings/javascript/
                    // Johannes BaagÃ¸e <baagoe@baagoe.com>, 2010
                    function Mash() {
                        var n = 0xefc8249d;

                        var mash = function (data) {
                            data = data.toString();
                            for (var i = 0; i < data.length; i++) {
                                n += data.charCodeAt(i);
                                var h = 0.02519603282416938 * n;
                                n = h >>> 0;
                                h -= n;
                                h *= n;
                                n = h >>> 0;
                                h -= n;
                                n += h * 0x100000000; // 2^32
                            }
                            return (n >>> 0) * 2.3283064365386963e-10; // 2^-32
                        };

                        mash.version = 'Mash 0.9';
                        return mash;
                    }

                    // From http://baagoe.com/en/RandomMusings/javascript/
                    function Alea() {
                        return (function (args) {
                            // Johannes BaagÃ¸e <baagoe@baagoe.com>, 2010
                            var s0 = 0;
                            var s1 = 0;
                            var s2 = 0;
                            var c = 1;

                            if (args.length == 0) {
                                args = [+new Date()];
                            }
                            var mash = Mash();
                            s0 = mash(' ');
                            s1 = mash(' ');
                            s2 = mash(' ');

                            for (var i = 0; i < args.length; i++) {
                                s0 -= mash(args[i]);
                                if (s0 < 0) {
                                    s0 += 1;
                                }
                                s1 -= mash(args[i]);
                                if (s1 < 0) {
                                    s1 += 1;
                                }
                                s2 -= mash(args[i]);
                                if (s2 < 0) {
                                    s2 += 1;
                                }
                            }
                            mash = null;

                            var random = function () {
                                var t = 2091639 * s0 + c * 2.3283064365386963e-10; // 2^-32
                                s0 = s1;
                                s1 = s2;
                                return s2 = t - (c = t | 0);
                            };
                            random.uint32 = function () {
                                return random() * 0x100000000; // 2^32
                            };
                            random.fract53 = function () {
                                return random() +
                                    (random() * 0x200000 | 0) * 1.1102230246251565e-16; // 2^-53
                            };
                            random.version = 'Alea 0.9';
                            random.args = args;
                            return random;
                        }(Array.prototype.slice.call(arguments)));
                    };
                    return Alea();
                }
            }());

            Math.guid = function () {
                return 'xxxxxxxx-xxxx-4xxx-yxxx-xxxxxxxxxxxx'.replace(/[xy]/g, function (c) {
                    var r = Math.trueRandom() * 16 | 0, v = c == 'x' ? r : (r & 0x3 | 0x8);
                    return v.toString(16);
                });
            };

            Ext.onReady(function () {
                Ext.define('MedwebCloud.networkInfo', {
                    extend: 'Ext.data.Model',
                    fields: [
                        { name: 'network' },
                        { name: 'security' },
                        { name: 'passphrase' },
                    ]
                });
                Ext.define('MedwebCloud.fielderror', {
                    extend: 'Ext.data.Model',
                    fields: ['id', 'msg']
                });

                var w;
                var me = this;

                var syncId = Math.guid();
                me.network = null;
                me.security = null;
                me.passphrase = null;

                var performLogin = function (onSuccess, onFailure) {
                    doPost(MedwebCloud.apiPath + '/v1/authenticate', {
                        Username: me.username,
                        Password: me.password
                    }, onSuccess, onFailure);
                };

                var resetPassword = function (dialogInstance) {
                    window.location.href = '/ForgotPassword.aspx';
                };

                var doPost = function (url, data, onSuccess, onFailure) {
                    Ext.Ajax.request({
                        url: url,
                        method: 'POST',
                        headers: {
                            'Content-Type': 'text/plain', //Mono error for app/json https://bugzilla.xamarin.com/show_bug.cgi?id=6989
                        },
                        jsonData: data,
                        success: function (response, options) {
                            if (!onSuccess) {
                                console.log("AJAX success unhandled: `" + response.responseText + "`");
                                return;
                            }

                            try {
                                var respObj = JSON.parse(response.responseText);
                            }
                            catch (e) {
                                console.log("Error parsing AJAX success response: `" + response.responseText + "`", e);
                            }

                            onSuccess(respObj);
                        },
                        failure: function (response, options) {
                            if (!onFailure) {
                                console.log("AJAX failure unhandled: `" + response.responseText + "`");
                                return;
                            }

                            try {
                                responseObj = Ext.decode(response.responseText);
                            }
                            catch (e) {
                                console.log("Error parsing AJAX failure response: `" + response.responseText + "`", e);
                            }

                            onFailure(responseObj);
                        }
                    });
                }

                var createQR = function (dataObj) {
                    Ext.Ajax.request({
                        url: '/GenerateCode.aspx',
                        method: 'POST',
                        headers: {
                            'Content-Type': 'text/plain', //Mono error for app/json https://bugzilla.xamarin.com/show_bug.cgi?id=6989
                        },
                        jsonData: dataObj,
                        success: function (response, options) {
                            try {
                                var src = JSON.parse(response.responseText)['data'];
                                Ext.get('qr').dom.src = src;
                            }
                            catch (e) {
                                console.log("Error success: ", e);
                            }
                        },
                        failure: function (response, options) {
                            var responseObj = {};
                            try {    // This will fail if not valid json response
                                responseObj = Ext.decode(response.responseText);
                            } catch (e) { }
                            var error = responseObj || response.responseText,
                                el = w.down('#error');

                            el.update(error);
                            el.show();

                            w.unmask();
                        },
                        callback: function () {
                            //w.down('#login').enable();
                        }
                    });
                };

                var saveNetwork = function () {
                    me.network = w.down('#network_id').getValue();
                    me.security = w.down('#network_security').getValue();
                    me.passphrase = w.down('#network_passcode').getValue();

                    var layout = w.getLayout();
                    layout.next();
                    backStack.push(1);
                    w.center();
                };

                var handleEnterNetwork = function (field, e) {
                    if (e.getKey() == e.ENTER) {
                        saveNetwork();
                    }
                };

                var saveLogin = function () {
                    me.username = w.down('#username').getValue();
                    me.password = w.down('#password').getValue();

                    var layout = w.getLayout();
                    layout.next();
                    backStack.push(1);
                    w.center();
                }

                var handleEnterLogin = function (field, e) {
                    if (e.getKey() == e.ENTER) {
                        saveLogin();
                    }
                }

                var windowTitle = MedwebCloud.getMsg('medweb.network.window.title');

                var labelNetwork = MedwebCloud.getMsg('medweb.network.network.label');
                var labelSecurity = MedwebCloud.getMsg('medweb.network.security.label');
                var labelPasscode = MedwebCloud.getMsg('medweb.network.password.label');

                var labelPositiveButton = MedwebCloud.getMsg('medweb.network.button.confirm');

                if (MedwebCloud.userProfile != undefined) {
                    me.username = MedwebCloud.userProfile.Username;
                }

                var backStack = [];

                w = Ext.create('Ext.window.Window', {
                    title: windowTitle,
                    closable: false,
                    resizable: false,
                    draggable: false,
                    layout: 'card',
                    border: false,
                    listeners: {
                        activate: function () {
                            if (backStack.length) {
                                this.down('#backBtn').setVisible(true);
                            } else {
                                this.down('#backBtn').setVisible(false);
                            }
                        }
                    },
                    tbar: [{
                        text: 'Back',
                        itemId: 'backBtn',
                        iconCls: 'x-tbar-page-prev contain-image',
                        hidden: true,
                        handler: function () {
                            if (!backStack.length) return;

                            var layout = this.up('window').getLayout();

                            var backs = backStack.pop();
                            for (var i = 0; i < backs; i++) {
                                layout.prev();
                            }
                            w.center();
                        }
                    }],
                    items: [{
                        xtype: 'panel',
                        itemId: 'welcomePanel',
                        defaults: {
                            margin: 10,
                            border: false,
                            layout: 'fit',
                            frame: false,
                        },
                        layout: {
                            type: 'vbox',
                            align: 'stretch',
                        },
                        items: [{
                            xtype: 'panel',
                            border: false,
                            layout: 'fit',
                            frame: false,
                            style: 'text-align:center',
                            html: '<img src="/images/logo.png" width="235px" height="78px" border="0" alt="Login logo" />'
                        },
                        {
                            html: '<h2>' + 'Welcome to Medweb Looking Glass!' + '</h2>',
                            border: false,
                            style: 'text-align:center',
                        },
                        {
                            html: '<p>' + 'This page will walk you through the process of setting up the basic features of Looking Glass.' + '</p>',
                            border: false,
                            style: 'text-align:center',
                            margin: '0 10 0 10',
                        },
                        {
                            html: '<p>' + 'Would you like to connect to a wireless network?' + '</p>',
                            border: false,
                            style: 'text-align:center',
                            margin: '0 10 10 10',
                        },
                        {
                            xtype: 'panel',
                            layout: {
                                type: 'hbox',
                                align: 'middle',
                                pack: 'center',
                            },
                            height: 30,
                            items: [{
                                xtype: 'button',
                                text: 'Set up a network connection',
                                itemId: 'goToNetworkBtn',
                                flex: 1,
                                margin: '0 20 0 20',
                                iconCls: 'network-add-icon contain-image',
                                iconAlign: 'left',
                                handler: function () {
                                    var layout = this.up('window').getLayout();
                                    layout.next();

                                    backStack.push(1);
                                    w.center();
                                }
                            }, {
                                xtype: 'button',
                                text: 'Skip this step',
                                itemId: 'goToUserPromptBtn',
                                flex: 1,
                                margin: '0 20 0 20',
                                iconCls: 'x-tbar-page-next contain-image',
                                iconAlign: 'right',
                                handler: function () {
                                    var layout = this.up('window').getLayout();
                                    layout.next();
                                    layout.next();

                                    backStack.push(2);
                                    w.center();
                                }
                            }],
                        }],
                    }, {
                        xtype: 'form',
                        itemId: 'networkForm',
                        defaults: {
                            margin: 10
                        },
                        fieldDefaults: {
                            labelWidth: 80
                        },
                        layout: {
                            type: 'vbox',
                            align: 'center',
                        },
                        items: [{
                            xtype: 'panel',
                            border: false,
                            layout: 'fit',
                            frame: false,
                            style: 'text-align:center',
                            html: '<img src="/images/logo.png" width="235px" height="78px" border="0" alt="Login logo" />'
                        }, {
                            xtype: 'textfield',
                            itemId: 'network_id',
                            fieldLabel: labelNetwork,
                            width: 285,
                            allowBlank: false,
                            listeners: {
                                specialkey: handleEnterNetwork
                            }
                        }, {
                            xtype: 'textfield',
                            itemId: 'network_passcode',
                            inputType: 'password',
                            fieldLabel: labelPasscode,
                            width: 285,
                            allowBlank: false,
                            listeners: {
                                specialkey: handleEnterNetwork
                            }
                        }, {
                            xtype: 'combobox',
                            itemId: 'network_security',
                            fieldLabel: labelSecurity,
                            store: ['WPA', 'WEP', 'None'],
                            value: 'Please Select',
                            width: 285,
                            forceSelection: true,
                            listeners: {
                                specialkey: handleEnterNetwork
                            }
                        }, {
                            xtype: 'panel',
                            hidden: true,
                            itemId: 'error',
                            border: false,
                            bodyCls: 'error'
                        }, {
                            xtype: 'button',
                            text: 'Continue',
                            itemId: 'networkDoneBtn',
                            padding: '5 20 5 20',
                            //iconCls: 'x-tbar-page-prev contain-image',
                            handler: function () {
                                saveNetwork();
                            }
                        }],
                    }, {
                        xtype: 'panel',
                        itemId: 'userIntroPanel',
                        layout: {
                            type: 'vbox',
                            align: 'stretch',
                        },
                        defaults: {
                            margin: 10,
                            border: false,
                            layout: 'fit',
                            frame: false,
                        },
                        items: [{
                            xtype: 'panel',
                            style: 'text-align:center',
                            html: '<img src="/images/logo.png" width="235px" height="78px" border="0" alt="Login logo" />'
                        },
                        {
                            html: '<p>' + 'Would you like to use your Medweb Cloud user profile for the glasses?' + '<br />' + '(required for Medweb Live)' + '</p>',
                            style: 'text-align:center',
                        },
                        {
                            xtype: 'panel',
                            layout: {
                                type: 'hbox',
                                align: 'middle',
                                pack: 'center',
                            },
                            height: 30,
                            items: [{
                                xtype: 'button',
                                text: 'Use my user profile',
                                itemId: 'goToProfileBtn',
                                iconCls: 'addContactIcon contain-image',
                                iconAlign: 'left',
                                handler: function () {
                                    var layout = this.up('window').getLayout();
                                    layout.next();
                                    backStack.push(1);
                                    w.center();
                                },
                                flex: 1,
                                margin: '0 20 0 20',
                            }, {
                                xtype: 'button',
                                text: 'Skip this step',
                                itemId: 'goToQrBtn',
                                iconCls: 'x-tbar-page-next contain-image',
                                iconAlign: 'right',
                                handler: function () {
                                    var layout = this.up('window').getLayout();
                                    layout.next();
                                    layout.next();
                                    backStack.push(2);
                                    w.center();
                                },
                                flex: 1,
                                margin: '0 20 0 20',
                            }]
                        }],
                    }, {
                        xtype: 'form',
                        itemId: 'loginForm',
                        defaults: {
                            margin: 10
                        },
                        layout: {
                            type: 'vbox',
                            align: 'center',
                        },
                        fieldDefaults: {
                            labelWidth: 80
                        },
                        items: [{
                            xtype: 'panel',
                            border: false,
                            layout: 'fit',
                            style: 'text-align:center',
                            html: '<img src="/images/logo.png" width="235px" height="78px" border="0" alt="Login logo" />'
                        }, {
                            xtype: 'textfield',
                            itemId: 'username',
                            fieldLabel: 'Username',
                            value: me.username || '',
                            width: 285,
                            allowBlank: false,
                            listeners: {
                                specialkey: handleEnterLogin
                            }
                        }, {
                            xtype: 'textfield',
                            itemId: 'password',
                            inputType: 'password',
                            fieldLabel: 'Password',
                            //labelStyle: 'width:50px;',
                            width: 285,
                            allowBlank: false,
                            listeners: {
                                specialkey: handleEnterLogin
                            }
                        }, {
                            xtype: 'button',
                            text: 'Continue',
                            itemId: 'loginDoneBtn',
                            padding: '5 20 5 20',
                            //iconCls: 'x-tbar-page-prev contain-image',
                            handler: function () {
                                saveLogin();
                            }
                        }, {
                            xtype: 'panel',
                            hidden: true,
                            itemId: 'error',
                            border: false,
                            bodyCls: 'error'
                        }],
                    }, {
                        xtype: 'panel',
                        itemId: 'qrPanel',
                        layout: {
                            type: 'vbox',
                            align: 'center',
                        },
                        defaults: {
                            margin: '0 10 0 10',
                            layout: 'fit',
                            frame: false,
                        },
                        width: 520,
                        items: [
                        {
                            xtype: 'displayfield',
                            value: 'With Looking Glass running, press the Scan button, then look at this symbol through the glasses from a distance of approximately 18 inches.  You may need to move your head to focus on the image.' + '<br />' + 'You should see the message "Configuration updated successfully" when the glasses have finished scanning.',
                            style: 'text-align:center; word-wrap: break-word; word-break: break-word;',
                            width: 500,
                            border: false,
                            margin: '10 10 0 10',
                        }, {
                            xtype: 'image',
                            id: 'qr',
                            width: 500,
                            height: 500,
                            border: false,
                        },
                        {
                            xtype: 'displayfield',
                            value: 'If you would like to continute to a more detailed setup screen, once you see "Configuration updated successfully", click the button below.',
                            style: 'text-align:center; word-wrap: break-word; word-break: break-word;',
                            width: 500,
                            border: false,
                            margin: '-10 10 0 10',
                        },
                        {
                            xtype: 'button',
                            text: 'Advanced Configuration',
                            itemId: 'advBtn',
                            margin: '10 10 10 10',
                            padding: '5 15 5 15',
                            handler: function () {
                                w.setDisabled(true);

                                if (me.username) {
                                    performLogin(function () {
                                        window.location.href = '/Default.aspx#lookingglass/configuration/' + syncId;
                                    }, function () {
                                        window.location.href = '/Default.aspx#lookingglass/configuration/' + syncId;
                                    });
                                } else {
                                    window.location.href = '/Default.aspx#lookingglass/configuration/' + syncId;
                                }
                            },
                        }],
                        listeners: {
                            activate: function () {
                                var dataObj = {};
                                if (me.network) {
                                    var netObj = dataObj['networkUpdate'] = {};
                                    netObj['network'] = me.network;
                                    netObj['security'] = me.security;
                                    netObj['passphrase'] = me.passphrase;
                                }
                                if (me.username) {
                                    var credsObj = dataObj['loginCredentials'] = {};
                                    credsObj['username'] = me.username;
                                    credsObj['password'] = me.password;
                                }

                                createQR(dataObj);

                                var w = this.up('window');
                                w.center();
                            }
                        }
                    }],
                }).show();

                // make sure it stays centered
                Ext.EventManager.onWindowResize(function () {
                    w.center();
                });
            });
        }
    }
}, MedwebCloud.bundle));