﻿(function (window, document, navigator, undefined) {
    "use strict";

    var Ext = window.Ext;
    var MedwebCloud = window.MedwebCloud;

    var ua = (navigator.userAgent || navigator.vendor || window.opera).toLowerCase();
    var location = window.location;
    var search = location.search;

    var start = (new Date()).valueOf();
    var reloadUrl = location.protocol + '//' + location.host + location.pathname + search + ((search.length > 1) ? "&" : '?') + '_r=1' + location.hash;
    var environment;

    if (search.indexOf("_r=") > -1) {
        environment = 'mobile';
    } else {
        if (ua.indexOf("android") > -1) {
            environment = 'android';
        } else if ((ua.indexOf("iphone") > -1) || (ua.indexOf("ipod") > -1) || (ua.indexOf("ipad") > -1)) {
            environment = 'ios';
        } else {
            environment = 'other';
        }
    }

    var googleStore = 'https://play.google.com/store/apps/details?id=com.medweb.Collage';
    var itunesStore = 'https://itunes.apple.com/us/app/medweb-collage/id875543886?ls=1&mt=8';

    var sid = MedwebCloud.getURLParameter('SessionId') || '';
    var scode = MedwebCloud.getURLParameter('SessionCode') || '';
    var username = MedwebCloud.getURLParameter('Username');
    username = username ? btoa(username) + '@' : '';

    var mode = MedwebCloud.getURLParameter('Mode') || '';
    mode = mode.toLowerCase();

    var hostBits = location.host.split('.');
    if (hostBits[0] === 'cloud' || hostBits[0] === 'alpha-cloud' || hostBits[0] === 'beta-cloud') {
        hostBits[0] = hostBits[0].replace('cloud', 'api');
    } else {
        hostBits[0] = 'api';
    }
    var host = hostBits.join('.');
    var appURL = 'medweb-cloud://' + username + host + '/';
    var desktopURL = 'medweb-collage://' + username + host + '/';
    var webURL = '/Default.aspx';

    if (sid && scode) {
        appURL += 'collage/session/' + sid + '/' + scode;
        desktopURL += 'collage/session/' + sid;
        webURL += '#collage/session/' + sid;
    }

    function appLoader(url, store, complete) {
        window.setTimeout(function () {
            var end = (new Date()).valueOf();
            if (end - start < 1000) { //probably didn't load the app
                window.setTimeout(function () { //Always go to the complete fallback
                    window.location = complete;
                }, 5);
                window.location = store;
            } else {
                window.location = complete;
            }
        }, 5);
        window.location = url;
    }

    window.MedwebCloud = window.MedwebCloud || {};
    // Setup the ExtJS Loader
    Ext.Loader.setConfig({
        enabled: true,
        paths: {
            'Ext.ux': '/scripts/app/ux',
        }
    });

    if (environment == 'android') {
        appLoader(appURL, googleStore, reloadUrl);
    } else if (environment == 'ios') {
        appLoader(appURL, itunesStore, reloadUrl);
    } else if (environment == 'mobile') {
        var format = function (str, col) {
            col = typeof col === 'object' ? col : Array.prototype.slice.call(arguments, 1);
            return str.replace(/\{\{|\}\}|\{(\w+)\}/g, function substitute(m, n) {
                if (m == "{{") {
                    return "{";
                } if (m == "}}") {
                    return "}";
                }
                return col[n] || n;
            });
        };

        var bundle = Ext.create('Ext.ux.i18n.Bundle', Ext.apply({
            autoLoad: true,
            listeners: {
                loaded: function () {
                    //Apply the host info to bundle data before using it
                    var format = function (str, col) {
                        col = typeof col === 'object' ? col : Array.prototype.slice.call(arguments, 1);
                        return str.replace(/\{\{|\}\}|\{([\w_]+)\}/g, function substitute(m, n) {
                            if (m == "{{") {
                                return "{";
                            } if (m == "}}") {
                                return "}";
                            }
                            return col[n] || '{' + n + '}';
                        });
                    };
                    var hostSubstitution = {}
                    Ext.Object.each(MedwebCloud.UIConfig.HostContext || {}, function (key, value, all) {
                        this["Host_" + key] = value;
                    }, hostSubstitution);

                    bundle.each(function (record, idx, count) {
                        var newVal = record.get('value');
                        record.set('value', format(newVal, hostSubstitution));
                    });

                    MedwebCloud.getMsg = function () {
                        var bdl = bundle;
                        return format(bdl.getMsg.apply(bdl, arguments), MedwebCloud.UIConfig.Host || {});
                    };
                    var windowTitle = MedwebCloud.getMsg('medweb.join.mobile.window.title');
                    var launchLabel = MedwebCloud.getMsg('medweb.join.mobile.launch');

                    //Show Ext window
                    Ext.onReady(function () {
                        // ok, the user wants to join a session; first, we need to determine if they're on a mobile device
                        // then, if so, we auto-launch to the store/app
                        // if not, we use the extjs picker to let them select

                        var w = Ext.create('Ext.window.Window', {
                            title: windowTitle,
                            closable: false,
                            resizable: false,
                            draggable: false,
                            layout: 'vbox',
                            border: false,
                            items: [{
                                xtype: 'form',
                                defaults: {
                                    margin: 10
                                },
                                items: [{
                                    xtype: 'panel',
                                    border: false,
                                    layout: 'fit',
                                    style: 'text-align:center',
                                    html: '<img src="/images/logo.png" width="235px" height="78px" border="0" alt="Login logo" />'
                                }, {
                                    xtype: 'button', text: launchLabel, handler: function () {
                                        window.location = appURL;
                                    }
                                }, {
                                    xtype: 'box',
                                    autoEl: { tag: 'a', href: googleStore, html: '<img src="/images/GooglePlay.svg" width="135" height="40" border="0" alt="Get it on Google Play" style="display:block;"/>', style: 'text-align:center;padding:5px;display:inline-block;vertical-align:bottom' },
                                }, {
                                    xtype: 'box',
                                    autoEl: { tag: 'a', href: itunesStore, html: '<img src="/images/iTunes.svg" width="135" height="40" border="0" alt="Download on the App Store" style="display:block;" />', style: 'text-align:center;padding:5px;display:inline-block;vertical-align:bottom' },
                                }]
                            }]
                        }).show();

                        // make sure it stays centered
                        Ext.EventManager.onWindowResize(function () {
                            w.center();
                        });

                        MedwebCloud.ready = true;
                    });
                }
            }
        }, MedwebCloud.bundle));
    } else {
        if (mode === 'web') {
            window.location.assign(webURL);
        } else if(mode === 'app'){
            window.location.assign(desktopURL);
        } else{
            var bundle = Ext.create('Ext.ux.i18n.Bundle', Ext.apply({
                autoLoad: true,
                listeners: {
                    loaded: function () {
                        //Apply the host info to bundle data before using it
                        var format = function (str, col) {
                            col = typeof col === 'object' ? col : Array.prototype.slice.call(arguments, 1);
                            return str.replace(/\{\{|\}\}|\{([\w_]+)\}/g, function substitute(m, n) {
                                if (m == "{{") {
                                    return "{";
                                } if (m == "}}") {
                                    return "}";
                                }
                                return col[n] || '{' + n + '}';
                            });
                        };
                        var hostSubstitution = {}
                        Ext.Object.each(MedwebCloud.UIConfig.HostContext || {}, function (key, value, all) {
                            this["Host_" + key] = value;
                        }, hostSubstitution);

                        bundle.each(function (record, idx, count) {
                            var newVal = record.get('value');
                            record.set('value', format(newVal, hostSubstitution));
                        });
                        MedwebCloud.getMsg = function () {
                            var bdl = bundle;
                            return bdl.getMsg.apply(bdl, arguments);
                        };
                        //Show Ext window
                        Ext.onReady(function () {
                            // ok, the user wants to join a session; first, we need to determine if they're on a mobile device
                            // then, if so, we auto-launch to the store/app
                            // if not, we use the extjs picker to let them select

                            var windowTitle = MedwebCloud.getMsg('medweb.join.window.title');
                            var launchWebLabel = MedwebCloud.getMsg('medweb.join.web.launch');
                            var launchDesktopLabel = MedwebCloud.getMsg('medweb.join.desktop.launch');

                            var w = Ext.create('Ext.window.Window', {
                                title: windowTitle,
                                closable: false,
                                resizable: false,
                                draggable: false,
                                layout: 'vbox',
                                border: false,
                                items: [{
                                    xtype: 'form',
                                    defaults: {
                                        margin: 10
                                    },
                                    items: [{
                                        xtype: 'panel',
                                        border: false,
                                        layout: 'fit',
                                        style: 'text-align:center',
                                        html: '<img src="/images/logo.png" width="235px" height="78px" border="0" alt="Login logo" />'
                                    }, {
                                        xtype: 'box',
                                        autoEl: { tag: 'a', href: webURL, html: launchDesktopLabel, cls: 'login-link', style: 'text-align:center;padding:5px' },
                                    }, {
                                        xtype: 'box',
                                        autoEl: { tag: 'a', href: desktopURL, html: launchWebLabel, cls: 'login-link', style: 'text-align:center;padding:5px' },
                                    }]
                                }]
                            }).show();

                            // make sure it stays centered
                            Ext.EventManager.onWindowResize(function () {
                                w.center();
                            });

                            MedwebCloud.ready = true;
                        });
                    }
                }
            }, MedwebCloud.bundle));
        }
    }
})(window, document, navigator);