﻿window.MedwebCloud = window.MedwebCloud || {};
// Setup the ExtJS Loader
Ext.Loader.setConfig({
    enabled: true,
    paths: {
        'Ext.ux': '/scripts/app/ux',
    }
});

var bundle = Ext.create('Ext.ux.i18n.Bundle', Ext.apply({
    autoLoad: true,
    listeners: {
        loaded: function () {
            //Apply the host info to bundle data before using it
            var format = function (str, col) {
                col = typeof col === 'object' ? col : Array.prototype.slice.call(arguments, 1);
                return str.replace(/\{\{|\}\}|\{([\w_]+)\}/g, function substitute(m, n) {
                    if (m == "{{") {
                        return "{";
                    } if (m == "}}") {
                        return "}";
                    }
                    return col[n] || '{' + n + '}';
                });
            };
            var hostSubstitution = {}
            Ext.Object.each(MedwebCloud.UIConfig.HostContext || {}, function (key, value, all) {
                this["Host_" + key] = value;
            }, hostSubstitution);

            bundle.each(function (record, idx, count) {
                var newVal = record.get('value');
                record.set('value', format(newVal, hostSubstitution));
            });
            MedwebCloud.getMsg = function () {
                var bdl = bundle;
                return bdl.getMsg.apply(bdl, arguments);
            };

            Ext.onReady(function () {
                Ext.define('MedwebCloud.loginInfo', {
                    extend: 'Ext.data.Model',
                    fields: [
                        { name: 'email' },
                        { name: 'password' }
                    ]
                });

                var w, performLogin;

                var initToken = Ext.History.getToken();

                performReset = function () {
                    var form = w.down('#loginForm');
                    var maskMessage = MedwebCloud.getMsg('medweb.reset.perform.mask.message');
                    var successMessage = MedwebCloud.getMsg('medweb.reset.success.message');
                    var successTitle = MedwebCloud.getMsg('medweb.reset.success.title');

                    if (!form.isValid()) {
                        return;
                    }

                    w.mask(maskMessage);
                    w.down('#login').disable();

                    var username = w.down('#username').getValue();

                    var subdomain = window.location.host.split('.')[0];
                    console.log("Using subdomain: " + subdomain);

                    Ext.Ajax.request({
                        url: MedwebCloud.api('/v1/' + (window.MWDecodedEmail ? 'change' : 'reset') + 'password'),
                        method: 'POST',
                        jsonData: {
                            Username: username,
                            Password: w.down('#password').getValue(),
                            ConfirmPassword: w.down('#confirmPassword').getValue(),
                            Code: window.MWCode,
                            Subdomain: subdomain
                        },
                        success: function (response, options) {
                            if (!window.MWDecodedEmail) {
                                Ext.Msg.alert(successTitle, successMessage);
                            } else {
                                window.location.href = '/Join.aspx';
                            }
                        },
                        failure: function (response, options) {
                            var responseObj = {};
                            try {    // This will fail if not valid json response
                                responseObj = Ext.decode(response.responseText);
                            } catch (e) { }
                            var error = responseObj || response.responseText,
                                el = w.down('#error');

                            el.update(error);
                            el.show();
                        },
                        callback: function () {
                            w.down('#login').enable();
                            w.unmask();
                        }
                    });
                };

                handleEnter = function (field, e) {
                    if (e.getKey() == e.ENTER) {
                        performReset();
                    }
                };

                var windowTitle = MedwebCloud.getMsg('medweb.reset.window.title');
                var labelDisplayName = MedwebCloud.getMsg('medweb.user.displayname.label');
                var labelEmail = MedwebCloud.getMsg('medweb.user.email.label');
                var labelPassword = MedwebCloud.getMsg('medweb.user.password.label');
                var labelPasswordConfirm = MedwebCloud.getMsg('medweb.user.password.confirm.label');

                var labelPositiveButton = MedwebCloud.getMsg(window.MWDecodedEmail ? 'medweb.reset.button.reset.label' : 'medweb.reset.button.save.label');
                var labelExistingAcct = MedwebCloud.getMsg('medweb.common.account.existing.question');

                w = Ext.create('Ext.window.Window', {
                    title: document.title,
                    closable: false,
                    resizable: false,
                    draggable: false,
                    layout: 'fit',
                    width: 350,
                    border: false,
                    items: [{
                        xtype: 'form',
                        itemId: 'loginForm',
                        defaults: {
                            margin: 10,
                            anchor: '100%'
                        },
                        fieldDefaults: {
                            labelWidth: 120
                        },
                        items: [{
                            xtype: 'panel',
                            border: false,
                            layout: 'fit',
                            style: 'text-align:center',
                            html: '<img src="/images/logo.png" width="235px" height="78px" border="0" alt="Login logo" />' + (window.MWError ? '<br/><br/>' + window.MWError : ''),
                        }, {
                            xtype: 'textfield',
                            itemId: 'username',
                            fieldLabel: 'Email',
                            vtype: 'email',
                            disabled: window.MWDecodedEmail ? true : false,
                            value: window.MWDecodedEmail,
                            allowBlank: false,
                            listeners: {
                                specialkey: handleEnter
                            }
                        }, {
                            xtype: 'textfield',
                            itemId: 'password',
                            inputType: 'password',
                            fieldLabel: 'Password',
                            hidden: window.MWDecodedEmail ? false : true,
                            allowBlank: window.MWDecodedEmail ? false : true,
                            listeners: {
                                specialkey: handleEnter
                            }
                        }, {
                            xtype: 'textfield',
                            itemId: 'confirmPassword',
                            inputType: 'password',
                            fieldLabel: 'Confirm Password',
                            hidden: window.MWDecodedEmail ? false : true,
                            allowBlank: window.MWDecodedEmail ? false : true,
                            listeners: {
                                specialkey: handleEnter
                            }
                        }, {
                            xtype: 'panel',
                            hidden: true,
                            itemId: 'error',
                            border: false,
                            bodyCls: 'error'
                        }],
                        // configure how to read the XML data
                        reader: Ext.create('Ext.data.reader.Json', {
                            model: 'MedwebCloud.loginInfo',
                            record: 'data',
                            successProperty: 'success'
                        })
                    }],
                    buttons: [{
                        xtype: 'box',
                        autoEl: { tag: 'a', href: '/Login.aspx', html: labelExistingAcct, cls: 'login-link' },
                        id: 'forgot-password'
                    }, '->', {
                        text: labelPositiveButton,
                        itemId: 'login',
                        handler: performReset
                    }]
                }).show();

                // make sure it stays centered
                Ext.EventManager.onWindowResize(function () {
                    w.center();
                    if (dlg.isVisible()) {
                        dlg.center();
                    }
                });
                MedwebCloud.ready = true;
            });
        }
    }
}, MedwebCloud.bundle));