﻿(function (window, document, navigator, undefined) {

    String.prototype.endsWith = function (suffix) {
        return this.indexOf(suffix, this.length - suffix.length) !== -1;
    };

    String.prototype.startsWith = function (prefix) {
        return this.indexOf(prefix) === 0;
    };

    window.MedwebCloud = window.MedwebCloud || {};
    var isIP = /^\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}$/gi
      , hostname = window.location.hostname,
      protocol = window.location.protocol;

    if (hostname.indexOf('solution.fm') > -1) {
        MedwebCloud.serverAddressIceLink = 'medweb-icelink.solution.fm';
        MedwebCloud.apiPath = 'http://medweb-api.solution.fm/rest.ashx';
        MedwebCloud.proxyPath = 'http://medweb-api.solution.fm/proxy.ashx';
        MedwebCloud.websyncPath = 'http://medweb-api.solution.fm/websync.ashx';
    } else if (isIP.test(hostname) || hostname === 'localhost' || hostname.indexOf('local') === 0) {
        // on localhost, we use cloud.medweb.net
        // and localhost:8080
        MedwebCloud.serverAddressIceLink = 'demo.icelink.fm';
        var protocol = window.location.protocol;
        var port = (window.location.protocol == 'https:') ? '8443' : '8080';
        MedwebCloud.apiPath = protocol + '//' + hostname + ':' + port + '/rest.ashx';
        MedwebCloud.proxyPath = protocol + '//' + hostname + ':' + port + '/proxy.ashx';
        MedwebCloud.websyncPath = protocol + '//' + hostname + ':' + port + '/websync.ashx';
        window.isDebug = true;
    } else if (window.MedwebCloud.UIConfig.ApiHost) {
        MedwebCloud.serverAddressIceLink = MedwebCloud.UIConfig.ApiHost.replace(/:\d+/, '');
        MedwebCloud.apiPath = protocol + '//' + MedwebCloud.UIConfig.ApiHost + '/rest.ashx';
        MedwebCloud.proxyPath = protocol + '//' + MedwebCloud.UIConfig.ApiHost + '/proxy.ashx';
        MedwebCloud.websyncPath = protocol + '//' + MedwebCloud.UIConfig.ApiHost + '/websync.ashx';
    } else {
        var cloud = /cloud\./i
          , apiHost = hostname.replace(cloud, 'api.');

        MedwebCloud.serverAddressIceLink = hostname.replace(cloud, 'icelink.');
        MedwebCloud.apiPath = protocol + '//' + apiHost + ':8080/rest.ashx';
        MedwebCloud.proxyPath = protocol + '//' + apiHost + ':8080/proxy.ashx';
        MedwebCloud.websyncPath = protocol + '//' + apiHost + ':8080/websync.ashx';
    }
    MedwebCloud.serverPortIceLink = 3478;
    MedwebCloud.api = function (path) {
        if (path.indexOf('/') !== 0) {
            path = '/' + path;
        }
        return MedwebCloud.apiPath + path;

    };
    MedwebCloud.proxy = function (url) {
        return MedwebCloud.proxyPath + '?url=' + encodeURIComponent(url) + '&_dt=' + (new Date().getTime() + '&disableSslValidation=true');
    };

    var browserLang = (navigator.languages ? navigator.languages[0] : (navigator.language || navigator.userLanguage));
    var userLang = MedwebCloud.UIConfig.UserCulture.Name;
    var browserLangIsSupported = false;
    for (var i = 0; i < MedwebCloud.UIConfig.SupportedCultures.length; i++) {
        if (MedwebCloud.UIConfig.SupportedCultures[i].Name == browserLang) {
            browserLangIsSupported = true;
            break;
        }
    }
    var selectedLang = '';

    if (browserLangIsSupported && !userLang) {
        selectedLang = browserLang;
    } else {
        selectedLang = userLang;
    }

    MedwebCloud.getURLParameter = function (name) {
        return decodeURIComponent((new RegExp('[?|&]' + name + '=' + '([^&;]+?)(&|#|;|$)').exec(location.search) || [, ""])[1].replace(/\+/g, '%20')) || null;
    };
    MedwebCloud.getHashParameter = function (name) {
        if (!this.hashParams) {
            var hashParams = {};
            var e,
                a = /\+/g,  // Regex for replacing addition symbol with a space
                r = /([^&;=]+)=?([^&;]*)/g,
                d = function (s) { return decodeURIComponent(s.replace(a, " ")); },
                q = window.location.hash.substring(1);

            while (e = r.exec(q))
                hashParams[d(e[1])] = d(e[2]);

            this.hashParams = hashParams;
        }
        return this.hashParams[name];
    };
    MedwebCloud.bundle = {
        bundle: 'Application',
        lang: selectedLang,
        path: '/scripts/lang',
        noCache: true
    };

    $.ajaxPrefilter(function (options, originalOptions, jqXHR) {
        options.crossDomain = {
            crossDomain: true
        };
        options.xhrFields = {
            withCredentials: true
        };
        var subdomain = window.location.host.split('.')[0];
        var headers = $.extend({}, originalOptions.headers || {});
        headers['X-Medweb-Company'] = subdomain;
        options.headers = headers;
    });

    if (!(window.Ext && window.Ext.Loader)) {//bail for non-Ext pages
        return;
    }
    // Setup the ExtJS Loader
    Ext.Loader.setConfig({
        enabled: false,
        paths: {
            'Ext.ux': '/scripts/app/ux',
            //'Ext.ux.upload': '/Scripts/ux/ex-upload-widget/lib/upload',
            'Ext': '/scripts/ext-4.2.2.1144/src',
            'MedwebCloud': '/scripts/app'
        }
    });

    MedwebCloud.loadModels = function (callback) {
        Ext.extend(Ext.data.Model, {
            destroy: function (args) {
                debugger;
            }
        });

        Ext.Ajax.request({
            url: MedwebCloud.apiPath + '/v1/meta/entities',
            withCredentials: true,
            method: 'GET',
            success: function (response, options) {
                var metaConfigs = MedwebCloud.meta || {};
                var responseObj = {};
                try {    // This will fail if not valid json response
                    responseObj = Ext.decode(response.responseText);
                } catch (e) { }
                
                // build up the models
                for (var i = 0; i < responseObj.Types.length; i++) {
                    var type = responseObj.Types[i];
                    var fields = [];
                    for (var j = 0; j < type.Properties.length; j++) {
                        var property = type.Properties[j];
                        if (property.IsComplexType) {
                            // TODO: could probably add relationships here...
                            continue;
                        }

                        // {Name: "Id", TypeName: "int", DisplayName: "Id", Nullable: false}
                        fields.push({
                            name: property.Name,
                            type: property.TypeName,
                            persist: !property.IsAuditColumn,
                            useNull: property.Nullable
                        });
                    }


                    if (!window.MedwebCloud.model[type.Name]) {
                        var modelName = 'MedwebCloud.model.' + type.Name;
                        var modelConfig =  {
                            extend: 'Ext.data.Model',
                            idProperty: 'Id',
                            fields: fields,
                            proxy: {
                                type: 'therest',
                                url: MedwebCloud.api('/v2/entities/' + type.PluralName)
                            }
                        }
                        if (modelName in metaConfigs) {
                            var meta = metaConfigs[modelName];
                            if(Ext.isFunction(meta)) {
                                modelConfig = meta.call(MedwebCloud, modelConfig, type);
                            } else if (meta.extend == 'Ext.data.Model') {
                                modelConfig = meta;
                            }
                        }

                        // create model
                        Ext.define(modelName, modelConfig);

                        (function (thisTypeName, thisModelName) {
                            MedwebCloud.model[thisTypeName].destroy = function (args) {
                                var model = Ext.create(thisModelName);
                                model.phantom = false;
                                model.destroy(args);
                            };
                        })(type.Name, modelName);
                    }

                    if (!window.MedwebCloud.store[type.PluralName]) {
                        // create store
                        Ext.define('MedwebCloud.store.' + type.PluralName, {
                            extend: 'MedwebCloud.store.BaseStore',
                            model: 'MedwebCloud.model.' + type.Name,
                            pageSize: 25,
                            filters: [{
                                property: 'IsDeleted',
                                value: false
                            }]
                        });
                    }
                }

                MedwebCloud.enums = MedwebCloud.enums || {};

                // build up the enumerations
                for (var i = 0; i < responseObj.Enums.length; i++) {
                    var en = responseObj.Enums[i];
                    // default object
                    MedwebCloud.enums[en.TypeName] = {};

                    // accessor functions to help w/ display
                    (function (selectedEnum) {
                        MedwebCloud.enums[selectedEnum.TypeName].getDisplayName = function (id) {
                            for (var k = 0; k < selectedEnum.Items.length; k++) {
                                var item = selectedEnum.Items[k];
                                if (item.Value == id) {
                                    return item.DisplayName;
                                }
                            }
                            return '';
                        };
                    })(en);

                    // actual enum values
                    for (var j = 0; j < en.Items.length; j++) {
                        var item = en.Items[j];
                        MedwebCloud.enums[en.TypeName][item.CodeName] = item.Value;
                    }
                }

                if (callback) {
                    callback();
                }
            },
            failure: function () {
                console.error(arguments);
                if (arguments[0].status === 0) {
                    console.log("Request appears to have been cancelled.  Retrying...");
                //    MedwebCloud.loadModels(callback);
                }
            }

        });
    }


    var templateArgs = /\{([0-9a-zA-Z]+)\}/g
    var slice = Array.prototype.slice
    MedwebCloud.renderTemplate = function (tpl, data) {
        return tpl.replace(templateArgs, function (match, i, index) {
            var result;

            if (tpl[index - 1] === '{' &&
                tpl[index+match.length] === '}') {
                return i;
            } else {
                result = data.hasOwnProperty(i) ? data[i] : null;
                if (result === null || result === undefined) {
                    result = match;
                }
                return result;
            }
        });
    };
    MedwebCloud.unCamelCase = function (str) {
        if (str == null) {
            return str;
        } else {
            str = str.toString();
        }

        return str
            // Underscores to space
            .replace(/_+/g, ' ')
            // insert a space between lower & upper
            .replace(/([a-z])([A-Z])/g, '$1 $2')
            // space before last upper in a sequence followed by lower
            .replace(/\b([A-Z]+)([A-Z])([a-z])/, '$1 $2$3')
            // uppercase the first character
            .replace(/^./, function (str) { return str.toUpperCase(); })
    };


})(window, document, navigator);