(function (window, Ext, MedwebCloud) {
    //private functions
    function makeMsgFrag(str) {
        if (str == null) {
            return '_default';
        } else {
            return str.toString().toLowerCase().replace(/\W+/gi, '_').replace(/(^_+)|(_+$)/gi, '');
        }
    }
    function getLblMsg(key) {
        var lookup = 'medweb.view.waitingroom.chatrequest.details.' + key + '.label';
        var msg = MedwebCloud.getMsg(lookup);
        if (msg == lookup + '.undefined') {
            return MedwebCloud.unCamelCase(key);
        } else {
            return msg;
        }
    }
    function getValMsg(key, value) {
        var lookup = 'medweb.view.waitingroom.chatrequest.details.' + key + '.value.' + makeMsgFrag(value);
        var msg = MedwebCloud.getMsg(lookup);
        if (msg == lookup + '.undefined') {
            return value;
        } else {
            return msg;
        }
    }
    function getHtmlMsg(key, args) {
        var lookup = 'medweb.view.waitingroom.chatrequest.details.' + key + '.html';
        args.unshift(lookup);
        var msg = MedwebCloud.getMsg.apply(MedwebCloud, args);
        if (msg == lookup + '.undefined') {
            return '';
        } else {
            return msg;
        }
    }
    function generateHtml(key, value) {
        var val = getValMsg(key, value);
        if (val == null || val.length == 0) {
            return '';
        } else {
            var lbl = getLblMsg(key);
            var valHtml = val.toString().replace(/\r?\n/gi, '<br />');
            var htm = getHtmlMsg(key, [lbl, valHtml]);
            return htm;
        }
    }
    function getTransferMsg(key, args) {
        var lookup = 'medweb.view.waitingroom.chatrequest.details.' + key + '.transfer';
        args.unshift(lookup);
        var msg = MedwebCloud.getMsg.apply(MedwebCloud, args);
        if (msg == lookup + '.undefined') {
            return '';
        } else {
            return msg;
        }
    }
    function generateTransfer(key, value, section) {
        var val = getValMsg(key, value);
        if (val == null || val.length == 0) {
            return '';
        } else {
            var lbl = Ext.String.htmlEncode(getLblMsg(key));
            var valHtml = Ext.String.htmlEncode(val);
            section = Ext.String.htmlEncode(section);
            var htm = getTransferMsg(key, [lbl, valHtml, section]);
            return htm;
        }
    }
    Ext.define('MedwebCloud.model.ChatRequest', {
        extend: 'Ext.data.Model',
        idProperty: 'id',
        fields: [
            { name: 'Id', type: 'int' },
            { name: 'Name', type: 'string' },
            { name: 'FirstName', type: 'string' },
            { name: 'LastName', type: 'string' },
            { name: 'DisplayName', type: 'string' },
            { name: 'Email', type: 'string' },
            { name: 'Question', type: 'string' },
            { name: 'IpAddress', type: 'string' },
            { name: 'IsVideoSupported', type: 'boolean' },
            { name: 'IsSMS', type: 'boolean' },
            { name: 'IsHandled', type: 'boolean' },
            { name: 'ChatId', type: 'auto' },
            { name: 'Channel', type: 'string' },
            { name: 'Identifier', type: 'string' },
            { name: 'CreatedBy', type: 'string', persist: false },
            { name: 'CreatedOn', type: 'date', persist: false, useNull: true },
            { name: 'ModifiedBy', type: 'string', persist: false },
            { name: 'ModifiedOn', type: 'date', persist: false, useNull: true },
            { name: 'IsDeleted', type: 'boolean' },
            { name: 'ExtraJson', type: 'string' }
        ],
        hasOne: [
            { name: 'Chat', model: 'MedwebCloud.model.Chat', associationKey: 'Chat', getterName: 'getChat', setterName: 'setChat', primaryKey: 'chatId', foreignKey: 'chatRequestId' }
        ],
        getExtra: function () {
            var extraJson = this.get('ExtraJson');
            if (extraJson && extraJson.length > 0) {
                return JSON.parse(extraJson);
            }
            return {};
        },
        getSubstitution: function (urlEncode) {
            urlEncode = !!urlEncode;
            var data = {};
            var extra = this.getExtra();
            var encode = urlEncode ? encodeURIComponent : function ident(a) { return a };

            Ext.Object.each(this.getData(false), function (key, value, self) {
                this[key] = encode(value);
            }, data);
            Ext.Object.each(extra, function (key, value, self) {
                this['Extra_' + key] = encode(value);
            }, data);

            return data;
        },

        getHTML: function () {
            var chatRequest = this.raw;
            var chatRequestUser = chatRequest.User;
            var sect, fmt, val;
            var chatRequestDataHtml = '';

            if (!MedwebCloud.PatientData) {
                MedwebCloud.PatientData = [];
            }
            MedwebCloud.PatientData[chatRequest.Email] = { 'Email': chatRequest.Email, 'DisplayName': chatRequest.DisplayName };

            chatRequestDataHtml += '<div class="chatRequestDetails">';
            chatRequestDataHtml += generateHtml('workflow', chatRequest.Workflow);
            chatRequestDataHtml += generateHtml('acceptterms', chatRequest.AcceptTerms);
            chatRequestDataHtml += '</div>';
            if (chatRequestUser) {
                sect = MedwebCloud.getMsg('medweb.view.waitingroom.chatrequest.details.section.collageuser');
                val = chatRequestUser.DisplayName + ' (' + chatRequestUser.Username + ')';
                chatRequestDataHtml += '<div class="chatRequestDetails">';
                chatRequestDataHtml += '<h3 class="chatRequestDetailTitle">' + sect + '</h3>';
                chatRequestDataHtml += generateHtml('user', val);
                chatRequestDataHtml += '</div>';
            }
            sect = MedwebCloud.getMsg('medweb.view.waitingroom.chatrequest.details.section.request');

            chatRequestDataHtml += '<div class="chatRequestDetails"><h3>' + sect + '</h3>';
            chatRequestDataHtml += generateHtml('displayname', chatRequest.DisplayName);

            var idType = getValMsg('identifiertype', chatRequest.IdentifierType || 'Id');
            var idVal = getValMsg('email', chatRequest.Identifier);
            var idHTML = getHtmlMsg('identifiertype', [idType, idVal]);
            chatRequestDataHtml += idHTML;

            var extra = this.getExtra();
            Ext.Object.each(extra, function (key, value, theObject) {
                var ekey = 'extra.' + makeMsgFrag(key);
                chatRequestDataHtml += generateHtml(ekey, value);
            });
            chatRequestDataHtml += generateHtml('question', chatRequest.Question);
            chatRequestDataHtml += '</div>';
            return chatRequestDataHtml;
        },
        getTransfer: function () {
            var chatRequest = this.raw;
            var chatRequestUser = chatRequest.User;
            var sect, fmt, val;
            var chatRequestDataTransfer = [];

            chatRequestDataTransfer.push(generateTransfer('workflow', chatRequest.Workflow, ''));
            chatRequestDataTransfer.push(generateTransfer('acceptterms', chatRequest.AcceptTerms, ''));

            sect = MedwebCloud.getMsg('medweb.view.waitingroom.chatrequest.details.section.request');
            chatRequestDataTransfer.push(generateTransfer('displayname', chatRequest.DisplayName, sect));

            var idType = getValMsg('identifiertype', chatRequest.IdentifierType || 'Id');
            var idVal = getValMsg('email', chatRequest.Identifier);
            chatRequestDataTransfer.push(getTransferMsg('identifiertype', [Ext.String.htmlEncode(idType), Ext.String.htmlEncode(idVal), Ext.String.htmlEncode(sect)]));

            var extra = this.getExtra();
            Ext.Object.each(extra, function (key, value, theObject) {
                var ekey = 'extra.' + makeMsgFrag(key);
                chatRequestDataTransfer.push(generateTransfer(ekey, value, sect));
            });
            chatRequestDataTransfer.push(generateTransfer('question', chatRequest.Question, sect));
            return chatRequestDataTransfer;
        }
    });
}(window, window.Ext, window.MedwebCloud));