Ext.define('MedwebCloud.v1.model.User', {
    extend: 'Ext.data.Model',
    idProperty: 'Id',
    fields: [
        { name: 'Id', type: 'int' },
        { name: 'CompanyId', type: 'int' },
        { name: 'Username', type: 'string' },
        { name: 'TrackLocation', type: 'boolean' },
        { name: 'DisplayName', type: 'string' },
        { name: 'PasswordHash', type: 'string' },
        { name: 'CreatedBy', type: 'string', persist: false },
        { name: 'CreatedOn', type: 'date', persist: false, useNull: true },
        { name: 'ModifiedBy', type: 'string', persist: false },
        { name: 'ModifiedOn', type: 'date', persist: false, useNull: true },
        { name: 'Presence', persist: false, type: 'int' },
        { name: 'Language', type: 'string' },
        { name: 'PreferredTimezone', type: 'string', defaultValue: "GMT Standard Time" },
        {
            name: 'Department', persist: false, type: 'string',
            convert: function (v, r) {
                if (r.raw && r.raw.Department) {
                    return r.raw.Department.Name;
                }
                return '';
            }
        },
        {
            name: 'LastMessageText', persist: false, type: 'string',
            convert: function (v, r) {
                if (r.raw && r.raw.LastMessage) {
                    return r.raw.LastMessage.Text;
                }
                return '';
            }
        },
        {
            name: 'PresenceSort', persist: false, type: 'int',
            convert: function (v, r) {
                switch (r.get('Presence')) {
                    case MedwebCloud.Core.Enums.Presence.Active:
                        return 1;
                    case MedwebCloud.Core.Enums.Presence.Idle:
                        return 2;
                    case MedwebCloud.Core.Enums.Presence.Busy:
                        return 3;
                    default:
                        return 4;
                }
            }
        },
        {
            name: 'GravatarImage',
            persist: false,
            convert: function (v, r) {
                return '<img class="gravatar" src="' + MedwebCloud.getGravatarUrl(r.get('Username')) + '" />';
            }
        },
        { name: 'IsDeleted', type: 'boolean' }

    ],
    associations: [
        { type: 'hasMany', model: 'MedwebCloud.model.Device', name: 'Devices' }
    ],
    set: function (fieldName, value) {
        this.callParent(arguments);
        // make sure they update together
        if (fieldName === 'Presence') {
            this.set('PresenceSort');
        }
    },
    proxy: {
        type: 'therest',
        url: MedwebCloud.api('/v1/users')
    }
});