﻿Ext.define('MedwebCloud.view.BuddyList', {
    extend: 'Ext.panel.Panel',
    xtype: 'MedwebCloud.buddylist',
    requires: ['MedwebCloud.view.BuddyGrid', 'MedwebCloud.view.BuddyMap'],

    usersStore: null,
    client: null,

    initComponent: function () {
        var me = this;
        this.chatWindows = new Ext.util.MixedCollection();

        this.notification = soundManager.createSound({
            url: '/sounds/notification.mp3'
        });

        this.ringtone = soundManager.createSound({
            url: '/sounds/ringtone.wav'
        });

        Ext.eventBus.on('user:join', function (user) {
            if (isNaN(user)) {
                user = user.getUserId();
            } else {
                user = parseInt(user);
            }
            console.log('join');
            me.setUserPresence(user, MedwebCloud.Core.Enums.Presence.Active);
        });
        Ext.eventBus.on('user:leave', function (user) {
            if (isNaN(user)) {
                user = user.getUserId();
            } else {
                user = parseInt(user);
            }

            me.setUserPresence(user, MedwebCloud.Core.Enums.Presence.Idle);
        });
        Ext.eventBus.on('user:busy', function (user) {
            if (isNaN(user)) {
                user = user.getUserId();
            } else {
                user = parseInt(user);
            }

            me.setUserPresence(user, MedwebCloud.Core.Enums.Presence.Busy);
        });
        Ext.eventBus.on('user:offline', function (user) {
            if (isNaN(user)) {
                user = user.getUserId();
            } else {
                user = parseInt(user);
            }

            me.setUserPresence(user, MedwebCloud.Core.Enums.Presence.Offline);
        });

        Ext.eventBus.on('internalchatmessage', function (e) {
            me.receiveChatMessage(e.getData().From, e.getData().InternalChat, e.getData().InternalChatMessage);
            me.notification.play();
        });
        Ext.eventBus.on('internalchatfile', function (e) {
            me.receiveChatFile(e.getData().From, e.getData().InternalChat, e.getData().InternalChatFile);
        });
        Ext.eventBus.on('internalchatvideo', function (e) {
            //            me.receiveChatVideo(e.getData().From, e.getData().InternalChat);
            me.receiveChatVideo(e.getData().From, e.getData());
        });

        me.usersStore = Ext.create('MedwebCloud.store.Buddies');

        me.usersStore.remoteSort = false;
        me.usersStore.remoteFilter = false;

        me.usersStore.sort('PresenceSort', 'ASC');

        me.usersStore.filter([
            { filterFn: function (item) { return item.get('Presence') != MedwebCloud.Core.Enums.Presence.Offline } },
        ]);
        me.isFiltering = true;

        var btnAdd = MedwebCloud.getMsg('medweb.view.buddylist.toolbar.add.title');
        var tabList = MedwebCloud.getMsg('medweb.view.buddylist.tab.list.title');
        var tabMap = MedwebCloud.getMsg('medweb.view.buddylist.tab.map.title');

        Ext.apply(me, {
            //autoHeight: true,
            //sortableColumns: false,
            //store: me.usersStore,
            layout: 'fit',
            tbar: {
                items: [{
                    icon: '/images/add.png',
                    text: btnAdd,
                    handler: function () {
                        MedwebCloud.createContact(function (values) {
                            var invitation = Ext.create('MedwebCloud.model.InternalChatInvitationModel', {
                                Email: values.Email,
                                DisplayName: values.DisplayName
                            });
                            invitation.save({
                                failure: function () {
                                    var ttl = MedwebCloud.getMsg('medweb.view.buddylist.action.add.failure.title');
                                    var msg = MedwebCloud.getMsg('medweb.view.buddylist.action.add.failure.message');
                                    Ext.Msg.alert(ttl, msg);
                                },
                                success: function (s) {
                                    var msg = MedwebCloud.getMsg('medweb.view.buddylist.action.add.success.message');
                                    Ext.Msg.toast({ body: msg });
                                }
                            });
                            me.fireEvent('invite');
                        });
                    }
                }, '->', {
                    text: 'Show All Users',
                    handler: function () {
                        //var col = me.down('grid').columnManager.getColumns()[0];
                        //console.log(col);

                        if (me.isFiltering) {
                            console.log("Clear filter");
                            me.usersStore.clearFilter(false);
                            me.usersStore.load();
                            me.isFiltering = false;
                            this.setText('Show Online Users');
                        } else {
                            console.log("Do filter")
                            me.usersStore.filter([
                                { filterFn: function (item) { return item.get('Presence') != MedwebCloud.Core.Enums.Presence.Offline } },
                            ]);
                            me.usersStore.load();
                            me.isFiltering = true;
                            this.setText('Show All Users');
                        }
                    }
                }]
            },
            items: [{
                defaults: {
                    border: false,
                    client: me.client,
                    store: me.usersStore
                },
                //layout: 'fit',
                xtype: 'tabpanel',
                border: false,
                deferredRender: false,
                items: [{
                    title: tabList,
                    xtype: 'MedwebCloud.buddygrid',
                    store: me.usersStore,
                    listeners: {
                        itemclick: function (grid, record, el, index, e) {
                            me.openChatWindow(record.data);
                        }
                    }
                }, {
                    title: tabMap,
                    user: {},
                    store: me.usersStore,
                    xtype: 'MedwebCloud.buddymap',
                    listeners: {
                        itemclick: function (user, device, location) {
                            me.openChatWindow(user.data);
                        }
                    }
                }]
            }],
            listeners: {
                collapse: function () {
                    me.arrangeChatWindows();
                },
                expand: function () {
                    me.arrangeChatWindows();
                }
            }
        });

        me.callParent(arguments);

        Ext.eventBus.on('window:resize', function () {
            me.arrangeChatWindows();
        });

        me.loadStore();
    },

    loadStore: function () {
        this.usersStore.load();
    },
    setUserPresence: function (userId, presence) {
        var me = this;
        var userIndex = me.usersStore.findBy(function (r) { return r.get('Id') == userId; });

        if (userIndex >= 0) {
            console.log(userIndex);
            var record = me.usersStore.getAt(userIndex);
            console.log(record);
            record.set('Presence', presence);
            record.commit();

            var w = this.chatWindows.getByKey(userId);
            if (w) {
                w.setPresence(presence);
            }
        }
    },
    getChatWindow: function (user, internalChat) {
        var grid = this;
        var w = this.chatWindows.getByKey(user.Id);
        if (w) {
            return w;
        }
        w = new MedwebCloud.view.InternalChatWindow({
            user: user,
            internalChatId: internalChat ? internalChat.Id : 0,
            client: this.client,
            listeners: {
                show: function () {
                    grid.arrangeChatWindows();
                },
                close: function () {
                    grid.chatWindows.remove(w);
                    grid.arrangeChatWindows();
                }
            }
        });
        this.chatWindows.add(user.Id, w);

        return w;
    },
    receiveChatFile: function (user, internalChat, internalFile) {
        var w = this.getChatWindow(user);
        w.show();

        w.appendFile(internalFile);

        this.arrangeChatWindows();
    },
    receiveChatMessage: function (user, internalChat, internalMessage) {
        var w = this.getChatWindow(user);
        w.show();

        w.appendMessage(internalMessage);

        this.arrangeChatWindows();
    },
    receiveChatVideo: function (user, data, onAccept, onReject) {
        var chat = data.InternalChat;
        if (data.InternalChatVideo != null) {
            /*
            TODO:
            keep track of the chat(s) for which we are showing confirmation dialogs.
            If data.InternalChatVideo is not null, then we need to cancel the dialog if:
            1)  my user is the value passed as the user. This always means I've accepted the
                chat on some other device.
            2)  The data.InternalChatVideo.Action is 'cancel' or 'end'. These occur if the person
                who send the original request hangs up. In that case we can hang up too.
            */
            return;
        }
        var me = this;

        var w = me.getChatWindow(user);
        var vp = w.down('#internalChatVideoPanel');

        if (w.hasVideoChat()) {
            w.chatVideoAccepted();
            if (onAccept) {
                onAccept();
            }
            return;
        }
        var title = MedwebCloud.getMsg('medweb.view.buddylist.action.receiveVideo.confirm.title');
        var msg = MedwebCloud.getMsg('medweb.view.buddylist.action.receiveVideo.confirm.Message', user.DisplayName);

        this.ringtone.play({
            loops: 3
        });

        Ext.Msg.confirm(title, msg, function (btn) {
            me.ringtone.stop();
            if (btn == 'yes') {
                w.show();
                w.joinAndStartVideoChat();
                if (onAccept) {
                    onAccept();
                }
            } else {
                if (onReject) {
                    vp.signalConferenceState("end");
                    onReject();
                }
            }
        });

        window.setTimeout(function () {
            if (onReject) {
                vp.signalConferenceState("end");
                onReject();
            }
        }, 1000 * 15); // 15 seconds or give up
    },
    openChatWindow: function (user) {
        var w = this.getChatWindow(user);
        w.show();

        this.arrangeChatWindows();
    },

    arrangeChatWindows: function () {
        var prev = this.collapsed ? this.placeholder : this;
        this.chatWindows.each(function (w, index) {
            if (!w.hidden) {
                w.alignTo(prev, 'br-bl');
                prev = w;
            }
        });
    }
});