﻿Ext.define('MedwebCloud.view.CompanyForm2', {
    xtype: 'medwebcloud.companyform2',
    extend: 'Ext.form.Panel',
    initComponent: function () {
        var utils = MedwebCloud.util.ViewUtils;
        var me = this;
        me.recordId = me.record.get("Id");

        me.hosts = Ext.create('MedwebCloud.store.Hosts');
        me.hosts.load();
        
        var sections = utils.getModuleStore();
        function moduleFilter(valid) {
            return {
                filterFn: function (item) {

                    return valid.indexOf(item.get("value")) > -1;
                }
            };
        }

        sections.addFilter(moduleFilter(utils.getAllModules()));

        var hostCombo = Ext.create('Ext.form.ComboBox', me.getField('HostId', Ext.apply(utils.getFormDefaults(), {
            xtype: 'combo',
            queryMode: 'remote',
            store: me.hosts,
            displayField: 'Name',
            valueField: 'Id',
            autoSelect: true,
            forceSelection: true,
            editable: false,
            allowBlank: true,
            listeners: {
                beforequery: function (qe) {
                    qe.combo.lastQuery = null;
                },
                change: function (combo, newVal, oldVal, eOpts) {
                    if (!!newVal) {
                        var host = me.hosts.getById(newVal);
                        var availableModules = host.get("ModuleData");
                        sections.clearFilter();
                        sections.filter(moduleFilter(availableModules));
                        
                        var incCollage = availableModules.indexOf('collage') > -1;
                        var incEVisit = availableModules.indexOf("evisit") > -1;
                        
                        var defSection = me.getComponent('DefaultSection');
                        if (availableModules.indexOf(defSection.getValue()) < 0) {
                            defSection.setValue('');
                        }

                        var defCollage = me.getComponent("DefaultCollageSection");
                        defCollage.setDisabled(!incCollage);
                        defCollage.setVisible(incCollage);

                        var wait = me.getComponent("WaitingRoom");
                        wait.setDisabled(!incEVisit);
                        wait.setVisible(incEVisit);
                        
                        var tferMod = wait.getComponent("TransferMode");
                        tferMod.setDisabled(!incCollage || !incEVisit);
                        tferMod.setVisible(incCollage && incEVisit);
                    }
                }
            }
        })));






        var sectionData = [];
        for (var i = 0; i < MedwebCloud.UIConfig.CollageSections.length; i++) {
            var s = MedwebCloud.UIConfig.CollageSections[i];
            sectionData.push(s);
        }
        var collageSections = Ext.create('Ext.data.Store', {
            fields: ['Name', 'DisplayName'],
            proxy: {
                type: 'memory',
                reader: {
                    type: 'json'
                }
            },
            data: sectionData
        });

        var transferModesData = [];
        var m = MedwebCloud.UIConfig.TransferModes.sort();
        for (var i = 0; i < m.length; i++) {
            var m = MedwebCloud.UIConfig.TransferModes[i];
            transferModesData.push({ Name: m, DisplayName: m.charAt(0).toUpperCase() + m.substring(1) });
        }
        var transferModes = Ext.create('Ext.data.Store', {
            fields: ['Name', 'DisplayName'],
            proxy: {
                type: 'memory',
                reader: {
                    type: 'json'
                }
            },
            data: transferModesData
        });

        me.departments = Ext.create('MedwebCloud.store.Departments');
        me.departments.load();
        
        Ext.apply(me, {
            layout: utils.getFormLayout(),
            defaults: utils.getFormDefaults(),
            items: [{
                fieldLabel: 'Name',
                name: 'Name',
                xtype: 'textfield',
                allowBlank: false

            }, {
                fieldLabel: 'Code Name',
                name: 'CodeName',
                itemId: 'CodeName',
                xtype: 'textfield',
                disabled: this.recordId > 0 ? true : false,
                allowBlank: false
            }, {
                fieldLabel: 'Subdomain',
                name: 'Subdomain',
                itemId: 'Subdomain',
                xtype: 'textfield',
                allowBlank: false,
            }, {
                fieldLabel: 'Twilio Phone Number',
                name: 'TwilioPhoneNumber',
                itemId: 'TwilioPhoneNumber',
                xtype: 'textfield',
            },
            hostCombo,
            {
                fieldLabel: 'Default Section',
                name: 'DefaultSection',
                itemId: 'DefaultSection',
                xtype: 'combo',
                queryMode: 'local',
                store: sections,
                displayField: 'name',
                valueField: 'value',
                autoSelect: true,
                forceSelection: true,
                editable: false,
                allowBlank: true
            }, {
                fieldLabel: 'Default Collage Section',
                name: 'DefaultCollageSection',
                itemId: 'DefaultCollageSection',
                xtype: 'combo',
                queryMode: 'local',
                store: collageSections,
                displayField: 'DisplayName',
                valueField: 'Name',
                autoSelect: true,
                forceSelection: true,
                editable: false,
                allowBlank: true
            }, {
                xtype: 'fieldset',
                itemId: 'WaitingRoom',
                title: 'Waiting Room',
                collapsible: true,
                defaults: {
                    anchor: '100%',
                    labelWidth: 150
                },
                items : [{
                    fieldLabel: 'Logo URL',
                    name: 'LogoUrl',
                    xtype: 'textfield',
                    allowBlank: true,
                    editable: true,
                }, {
                    fieldLabel: 'Custom HTML',
                    name: 'ChatWaitSection',
                    itemId: 'ChatWaitSection',
                    xtype: 'htmleditor',
                    editable: true,
                    allowBlank: true
                }, {
                    fieldLabel: 'Allow Video Start',
                    name: 'ChatRemoteCanStartVideo',
                    itemId: 'ChatRemoteCanStartVideo',
                    xtype: 'checkbox',
                    editable: true,
                    allowBlank: true
                }, {
                    fieldLabel: 'Transfer Mode',
                    name: 'TransferMode',
                    itemId: 'TransferMode',
                    xtype: 'combo',
                    queryMode: 'local',
                    store: transferModes,
                    displayField: 'DisplayName',
                    valueField: 'Name',
                    autoSelect: true,
                    forceSelection: true,
                    editable: false,
                    allowBlank: true
                }],
            }, {
                xtype: 'tabpanel',
                flex: 1, // full-height
                items: [ {
                    title: 'Users',
                    itemName: 'User',
                    itemsName: 'Users',
                    restoreEnabled:true,
                    storeType: 'MedwebCloud.store.Users',
                    xtype: 'medwebcloud.windoweditor',
                    formType: 'medwebcloud.userform2',
                    expands: ['Department'],
                    storeType: 'MedwebCloud.store.Users',
                    layout: 'fit',
                    
                    filters: [{
                        property: 'CompanyId',
                        operator: '=',
                        value: me.record.get("Id")
                    }],

                    columns: [{
                        dataIndex: 'DisplayName',
                        text: 'Name',
                        flex: 2,
                        //header: '<div>Name<span class="searchIcon header-icon" style="float: right; width: 16px; height: 16px; cursor: pointer;"></span></div>',
                        filter: [{
                            type: 'text',
                            exactMatch: false,
                        }]
                    }, {
                        dataIndex: 'Username',
                        text: 'Email',
                        flex: 2,
                        filter: [{
                            type: 'text',
                            exactMatch: false,
                        }],
                    }, {
                        dataIndex: 'IsDoctor',
                        text: 'Is Doctor',
                        flex: 0,
                        renderer: function (isDoc) {
                            var src = isDoc ? "/images/tick.png" : "data:image/gif;base64,R0lGODlhAQABAAAAACwAAAAAAQABAAA=",
                                tag = '<img style="display:block;height:16px;width:16px;margin:0px auto" src="' + src +'" />';
                            return tag;
                        },
                        filter: [{
                            type: 'dropdown',
                            values: { 'Yes': true, 'No': false },
                            defaultValue: '',
                        }],
                    }, {
                        dataIndex: 'DepartmentId',
                        text: 'Department',
                        flex: 1,
                        renderer: function (dept, meta, record) {
                            dept = dept || 0;
                            try {
                                var dpt = me.departments.getById(dept);
                                return dpt.get("Name");
                            }
                            catch (ex) {
                                console.log(ex);
                                return "";
                            }
                        },
                        filter: [{
                            type: 'dropdown',
                            values: me.departments,
                            valueField: 'Id',
                            displayField: 'Name',
                        }]
                    }],

                    beforeAdd: function (item) {
                        item.set("CompanyId", me.record.get("Id"));
                    },
                },  {
                    title: 'Affiliates',
                    itemName: 'Affiliate',
                    xtype: 'medwebcloud.windoweditor',
                    formType: 'medwebcloud.affiliationform2',
                    storeType: 'MedwebCloud.store.Affiliations',
                    layout: 'fit',
                    editEnabled: false,

                    filters: [{
                        property: 'CompanyId',
                        operator: '=',
                        value: me.record.get("Id")
                    }],

                    columns: [{
                        dataIndex: 'UserDisplayName',
                        text: 'User',
                        flex: 3,
                        filter: [{
                            type: 'text',
                            exactMatch: false,
                        }],
                    }, {
                        dataIndex: 'UserName',
                        text: 'Username',
                        flex: 3,
                        filter: [{
                            type: 'text',
                            exactMatch: false,
                        }],
                    }, {
                        dataIndex: 'IsDoctor',
                        text: 'Is Doctor',
                        flex: 0,
                        renderer: function (isDoc, meta, record) {
                            //console.log(record, record.raw);
                            var src = isDoc ? "/images/tick.png" : "data:image/gif;base64,R0lGODlhAQABAAAAACwAAAAAAQABAAA=",
                                tag = '<img style="display:block;height:16px;width:16px;margin:0px auto" src="' + src + '" />';
                            return tag;
                        },
                        filter: [{
                            type: 'dropdown',
                            values: { 'Yes': true, 'No': false },
                            defaultValue: '',
                        }],
                    }, {
                        dataIndex: 'DepartmentName',
                        text: 'Department',
                        flex: 3,
                    }, {
                        dataIndex: 'UserCompanyName',
                        text: 'From',
                        flex: 1,
                    }],
                    filterer: {
                        ftype: 'filters',
                        encode: false, // json encode the filter query
                        local: true,   // defaults to false (remote filtering)

                        filters: [{
                            dataIndex: 'UserDisplayName',
                            type: 'string',
                        }, {
                            dataIndex: 'UserName',
                            type: 'string',
                        }, {
                            dataIndex: 'IsDoctor',
                            type: 'boolean',
                        }]
                    },

                    beforeAdd: function (item) {
                        item.set("CompanyId", me.record.get("Id"));
                    },
                }, {
                    title: 'eVisit Links',
                    itemName: 'Link',
                    xtype: 'medwebcloud.windoweditor',
                    formType: 'medwebcloud.evisitlinkform2',
                    storeType: 'MedwebCloud.store.EVisitLinks',
                    layout: 'fit',

                    filters: [{
                        property: 'CompanyId',
                        operator: '=',
                        value: me.record.get("Id")
                    }],

                    columns: [{
                        dataIndex: 'Name',
                        text: 'Name',
                        flex: 1,
                        filter: [{
                            type: 'text',
                            exactMatch: false,
                        }],
                    }, {
                        dataIndex: 'Url',
                        text: 'Url',
                        flex: 2,
                        filter: [{
                            type: 'text',
                            exactMatch: false,
                        }],
                    }],
                    filterer: {
                        ftype: 'filters',
                        encode: false, // json encode the filter query
                        local: true,   // defaults to false (remote filtering)

                        filters: [{
                            dataIndex: 'Name',
                            type: 'string',
                        }, {
                            dataIndex: 'Url',
                            type: 'string',
                        }]
                    },

                    beforeAdd: function (item) {
                        item.set("CompanyId", me.record.get("Id"));
                    },
                }, {
                    title: 'Email Domains',
                    itemName: 'Domain',
                    xtype: 'medwebcloud.windoweditor',
                    formType: 'medwebcloud.companydomainform2',
                    storeType: 'MedwebCloud.store.CompanyDomains',
                    layout: 'fit',

                    filters: [{
                        property: 'CompanyId',
                        operator: '=',
                        value: me.record.get("Id")
                    }],

                    columns: [{
                        dataIndex: 'Name',
                        text: 'Name',
                        flex: 1,
                        filter: [{
                            type: 'text',
                            exactMatch: false,
                        }],
                    }],
                    filterer: {
                        ftype: 'filters',
                        encode: false, // json encode the filter query
                        local: true,   // defaults to false (remote filtering)

                        filters: [{
                            dataIndex: 'Name',
                            type: 'string',
                        }]
                    },

                    beforeAdd: function (item) {
                        item.set("CompanyId", me.record.get("Id"));
                    },
                }, {
                    title: 'Cameras',
                    itemName: 'Camera',
                    xtype: 'medwebcloud.windoweditor',
                    formType: 'medwebcloud.remotecameraform2',
                    storeType: 'MedwebCloud.store.RemoteCameras',
                    layout: 'fit',

                    filters: [{
                        property: 'CompanyId',
                        operator: '=',
                        value: me.record.get("Id")
                    }],

                    columns: [{
                        dataIndex: 'Name',
                        text: 'Name',
                        flex: 2,
                        filter: [{
                            type: 'text',
                            exactMatch: false,
                        }],
                    }, {
                        dataIndex: 'Url',
                        text: 'URL',
                        flex: 2,
                        filter: [{
                            type: 'text',
                            exactMatch: false,
                        }],
                    }],
                    filterer: {
                        ftype: 'filters',
                        encode: false, // json encode the filter query
                        local: true,   // defaults to false (remote filtering)

                        filters: [{
                            dataIndex: 'Name',
                            type: 'string',
                        }, {
                            dataIndex: 'Url',
                            type: 'string',
                        }]
                    },

                    beforeAdd: function (item) {
                        item.set("CompanyId", me.record.get("Id"));
                    },
                }, {
                    title: 'Glasses',
                    itemName: 'Glasses',
                    xtype: 'medwebcloud.windoweditor',
                    addEnabled: MedwebCloud.userProfile.IsSuperAdmin,
                    removeEnabled: MedwebCloud.userProfile.IsSuperAdmin,
                    editForm: 'medwebcloud.lookingglassform2',
                    addForm: 'medwebcloud.addlookingglassform',
                    addButton: {
                        text: MedwebCloud.getMsg('medweb.view.lookingglass.import_glasses'),
                        iconCls: 'importGlassesIcon',
                    },
                    addTitle: MedwebCloud.getMsg('medweb.view.lookingglass.import_glasses'),
                    storeType: 'MedwebCloud.store.Devices',
                    layout: 'fit',
                    customArgs: {
                        'CompanyId': me.record.get('Id'),
                    },

                    filters: [{
                        property: 'CompanyId',
                        operator: '=',
                        value: me.record.get("Id")
                    }, {
                        property: 'DeviceType',
                        operator: '=',
                        value: MedwebCloud.enums.DeviceType.glasses
                    }],

                    columns: [{
                        dataIndex: 'Id',
                        text: 'Device Id',
                        flex: 1,
                        filter: [{
                            type: 'text',
                            exactMatch: true,
                        }],
                    }, {
                        dataIndex: 'Name',
                        text: 'Name',
                        flex: 9,
                        filter: [{
                            type: 'text',
                            exactMatch: false,
                        }],
                    }],
                    filterer: {
                        ftype: 'filters',
                        encode: false, // json encode the filter query
                        local: true,   // defaults to false (remote filtering)

                        filters: [{
                            dataIndex: 'Id',
                            type: 'number',
                        }, {
                            dataIndex: 'Name',
                            type: 'string',
                        }]
                    },

                    beforeAdd: function (item) {
                        item.set("CompanyId", me.record.get("Id"));
                    },
                }, {
                    title: 'Servers',
                    itemName: 'Server',
                    xtype: 'medwebcloud.windoweditor',
                    formType: 'medwebcloud.supportserverform2',
                    storeType: 'MedwebCloud.store.SupportServers',
                    layout: 'fit',

                    filters: [{
                        property: 'CompanyId',
                        operator: '=',
                        value: me.record.get("Id")
                    }],

                    columns: [{
                        dataIndex: 'Name',
                        text: 'Name',
                        flex: 2,
                        filter: [{
                            type: 'text',
                            exactMatch: false,
                        }],
                    }, {
                        dataIndex: 'HostUrl',
                        text: 'Url',
                        flex: 5,
                        filter: [{
                            type: 'text',
                            exactMatch: false,
                        }],
                    }, {
                        dataIndex: 'ServerType',
                        text: 'Type',
                        renderer: function (type, meta, record) {
                            switch (type) {
                                case 2:
                                    return 'Medweb EMR';

                                default:
                                    return '--';
                            }
                        },
                        flex: 1,
                    }],
                    filterer: {
                        ftype: 'filters',
                        encode: false, // json encode the filter query
                        local: true,   // defaults to false (remote filtering)

                        filters: [{
                            dataIndex: 'Name',
                            type: 'string',
                        }, {
                            dataIndex: 'Url',
                            type: 'string',
                        }]
                    },

                    beforeAdd: function (item) {
                        item.set("CompanyId", me.record.get("Id"));
                    },
                }]
            }]
        });
        this.callParent(arguments);
    },
    getField: function (fieldName, opts) {
        return MedwebCloud.util.ViewUtils.getField(fieldName, opts, 'medweb.view.admin.company.', '.label');
    },
    getLabel: function (fieldName) {
        return MedwebCloud.getMsg('medweb.view.admin.company.' + fieldName.toLowerCase() + '.label');
    }
});