﻿Ext.define('MedwebCloud.view.ContactsGrid', {
    extend: 'Ext.grid.Panel',
    xtype: 'MedwebCloud.contactsgrid',
    contactType: 1,
    patientData: null,
    loadMask: true,
    canEdit: false,
    initComponent: function () {
        var me = this;

        var contentColumns = [{
            dataIndex: 'Name',
            text: 'Name',
            flex: 1,
            renderer: function (value, meta, record) {
                if (this.patientData) {
                    return '<h4 class="contact-grid-name">' + record.raw.DisplayName + '</h4><div class="contact-grid-details">' + record.raw.Email + '</div>';
                } else {
                    var details = record.get('ContactDetails');
                    var emails = [];
                    if (details && details.length > 0) {
                        for (var i = 0; i < details.length; i++) {
                            if (details[i].TypeName == 'email' && !details[i].IsDeleted) {
                                if (details[i].Leve1Name == 'work') {
                                    emails.push(details[i].Value + MedwebCloud.getMessage('medweb.contact.detail.primary'));
                                } else {
                                    emails.push(details[i].Value);
                                }
                            }
                        }

                        meta['tdAttr'] = 'data-qtip="' + emails.join(', ') + '"';
                    }

                    return '<h4 class="contact-grid-name">' + value + '</h4><div class="contact-grid-details">' + emails.join(', ') + '</div>';
                }
            }
        }];

        if (me.canEdit) {
            contentColumns.push({
                width: 50,
                xtype: 'actioncolumn',
                items: [{
                    icon: '/images/user_edit.png',
                    tooltip: 'Edit this contact',
                    handler: function (grid, rowIndex, colIndex) {
                        var rec = grid.getStore().getAt(rowIndex);
                        Ext.create('MedwebCloud.view.ContactPanel', {
                            contactRecord: rec
                        }).show();
                    }
                }, {
                    icon: '/images/user_delete.png',
                    tooltip: 'Delete this contact from your address book',
                    handler: function (grid, rowIndex, colIndex) {
                        MedwebCloud.view.ContactPanel
                        var store = grid.getStore()
                        var rec = store.getAt(rowIndex);
                        rec.set('IsDeleted', true);
                        rec.save();
                        store.removeAt(rowIndex);
                    }
                }]
            });
        }

        var store = Ext.create('MedwebCloud.store.Contacts', {
            filters: [{
                property: 'ContactType',
                value: this.contactType
            },
            { property: 'IsDeleted', value: false }],
            autoLoad: true
        });

        if (this.patientData) {
            var storeData = [];
            for (var i in this.patientData) {
                storeData.push(this.patientData[i]);
            }
            store = new Ext.data.JsonStore({
                fields: ['Email', 'DisplayName'],
                autoLoad: true,
                data: storeData
            });
        }

        Ext.apply(me, {
            //autoHeight: true,
            sortableColumns: false,
            store: store,
            columns: contentColumns
        });

        me.enableBubble('cellclick', 'celldblclick', 'select');

        me.callParent(arguments);
    }
});