﻿Ext.define('MedwebCloud.view.InternalChatFilePanel', {
    extend: 'Ext.grid.Panel',
    xtype: 'MedwebCloud.internalchatfilepanel',
    user: null,
    internalChatId: 0,
    client: null,
    fileList: null,
    uploadColor: '#0f6',
    initComponent: function () {
        var me = this;

        var columnFilename = MedwebCloud.getMsg('medweb.view.internalchatfilepanel.column.filename.title');
        var columnCreatedOn = MedwebCloud.getMsg('medweb.view.internalchatfilepanel.column.creationdate.title');
        var buttonPreview = MedwebCloud.getMsg('medweb.view.internalchatfilepanel.button.preview.label');
        var buttonDownload = MedwebCloud.getMsg('medweb.view.internalchatfilepanel.button.download.label');
        var buttonDelete = MedwebCloud.getMsg('medweb.view.internalchatfilepanel.button.delete.label');
        var buttonDrag = MedwebCloud.getMsg('medweb.view.internalchatfilepanel.button.dragdrop.label');
        Ext.apply(this, {
            columns: [{
                text: columnFilename,
                flex: 1,
                dataIndex: 'Filename',
                renderer: function (value, meta, record) {
                    return value;
                }
            }, {
                text: columnCreatedOn,
                width: 140,
                dataIndex: 'CreatedOn',
                renderer: function (value, meta, record) {
                    if (!value) {
                        var percent = record.get('UploadProgress');
                        if (percent == 100) {
                            return '<div style="width:100%;height:100%;text-align:center;">' + MedwebCloud.getMsg('medweb.view.internalchatfilepanel.column.creationdate.progress.default') + '</div>';
                        } else {
                            return '<div style="width:' + percent + '%;background-color:' + me.uploadColor + ';height: 100%;text-align:center;color:#333;">' + percent + '%</div>';
                        }
                    } else {
                        return MedwebCloud.formatDate(value);
                    }
                }
            }, {
                xtype: 'actioncolumn',
                width: 75,
                renderer: function (value, meta, record) {
                    if (record.get('CreatedOn')) {
                        meta.column.items[0].icon = '/images/magnifier.png';
                        meta.column.items[0].disabled = false;
                        meta.column.items[1].icon = '/images/page_save.png';
                        meta.column.items[1].disabled = false;
                    } else {
                        meta.column.items[0].icon = Ext.BLANK_IMAGE_URL;
                        meta.column.items[0].disabled = true;
                        meta.column.items[1].icon = Ext.BLANK_IMAGE_URL;
                        meta.column.items[1].disabled = true;
                    }
                },
                items: [{
                    tooltip: buttonPreview,
                    handler: function (grid, rowIndex, colIndex) {
                        var rec = grid.getStore().getAt(rowIndex);
                        me.viewFile(rec);
                        return false;
                    }
                }, {
                    tooltip: buttonDownload,
                    handler: function (grid, rowIndex, colIndex) {
                        var rec = grid.getStore().getAt(rowIndex);
                        me.downloadFile(rec);
                        return false;
                    }
                }, {
                    icon: '/images/delete.png',
                    tooltip: buttonDelete,
                    handler: function (grid, rowIndex, colIndex) {
                        var rec = grid.getStore().getAt(rowIndex);
                        if (rec.get('CreatedOn')) {
                            // delete
                            rec.destroy({
                                success: function () {
                                    grid.getStore().remove(rec);
                                }
                            });
                        } else {
                            me.fireEvent('cancelFile', rec);
                        }
                        return false;
                    }
                }]
            }],
            bbar: {
                items: [{
                    xtype: 'tbtext',
                    text: buttonDrag
                }]
            }
        });
        this.callParent(arguments);
    },
    downloadFile: function (file) {
        window.open(file.get('Url'));
        //window.open(MedwebCloud.api('/v1/internalchatfiles/' + file.get('Id') + '?download=true'));
    },
    viewFile: function (file) {
        var name = file.get('Filename');
        var parts = name.split('.');
        var extension = parts[parts.length - 1];
        var path = MedwebCloud.api('/v1/internalchatfiles/' + file.get('Id') + '?view=true');

        if (this.isInlineViewable(extension)) {
            var w = new Ext.Window({
                title: name,
                width: 800,
                height: 600,
                maximizable: true,
                layout: 'fit',
                items: [{
                    border: false,
                    xtype: 'component',
                    autoEl: {
                        tag: 'iframe',
                        src: path
                    }
                }]
            });

            w.show();
        } else {
            window.open(path);
        }
    },
    isInlineViewable: function (extension) {
        if (!extension) {
            return false;
        }
        var valid = ['png', 'jpeg', 'jpg', 'bmp', 'pdf'];
        for (var i = 0; i < valid.length; i++) {
            if (extension.toLowerCase() == valid[i]) {
                return true;
            }
        }
        return false;
    },
    onDestroy: function () {
        this.callParent(arguments);
    },
    appendFile: function (file) {
        this.filesStore.insert(0, new MedwebCloud.model.InternalChatFile(file));
    },
    updateFile: function (file) {
        var index = this.filesStore.findBy(function (f) {
            return f.get('UniqueIdentifier') == file.UniqueIdentifier;
        });

        if (index > -1) {
            var record = this.filesStore.getAt(index);
            record.set('UploadProgress', file.UploadProgress);
        }
    },
    removeFile: function (file) {
        var index = this.filesStore.findBy(function (f) {
            return f.get('UniqueIdentifier') == file.UniqueIdentifier;
        });

        if (index > -1) {
            this.filesStore.removeAt(index);
        }
    },
    refresh: function (highlight) {
        var me = this;
        if (this.chatId) {
            // we load a new store so we can merge local (uploading) and remote (completed) files
            var tempStore = Ext.create('MedwebCloud.store.InternalChatFile', {
                proxy: {
                    type: 'therest',
                    url: MedwebCloud.api('/v1/internalchat/' + this.chatId + '/files')
                }
            });
            tempStore.on('load', function () {
                // run through the temp files
                tempStore.each(function (r) {
                    // find the items that match this file unique id and replace them
                    var idx = me.filesStore.findBy(function (f) {
                        return f.get('UniqueIdentifier') == r.get('UniqueIdentifier');
                    });
                    me.filesStore.removeAt(idx);
                    me.filesStore.insert(idx, r.copy());
                });

                if (highlight) {
                    me.hookHighlight(me.filesStore);
                }
            });
            tempStore.load();
        } else {
            // no chat id set, so we can't really do anything
        }
    },
    setChatId: function (chatId) {
        // load files
        var me = this;
        this.filesStore = Ext.create('MedwebCloud.store.InternalChatFile', {
            proxy: {
                type: 'therest',
                url: MedwebCloud.api('/v1/internalchat/' + chatId + '/files')
            }
        });
        this.reconfigure(this.filesStore);

        if (this.highlightFirst) {
            this.hookHighlight(this.filesStore);
        }

        this.filesStore.load();

        this.chatId = chatId;
    },
    hookHighlight: function (store) {
        var me = this;
        store.on('load', function () {
            // remove the default highlight and selection
            me.getSelectionModel().deselectAll();
            me.getView().clearHighlight();

            // get the row node
            var row = me.getView().getNode(0);
            //TODO: for some reason, this fails. don't have time to screw with it right now
            Ext.fly(row).highlight("ff0000", {
                attr: "background-color",
                easing: 'easeOut',
                duration: 1000
            });
        }, this, { single: true });
    }
});