﻿Ext.define('MedwebCloud.view.InternalChatMessagePanel', {
    extend: 'Ext.panel.Panel',
    xtype: 'MedwebCloud.internalchatmessagepanel',
    user: null,
    chatId: 0,
    client: null,
    messageList: null,
    initComponent: function () {
        var me = this;
        var plchldrMessage = MedwebCloud.getMsg('medweb.view.internalchatmessagepanel.message.placeholder');
        Ext.apply(this, {
            border: false,

            tbar: {
                items: [{
                    text: 'Clear History',
                    handler: function () {
                        Ext.Msg.confirm('Are you sure?', 'This will delete this entire chat history, for both participants in this chat. Are you sure?', function (btn) {
                            if (btn == 'yes') {
                                Ext.Ajax.request({
                                    method: 'DELETE',
                                    url: MedwebCloud.api('/v1/internalchat/' + me.chatId + '/messages'),
                                    success: function (response, opts) {
                                        me.messageList.clear();
                                        me.appendMessage();
                                    },
                                    failure: function (response, opts) {
                                        Ext.Msg.alert('Error', 'Failed to clear history. Error: ' + response.status);
                                    }
                                });
                            }
                        });
                    }
                }]
            },
            layout: {
                type: 'vbox',
                align: 'stretch'
            },
            items: [{
                xtype: 'panel',
                border: false,
                itemId: 'chatlog',
                autoScroll: true,
                flex: 4
            }, {
                layout: 'fit',
                flex: 1,
                border: false,
                items: [{
                    xtype: 'textarea',
                    itemId: 'chatInput',
                    disabled: true,
                    emptyText: plchldrMessage,
                    enterIsSpecial: true,
                    flex: 1,
                    listeners: {
                        specialkey: function (field, e) {
                            if (e.getKey() == e.ENTER) {
                                me.submitText()
                            }
                        }
                    }
                }]
            }]
        });
        this.messageList = new Ext.util.MixedCollection();
        this.callParent(arguments);
    },
    appendMessage: function (data) {
        var me = this;

        if (data) {
            if (this.messageList.getByKey(data.Id)) {
                // duplicate, ignore it
                return;
            }

            this.messageList.add(data.Id, data);
            this.messageList.sort('CreatedOn', 'ASC');
        }

        var log = me.down('#chatlog');
        // NOTE: if we swap from an hbox layout, this next line will break, so we have to check it
        if (log) {
            target = log.getLayout().innerCt;
        }

        if (!target) {
            return;
        }

        target.update('');

        var prev = null;
        var minutesForSentNotice = 2;

        var messageTemplate = new Ext.XTemplate(
            '<div class="message">',
                '<span class="speaker">{User.DisplayName}: </span>',
                '<span class="text" id="chatlog_{Id}">{Text}</span>',
            '</div>'
        );
        //TODO: Add this to Internationalizatoin
        var dateTemplate = new Ext.XTemplate(
            '<div class="message">',
                '<span class="when">Sent at {CreatedOn:date("g:i a")} on {CreatedOn:date("l")}</span>',
            '</div>'
        );

        var lastIsTimestamp = false;
        this.messageList.each(function (data) {
            if (data.InternalChatFile) {
                data.Text = '<a href="' + data.InternalChatFile.Url + '" target="_blank">' + data.InternalChatFile.Filename + '</a>';
            }
            messageTemplate.append(target, data);
            lastIsTimestamp = false;

            if (prev != null) {
                var d1 = Ext.Date.parseDate(prev.CreatedOn, 'c'),
                    d2 = Ext.Date.parseDate(data.CreatedOn, 'c'),
                    milliseconds = Math.abs(d1 - d2);

                // if the time is > 2 minutes between messages, show the time
                if (milliseconds > 1000 * 60 * minutesForSentNotice) {
                    dateTemplate.append(target, data);
                    lastIsTimestamp = true;
                }
            }

            prev = data;
        });

        // on the last one, also check and show the time
        if (prev != null && !lastIsTimestamp) {
            var d1 = Ext.Date.parseDate(prev.CreatedOn, 'c'),
                d2 = new Date(),
                milliseconds = Math.abs(d1 - d2);

            // if the time is > 2 minutes between messages, show the time
            if (milliseconds > 1000 * 60 * minutesForSentNotice) {
                dateTemplate.append(target, prev);
            }
        }

        var d = log.body.dom;
        if (d) {
            d.scrollTop = d.scrollHeight - d.offsetHeight + 2; // +2 accounts for border...bit hacky, but gets the job done for now
        }

        // re-render 4x a minute; this ensures we keep our timers up to date
        window.clearInterval(this._renderInterval);
        this._renderInterval = window.setInterval(Ext.bind(this.appendMessage, this), 1000 * 15);
    },
    destroy: function () {
        this.callParent(arguments);
        window.clearInterval(this._renderInterval);
    },
    setChatId: function (chatId) {
        // load files
        var me = this;
        this.messagesStores = Ext.create('MedwebCloud.store.InternalChatMessage', {
            proxy: {
                type: 'therest',
                url: MedwebCloud.api('/v1/internalchat/' + chatId + '/messages')
            }
        });
        this.messagesStores.on('load', function (store, records, success) {
            if (success) {
                for (var i = 0; i < records.length; i++) {
                    me.appendMessage(records[i].raw);
                }
            }
        });
        this.messagesStores.load();

        this.chatId = chatId;

        this.enableInput();
    },

    submitText: function () {
        var client = this.client;
        var item = this.down('.textarea');
        var me = this;

        if (!item.getValue()) {
            return;
        }

        var data = {
            Text: item.getValue()
        };
        item.disable();

        client.service({
            channel: '/' + MedwebCloud.userProfile.CompanyId + '/internalchat/' + this.chatId + '/messages',
            data: data,
            onSuccess: function (e) {
                item.setValue('');
                item.enable();

                me.appendMessage(e.getData());

                item.focus();
            },
            onFailure: function (e) {
                Ext.Msg.alert('Error', 'The message could not be sent.');
                item.enable();
            }
        });
    },
    enableInput: function () {
        this.down('#chatInput').enable();
    }
});