﻿Ext.define('MedwebCloud.view.InternalChatWindow', {
    extend: 'Ext.Window',
    requires: [
        'MedwebCloud.view.InternalChatMessagePanel',
        'MedwebCloud.view.InternalChatFilePanel',
        'MedwebCloud.view.InternalChatVideoPanel',
        'MedwebCloud.view.InternalChatMapPanel'
    ],
    xtype: 'MedwebCloud.internalchatwindow',
    internalChatId: 0,
    user: null,
    client: null,
    initComponent: function () {
        var me = this;

        var tabChat = MedwebCloud.getMsg('medweb.view.internalchatwindow.tab.chat.title');
        var tabFile = MedwebCloud.getMsg('medweb.view.internalchatwindow.tab.file.title');
        var tabVideo = MedwebCloud.getMsg('medweb.view.internalchatwindow.tab.video.title');
        var tabMap = MedwebCloud.getMsg('medweb.view.internalchatwindow.tab.map.title');
        var btnBegin = MedwebCloud.getMsg('medweb.view.internalchatwindow.video.button.begin.label');
        var btnEnd = MedwebCloud.getMsg('medweb.view.internalchatwindow.video.button.end.label');
        var msgWait = MedwebCloud.getMsg('medweb.view.internalchatwindow.video.waiting.message');

        this.connecting = soundManager.createSound({
            url: '/sounds/connecting.mp3'
        });

        Ext.apply(this, {
            files: [],
            title: this.user.DisplayName,
            cls: 'presence internalchatwindow',
            bodyCls: 'internalchatwindow-body',
            icon: MedwebCloud.getGravatarUrl(this.user.Username),
            height: 350,
            width: 450,
            border: false,
            layout: 'card',
            activeItem: 0,
            items: [{
                border: false,
                xtype: 'tabpanel',
                itemId: 'tab',
                deferredRender: false,
                tabPosition: 'bottom',
                defaults: {
                    border: false
                },
                items: [{
                    title: tabChat,
                    user: this.user,
                    channel: this.getMessagesChannel(this.internalChatId),
                    client: this.client,
                    itemId: 'internalChatMessagePanel',
                    xtype: 'MedwebCloud.internalchatmessagepanel',
                    listeners: {
                        newmessage: function () {
                            me.show();
                        }
                    }
                }, {
                    title: tabFile,
                    layout: 'fit',
                    itemId: 'internalChatFilePanel',
                    xtype: 'MedwebCloud.internalchatfilepanel',
                    listeners: {
                        cancelFile: function (internalChatFile) {
                            if (me.r) {
                                var uniqueId = internalChatFile.get('UniqueIdentifier');
                                var f = me.r.getFromUniqueIdentifier(uniqueId);
                                if (f) {
                                    f.cancel();

                                    me.removeFile({ UniqueIdentifier: uniqueId });
                                    me.setTitle(me.user.DisplayName);
                                }
                            }
                        }
                    }
                }, {
                    title: tabVideo,
                    layout: 'fit',
                    itemId: 'internalChatVideoPanelContainer',

                    tbar: {
                        items: [{
                            text: btnBegin,
                            itemId: 'startVideoChat',
                            icon: '/images/webcam.png',
                            disabled: true,
                            handler: function () {
                                me.startVideoChat();
                                me.connecting.play();
                            }
                        }, {
                            text: btnEnd,
                            itemId: 'endVideoChat',
                            icon: '/images/webcam_delete.png',
                            disabled: true,
                            handler: function () {
                                me.endVideoChat();
                                me.connecting.stop();
                            }
                        }]
                    },
                    items: [{
                        border: false,
                        header: false,
                        //TODO: not resizing properly after open/close
                        itemId: 'internalChatVideoPanel',
                        xtype: 'MedwebCloud.videopanel',
                        basechannel: MedwebCloud.userProfile.CompanyId + '/internalchat',
                        autoJoin: false,
                        autoLeave: true,
                        uniqueId: this.internalChatId,
                        waitingText: msgWait,
                        restoreToContainerSelector: '#internalChatVideoPanelContainer',
                        client: this.client,
                        record: this.record,
                        listeners: {
                            stop: function () {
                                me.fireEvent('video:stop');
                            },
                            start: function () {
                                me.fireEvent('video:start');
                            },
                            peerarrived: function () {
                                me.connecting.stop();
                            }
                        }
                    }]
                }, {
                    title: tabMap,
                    border: false,
                    user: this.user,
                    xtype: 'MedwebCloud.internalchatmappanel'
                }]
            }]
        });
        this.callParent(arguments);

        this.setPresence(this.user.Presence);
        this.join(this.internalChatId);
    },
    onDestroy: function () {
        window.clearInterval(this._renderInterval);
        this.leave();
        this.callParent(arguments);
    },
    getChannel: function (chatId, subchannel) {
        var base = '/' + MedwebCloud.userProfile.CompanyId + '/internalchat/' + chatId
        if (subchannel) {
            return base + '/' + subchannel;
        }
        return base;
    },
    getFilesChannel: function (chatId) {
        return this.getChannel(chatId, 'files');
    },
    getMessagesChannel: function (chatId) {
        return this.getChannel(chatId, 'messages');
    },
    leave: function () {
        MedwebCloud.openSessions -= 1;
        this.client.unsubscribe({
            channel: this.getChannel()
        });
        if (this.connecting && this.connecting.stop) {
            this.connecting.stop();
        }
    },
    join: function (chatId, callback) {
        var client = this.client;
        var me = this;

        if (chatId) {
            this._join(chatId);
            if (callback) {
                callback();
            }
        } else {
            // create the chat, or find it if it exists
            client.service({
                channel: '/' + MedwebCloud.userProfile.CompanyId + '/internalchat/with/' + this.user.Id,
                onSuccess: function (serviceArgs) {
                    me._join(serviceArgs.getData().Id);
                    if (callback) {
                        callback();
                    }
                    MedwebCloud.openSessions += 1;
                },
                onFailure: function () {
                    Ext.Msg.alert(
                        MedwebCloud.getMsg('medweb.view. internalchatwindow.video.action.join.error.title'),
                        MedwebCloud.getMsg('medweb.view. internalchatwindow.video.action.join.error.message'));
                }
            });
        }
    },
    joinAndStartVideoChat: function () {
        var me = this;
        this.join(this.internalChatId, function () {
            me.startVideoChat();
        });
    },
    _join: function (chatId) {
        var client = this.client;
        var me = this;

        this.internalChatId = chatId;

        this.down('#internalChatMessagePanel').setChatId(chatId);
        this.down('#internalChatFilePanel').setChatId(chatId);

        var videoPanel = this.down('#internalChatVideoPanel');
        videoPanel.setUniqueId(chatId);

        this.down('#startVideoChat').enable();
        this.down('#endVideoChat').disable();
    },
    hasVideoChat: function () {
        var videoPanel = this.down('#internalChatVideoPanel');
        return videoPanel.conferenceIsActive();
    },
    startVideoChat: function () {
        var videoPanel = this.down('#internalChatVideoPanel');
        videoPanel.joinConference();

        this.down('#tab').getLayout().setActiveItem(2);

        this.down('#startVideoChat').disable();
        this.down('#endVideoChat').enable();
    },
    endVideoChat: function () {
        var videoPanel = this.down('#internalChatVideoPanel');
        videoPanel.leave();

        this.down('#tab').getLayout().setActiveItem(0);

        this.down('#startVideoChat').enable();
        this.down('#endVideoChat').disable();
    },
    chatVideoAccepted: function () {
    },
    onRender: function () {
        this.callParent(arguments);

        var holder = this.el.dom;
        var client = this.client;
        var me = this;

        // when a file is dragged over the window, show the drop target area
        Ext.get(window).on('dragover', function () {
            Ext.get(holder).addCls('file-hover');
            return false;
        });
        Ext.get(window).on('dragend', function () {
            Ext.get(holder).removeCls('file-hover');
            return false;
        });

        // resumable file manager
        // http://www.resumablejs.com/
        var r = this.r = new Resumable({
            chunkSize: 1 * 1024 * 1024, // 1 meg
            fileParameterName: 'filename',
            generateUniqueIdentifier: Math.guid
        });
        r.assignDrop(holder);

        // when a file is uploaded, immediately trigger the upload
        r.on('fileAdded', function (file) {
            r.opts.target = '/handlers/InternalChatUploadHandler.ashx?internalChatId=' + me.internalChatId;

            Ext.get(holder).removeCls('file-hover');

            r.upload();
            me.appendFile({ Filename: file.fileName, UniqueIdentifier: file.uniqueIdentifier });
        });

        // cleanup
        r.on('complete', function (file, message) {
            me.setTitle(MedwebCloud.getMsg('medweb.view.internalchatwindow.file.action.upload.complete.message', me.user.DisplayName ));
            me._resetTitleTimeout = window.setTimeout(function () {
                me.setTitle(me.user.DisplayName);
                me.refreshFiles();
            }, 2000);
        });

        // status updates
        r.on('progress', function () {
            var percentage = (r.progress().toFixed(2) * 100);
            me.setTitle('Uploading... (' + parseInt(percentage, 10) + '%)');
        });

        r.on('fileProgress', function (file) {
            me.updateFile({ Filename: file.fileName, UniqueIdentifier: file.uniqueIdentifier, UploadProgress: file.progress() * 100 });
        });

        // everything before here ensures we show progress according to the sum of all the files
        // now we need to add per-file status updates and cancellation
    },
    appendMessage: function (data) {
        var internalChatMessagePanel = this.down('#internalChatMessagePanel');
        internalChatMessagePanel.appendMessage(data);

        //TODO: activate message tab
        this.down('#tab').getLayout().setActiveItem(internalChatMessagePanel);
    },
    appendFile: function (data) {
        var internalChatFilePanel = this.down('#internalChatFilePanel');
        internalChatFilePanel.appendFile(data);

        //TODO: activate file tab
        this.down('#tab').getLayout().setActiveItem(internalChatFilePanel);
    },
    updateFile: function (data) {
        var internalChatFilePanel = this.down('#internalChatFilePanel');
        internalChatFilePanel.updateFile(data);
    },
    removeFile: function (data) {
        var internalChatFilePanel = this.down('#internalChatFilePanel');
        internalChatFilePanel.removeFile(data);
    },
    refreshFiles: function () {
        this.down('#internalChatFilePanel').refresh();
    },
    setPresence: function (presence) {
        this.removeCls('busy idle offline active');
        this.addCls(MedwebCloud.getPresenceClassName(presence));
    }
});