﻿Ext.define('MedwebCloud.view.InvitationPanel', {
    extend: 'Ext.TabPanel',
    xtype: 'MedwebCloud.invitationpanel',
    requires: ['MedwebCloud.view.ContactsGrid'],
    record: null,
    initComponent: function () {
        var me = this;
        var tabPersonal = MedwebCloud.getMsg('medweb.view.invitationpanel.tab.personal.title');
        var btnAdd = MedwebCloud.getMsg('medweb.view.invitationpanel.personal.button.add.title');
        var tabInstitutional = MedwebCloud.getMsg('medweb.view.invitationpanel.tab.institutional.title');
        var tabGlobal = MedwebCloud.getMsg('medweb.view.invitationpanel.tab.global.title');

        Ext.apply(this, {
            border: false,
            activeItem: 0,

            items: [{
                title: tabPersonal,
                xtype: 'MedwebCloud.contactsgrid',
                itemId: 'personalGrid',
                contactType: 1,
                canEdit: true,
                tbar: {
                    items: [{
                        text: btnAdd,
                        iconCls: 'addContactIcon',
                        handler: function () {
                            MedwebCloud.createContact(function (values) {
                                var invitation = Ext.create('MedwebCloud.model.InvitationModel', {
                                    SessionId: me.record.get('Id'),
                                    Email: values.Email,
                                    DisplayName: values.DisplayName
                                });
                                invitation.save({
                                    failure: function () {
                                        Ext.Msg.alert(
                                            MedwebCloud.getMsg('medweb.view.invitationpanel.action.add.failure.title'),
                                            MedwebCloud.getMsg('medweb.view.invitationpanel.action.add.failure.message'));
                                    },
                                    success: function (s) {
                                        Ext.Msg.toast({ body: MedwebCloud.getMsg('medweb.view.invitationpanel.action.add.success.message') });
                                    }
                                });
                                me.fireEvent('invite');
                            });
                        }
                    }]
                }
            }, {
                title: tabInstitutional,
                xtype: 'MedwebCloud.contactsgrid',
                contactType: 2
            }]
        });

        Ext.apply(this.listeners, {
            addcontact: function (contact) {
                me.inviteContact(contact);
            },
            cellclick: function (grid, td, cellIndex, record, tr, rowIndex, e, eOpts) {
                me.selectedRecord = record;
                //w.down('#invite').enable();
            },
            celldblclick: function (grid, td, cellIndex, record, tr, rowIndex, e, eOpts) {
                me.inviteContact(record);
            }
        });
        this.callParent(arguments);
    },
    inviteSelected: function () {
        if (!this.selectedRecord) {
            return;
        }
        this.inviteContact(this.selectedRecord);
    },
    inviteContact: function (contactRecord) {
        var me = this;
        this.showEmails(contactRecord, function (displayName, email) {
            var invitation = Ext.create('MedwebCloud.model.InvitationModel', {
                SessionId: me.record.get('Id'),
                DisplayName: displayName,
                Email: email
            });
            invitation.save({
                failure: function () {
                    Ext.Msg.alert(
                        MedwebCloud.getMsg('medweb.view.invitationpanel.action.add.failure.title'),
                        MedwebCloud.getMsg('medweb.view.invitationpanel.action.add.failure.message'));
                },
                success: function (s) {
                    Ext.Msg.toast({ body: MedwebCloud.getMsg('medweb.view.invitationpanel.action.add.success.message') });
                }
            });
            me.fireEvent('invite');
        });
    },

    showEmails: function (contactRecord, callback) {
        var details = contactRecord.data.ContactDetails;

        if (!details) {
            Ext.Msg.alert(MedwebCloud.getMsg('medweb.view.invitationpanel.action.show.failure.message')    );
            return;
        }

        var detailsStore = Ext.create('Ext.data.Store', {
            filters: [{
                property: 'TypeName', value: 'email'
            }],
            fields: ['TypeName', 'Value', 'Level1Name', 'Level2Name'],
            data: {
                'items': details
            },
            proxy: {
                type: 'memory',
                reader: {
                    type: 'json',
                    root: 'items'
                }
            }
        });

        if (detailsStore.getCount() > 1) {
            var btnOk = MedwebCloud.getMsg('medweb.view.invitationpanel.action.show.button.positive.label');
            var btnCancel = MedwebCloud.getMsg('medweb.view.invitationpanel.action.show.button.negative.label');
            var title = MedwebCloud.getMsg('medweb.view.invitationpanel.action.show.window.title');
            var email = MedwebCloud.getMsg('medweb.user.email.label')
            var w = new Ext.Window({
                modal: true,
                title: title,
                height: 300,
                width: 300,
                layout: 'fit',
                border: false,
                items: [{
                    xtype: 'grid',
                    store: detailsStore,
                    columns: [
                        { text: 'Email', dataIndex: 'Value', flex: 1 }
                    ],
                    listeners: {
                        celldblclick: function (grid, td, cellIndex, record, tr, rowIndex, e, eOpts) {
                            callback(contactRecord.get('DisplayName'), record.get('Value'));
                            w.close();
                        }
                    }
                }],
                buttons: [{
                    text: btnOk,
                    handler: function () {
                        var records = w.down('.grid').getSelectionModel().getSelection();
                        if (!records || records.length == 0) {
                            return;
                        }
                        w.close();
                        callback(contactRecord.get('DisplayName'), records[0].get('Value'));
                    }
                }, {
                    text: btnCancel,
                    handler: function () {
                        w.close();
                    }
                }]
            });
            w.show();
        } else {
            callback(contactRecord.get('DisplayName'), detailsStore.getAt(0).get('Value'));
        }
    }
});