﻿Ext.define('MedwebCloud.view.LookingGlassPanel', {
    xtype: 'medwebcloud.lookingglasspanel',
    extend: 'Ext.Panel',
    client: null,
    initComponent: function () {
        var me = this;

        Ext.apply(this, {
            layout: 'fit'
        });

        this.callParent(arguments);
        this._glassesMap = {};
    },
    doLayout: function () {
        this.callParent(arguments);
        if (window.google) {
            google.maps.event.trigger(this.map, "resize");
            this.map.setCenter(this.position);
        }
    },
    afterRender: function () {
        this.callParent(arguments);
        var me = this;
        var handleNoGeolocation = function (errorFlag) {
            if (errorFlag) {
                var content = 'Error: The Geolocation service failed.';
            } else {
                var content = 'Error: Your browser doesn\'t support geolocation.';
            }

            var options = {
                map: map,
                position: new google.maps.LatLng(60, 105),
                content: content
            };

            var infowindow = new google.maps.InfoWindow(options);
            map.setCenter(options.position);
        };
        if (!window.google) {
            return;
        }

        var opts = {
            zoom: 6,
            center: new google.maps.LatLng(60, 105),
            mapTypeControl: false,
            navigationControlOptions: { style: google.maps.NavigationControlStyle.SMALL },
            mapTypeId: google.maps.MapTypeId.ROADMAP
        };

        this.sv = new google.maps.StreetViewService();
        var map = this.map = new google.maps.Map(this.getLayout().getRenderTarget().dom, opts);
        // Try HTML5 geolocation
        if (navigator.geolocation) {
            navigator.geolocation.getCurrentPosition(function (position) {
                var pos = new google.maps.LatLng(position.coords.latitude,
                                                 position.coords.longitude);

                me.youAreHere = new google.maps.InfoWindow({
                    map: map,
                    position: pos,
                    content: 'You are here.'
                });

                map.setCenter(pos);

                me.position = pos;
            }, function () {
                handleNoGeolocation(true);
            });
        } else {
            // Browser doesn't support Geolocation
            handleNoGeolocation(false);
        }
    },
    onLocationChanged: function (localRecord, locationChangedEvent) {
        var newLocation = {
            lat: locationChangedEvent.Latitude,
            lng: locationChangedEvent.Longitude,
            dir: locationChangedEvent.Direction
        };

        if (this.isSamePosition(localRecord.lastPosition, newLocation)) {
            if (localRecord.streetview && localRecord.syncSVCam) {
                this.updateStreetViewAngle(localRecord.streetview, newLocation);
            }
            localRecord.lastPosition.dir = newLocation.dir;
        } else {
            if (localRecord.infowindow) {
                this.updateInfoWindowPosition(localRecord.infowindow, newLocation);
            }

            if (localRecord.streetview && localRecord.syncSVCam) {
                this.updateStreetView(localRecord, newLocation);
            }

            localRecord.lastPosition = newLocation;
        }
    },
    updateInfoWindowPosition: function (infoWindow, newLocation) {
        var gLatLng = new google.maps.LatLng(newLocation.lat, newLocation.lng);

        infoWindow.setPosition(gLatLng);
    },
    updateStreetViewAngle: function (streetView, newLocation) {
        streetView.setPov({
            heading: newLocation.dir,
            pitch: 0
        });
    },
    updateStreetView: function (localRecord, newLocation) {
        var me = this;
        var recordId = localRecord.ref;

        if (this.isSVLocked(recordId)) {
            localRecord.svQueue = localRecord.svQueue || [];
            localRecord.svQueue.push(newLocation);
        }
        else {
            this.lockSV(recordId);

            this.sv.getPanorama({
                location: newLocation,
                radius: 50
            }, function (data, status) {
                try {
                    if (status === google.maps.StreetViewStatus.OK) {
                        localRecord.streetview.setPano(data.location.pano);
                        localRecord.streetview.setPov({
                            heading: newLocation.dir,
                            pitch: 0
                        });
                        localRecord.lastPosition.dir = newLocation.dir;
                    }
                    else {
                        console.log("Failure", data, status);
                    }
                }
                finally {
                    me.unlockSV(recordId);
                    if (me._glassesMap[recordId].svQueue && me._glassesMap[recordId].svQueue.length) {
                        var newLoc = me._glassesMap[recordId].svQueue.pop();
                        me._glassesMap[recordId].svQueue = [];
                        updateStreetView(me._glassesMap[recordId], newLoc);
                    }
                }
            });
        }
    },
    isSamePosition: function (pos1, pos2) {
        var deltaDir = 0.0001; //Roughly 10m
        var deltaDir2 = deltaDir * deltaDir;

        var deltaLat = pos1.lat - pos2.lat;
        var deltaLat2 = deltaLat * deltaLat;
        var deltaLng = pos1.lng - pos2.lng;
        var deltaLng2 = deltaLng * deltaLng;

        if (deltaLat2 + deltaLng2 < deltaDir2) {
            return true;
        }

        return false;
    },
    isSVLocked: function (recordId) {
        return this._glassesMap[recordId].isSVLocked;
    },
    lockSV: function (recordId) {
        this._glassesMap[recordId].isSVLocked = true;
    },
    unlockSV: function (recordId) {
        this._glassesMap[recordId].isSVLocked = false;
    },
    disableSvUi: function (glassId) {
        this._glassesMap[glassId].streetview.setOptions({
            disableDefaultUI: true,
            addressControl: true,
            clickToGo: false,
            panControl: false,
            zoomControl: false,
            draggable: false,
            scrollwheel: false,
        });
    },
    enableSvUi: function (glassId) {
        this._glassesMap[glassId].streetview.setOptions({
            disableDefaultUI: false,
            clickToGo: true,
            panControl: true,
            zoomControl: true,
            draggable: true,
            scrollwheel: true,
        });
    },
    addGlasses: function (record) {
        if (this._glassesMap[record.get('UniqueId')]) {
            return;
        }

        this._glassesMap[record.get('UniqueId')] = {
            ref: record.get('UniqueId'),
            record: record,
            infowindow: null,
            syncSVCam: true,
        };

        var w = 320, h = 270; //320 + (6 * 2), h = 240 + (16 + 22 + 6); //320, h = 270;
        var me = this;
        var containerProps = {
            layout: 'fit',
            header: true,
            closable: true,
            closeAction: 'destroy',
            title: record.get('Id') + ' (' + record.get('Name') + ')',
            items: [{
                border: false,
                xtype: 'medwebcloud.lookingglassvideopanel',
                height: h,
                width: w,
                client: this.client,
                leaveButton: true,
                textOnButtons: false,
                uniqueId: record.get('UniqueId'),
                record: record,
                listeners: {
                    locationChanged: function (locationChangedEvent) {
                        try {
                            console.log("New location: ", locationChangedEvent);
                            var localRecord = me._glassesMap[record.get('UniqueId')];

                            if (localRecord) {
                                me.onLocationChanged(localRecord, locationChangedEvent);
                            }
                        } catch (ex) {
                            console.log("Exception on locationChanged: ", ex);
                        }
                    }
                },
            }],
            listeners: {
                beforedestroy: function () {
                    var popup = me._glassesMap[record.get('UniqueId')].infowindow;
                    if (popup) {
                        popup.close();
                    }
                    delete me._glassesMap[record.get('UniqueId')];
                }
            }
        };

        var id = Ext.id();
        var location = record.raw.Location;
        if (location) {
            this._glassesMap[record.get('UniqueId')].lastPosition = {
                lat: location.Latitude,
                lng: location.Longitude,
                dir: location.Direction,
            };

            var bubbleDom = document.createElement('div');

            if (window.google) {
                var options = {
                    map: me.map,
                    position: new google.maps.LatLng({ lat: location.Latitude, lng: location.Longitude }),
                    content: bubbleDom,
                    closeBoxURL: '', //Hide closebox (and ugly right margin) for InfoBox
                    boxStyle: {
                        width: '640px'
                    },
                    //-19 = 20px - 1px margin-top
                    pixelOffset: new google.maps.Size(-320, -19),
                    alignBottom: true,
                };

                containerProps.layout = {
                    type: 'table',
                    columns: 2,
                };

                var streetviewPanelProps = {
                    header: false,
                    width: w,
                    height: h,
                    border: false,
                    itemId: 'streetview',
                    style: 'border-left: 1px solid #99BBE8',
                    html: '<div></div>',
                    bbar: ['->', {
                        text: 'Locked',
                        iconCls: 'lockedVideoIcon',
                        iconAlign: 'right',
                        handler: function () {
                            //Lock auto-sync
                            if (this.iconCls === 'videoIcon') {
                                me.disableSvUi(record.get('UniqueId'));
                                me._glassesMap[record.get('UniqueId')].syncSVCam = true;
                                this.setIconCls('lockedVideoIcon');
                                this.setText('Locked');
                                me.updateStreetView(me._glassesMap[record.get('UniqueId')], me._glassesMap[record.get('UniqueId')].lastPosition);
                            } else {
                                me.enableSvUi(record.get('UniqueId'));
                                me._glassesMap[record.get('UniqueId')].syncSVCam = false;
                                this.setIconCls('videoIcon');
                                this.setText('Unlocked');
                            }
                        }
                    }],
                };

                containerProps.items.push(streetviewPanelProps);
                var infoPanel = new Ext.Panel(containerProps);

                me._glassesMap[record.get('UniqueId')].infowindow = new InfoBox(options);
                google.maps.event.addListener(me._glassesMap[record.get('UniqueId')].infowindow, 'domready', function () {
                    infoPanel.on('afterrender', function (o, e) {
                        me.sv.getPanorama({ location: options.position, radius: 50 }, function (data, status) {
                            if (status === google.maps.StreetViewStatus.OK) {
                                panorama = new google.maps.StreetViewPanorama(infoPanel.getComponent('streetview').body.dom);
                                panorama.setPano(data.location.pano);
                                panorama.setPov({
                                    heading: 0,
                                    pitch: 0
                                });
                                me._glassesMap[record.get('UniqueId')].streetview = panorama;
                                me.disableSvUi(record.get('UniqueId'));
                                panorama.setVisible(true);
                            }
                            else {
                                console.log("Failure", data, status);
                            }
                        });
                    });
                    infoPanel.render(bubbleDom);
                });

                me._glassesMap[record.get('UniqueId')].infowindow.open(me.map);
            }
        } else {
            var infoPanel = new Ext.Panel(containerProps);

            var x = new Ext.Window({
                height: h + 30,
                layout: 'fit',
                border: false,
                title: infoPanel.title,
                items: [infoPanel]
            });

            x.show();
            infoPanel.getHeader().hide();
        }

        if (this.youAreHere) {
            this.youAreHere.close();
        }
    }
});