﻿Ext.define('MedwebCloud.view.LookingGlassSetupWizard.LocalNetwork', {
    xtype: 'medwebcloud.lookingglasssetupwizard.localnetwork',
    extend: 'Ext.form.Panel',

    initComponent: function () {
        var me = this;

        var labelNetwork = MedwebCloud.getMsg('medweb.network.network.label');
        var labelSecurity = MedwebCloud.getMsg('medweb.network.security.label');
        var labelPasscode = MedwebCloud.getMsg('medweb.network.password.label');

        Ext.apply(this, {
            items: [{
                xtype: 'textfield',
                itemId: 'network_id',
                fieldLabel: labelNetwork,
                width: 285,
                allowBlank: false,
                listeners: {
                    specialkey: me.handleEnter
                }
            }, {
                xtype: 'combobox',
                itemId: 'network_security',
                fieldLabel: labelSecurity,
                store: ['WPA', 'WEP', 'None'],
                value: 'Please Select',
                width: 285,
                forceSelection: true,
                editable: false,
                listeners: {
                    specialkey: me.handleEnter
                }
            }, {
                xtype: 'textfield',
                itemId: 'network_passcode',
                inputType: 'password',
                fieldLabel: labelPasscode,
                width: 285,
                allowBlank: false,
                listeners: {
                    specialkey: me.handleEnter
                }
            }, {
                xtype: 'button',
                itemId: 'live-sync',
                name: 'live-sync',
                text: 'Generate Scannable Barcode',
                handler: me.createQR,
            }],
            listeners: {
                activate: function () {
                    me.down('#skipBtn').disabled = true;
                    me.down('#nextBtn').disabled = true;
                }
            }
        });

        me.callParent(arguments);
    },
    getValues: function () {
        return this.callParent(arguments);
    },
    handleEnter: function (field, e) {
        if (e.getKey() == e.ENTER) {
            me.createQR();
        }
    },
    createQR: function () {
        var me = this;

        if (!me.isValid()) return;

        var guid = Math.guid();
        me.wiz.syncChannel = '/lookingglass/setup/' + guid;
        var obj = {
            network: {
                wifi: {
                    network: me.down('#network_id').getValue(),
                    security: me.down('#network_security').getValue(),
                    passcode: me.down('#network_passcode').getValue(),
                },
                websync: MedwebCloud.websyncPath,
                icelink: MedwebCloud.serverAddressIceLink + ':' + MedwebCloud.serverPortIceLink,
            },
            sync: {
                channel: me.wiz.syncChannel,
            },
        };

        var sub = new fm.websync.subscribeArgs([me.wiz.syncChannel]);
        sub.setOnReceive(function (r) {
            var d = JSON.parse(r.getDataJson());

            if (d['glass'] && d['glass'] === 'OK') {
                Ext.Msg.toast({ body: 'Glasses synced.' });
                me.down('nextBtn').disabled = false;
            }
            else {
                Ext.Msg.toast({ body: 'An error occurred during sync.  Please try again.' });
            }
        });
        sub.setOnFailure(function (f) {
            Ext.Msg.toast({ body: 'An error occured connecting to MedwebCloud.  Please refresh the page and try again.' });
        });
        MedwebCloud.websyncClient.subscribe(sub);

        Ext.Ajax.request({
            url: '/GenerateCode.aspx',
            method: 'POST',
            headers: {
                'Content-Type': 'text/plain', //Mono error for app/json https://bugzilla.xamarin.com/show_bug.cgi?id=6989
            },
            jsonData: obj,
            success: function (response, options) {
                //console.log(response);
                try {
                    var dta = JSON.parse(response.responseText);
                    var src = dta['data'];

                    if (!src) {
                        if (dta['error']) {
                            //Uh oh!
                            console.log(dta);
                            Ext.Msg.toast({ body: 'Could not generate QR code.  Please check your values and try again.' });
                        }
                    }

                    var previewWindow = new Ext.Window({
                        modal: true,
                        border: false,
                        plain: true,
                        width: 600,
                        height: 600,
                        html: '<img src="' + src + '" />',
                        title: 'Scan to Sync',
                        constrain: true,
                        resizable: { preserveAspectRatio: true }
                    });
                    previewWindow.show();
                }
                catch (e) {
                    console.log("Error success: ", e);
                    console.log(response);
                }
            },
            failure: function (response, options) {
                var responseObj = {};
                try {    // This will fail if not valid json response
                    responseObj = Ext.decode(response.responseText);
                } catch (e) { }
                var error = responseObj || response.responseText;;

                console.log(error);
                console.log(responseObj);

                //w.unmask();
            },
            callback: function () {
                //w.down('#login').enable();
            }
        });
    }
});