﻿Ext.define('MedwebCloud.view.LookingGlassSetupWizard.Networks', {
    xtype: 'medwebcloud.lookingglasssetupwizard.networks',
    extend: 'Ext.form.Panel',

    initComponent: function () {
        var me = this;

        Ext.define("Network", {
            extend: 'Ext.data.Model',
            fields: ['SSID', 'Security', 'Passphrase'],
        });

        me.networkStore = Ext.create('Ext.data.ArrayStore', {
            model: 'Network',
        });

        Ext.define("NetworkSecurityType", {
            extend: 'Ext.data.Model',
            fields: ['Type'],
        });

        me.secuirtyTypeStore = Ext.create('Ext.data.ArrayStore', {
            model: 'NetworkSecurityType',
            data: [['None'], ['WEP'], ['WPA']]
        });

        var rowEditing = Ext.create('Ext.grid.plugin.RowEditing', {
            clicksToEdit: 2
        });

        Ext.apply(this, {
            defaults: {
                margin: '0 0 20 0',
            },
            items: [{
                xtype: 'panel',
                title: false,
                border: false,
                cls: 'wiz-heading',
                width: '100%',
                html: 'Configure default networks:',
            }, {
                xtype: 'panel',
                title: false,
                border: false,
                cls: 'wiz-text',
                width: '100%',
                html: 'Add WiFi networks to the list of automatic connections for the glasses.' +
                    '<br /><br />' +
                    'These networks will be added to the device&#39;s saved connections.  No previously saved networks will be removed.',
            }, {
                xtype: 'grid',
                width: '90%',
                viewConfig: {
                    stripeRows: true
                },
                selType: 'rowmodel',
                sortable: false,
                plugins: [rowEditing],
                viewConfig: {
                    deferEmptyText: false,
                    emptyText: '<div class="empty-grid">No additional networks.</div>',
                },
                maxHeight: 300,
                height: 300,
                tbar: [{
                    text: 'Add Network',
                    iconCls: 'network-add-icon contain-image',
                    handler: function () {
                        rowEditing.cancelEdit();

                        var r = Ext.create('Network', {
                            'SSID': '',
                            'Security': 'WPA',
                            'Passphrase': '',
                        });

                        me.networkStore.insert(0, r);
                        rowEditing.startEdit(0, 0);
                    }
                }, {
                    itemId: 'removeNetwork',
                    text: 'Erase Network',
                    iconCls: 'network-remove-icon contain-image',
                    handler: function () {
                        var sm = grid.getSelectionModel();
                        rowEditing.cancelEdit();
                        me.networkStore.remove(sm.getSelection());
                        if (me.networkStore.getCount() > 0) {
                            sm.select(0);
                        }
                    },
                    disabled: true
                }],
                columns: [
                    { dataIndex: 'SSID', header: 'Name', flex: 0.4, sortable: true, editor: { xtype: 'textfield', allowBlank: false } },
                    { dataIndex: 'Security', header: 'Security', flex: 0.2, editor: { xtype: 'combobox', store: me.secuirtyTypeStore, displayField: 'Type', valueField: 'Type' } },
                    { dataIndex: 'Passphrase', header: 'Password', flex: 0.4, sortable: false, editor: { xtype: 'textfield' }, renderer: function () { return '********'; } },
                ],
                store: me.networkStore,
                listeners: {
                    'selectionchange': function (view, records) {
                        this.down('#removeNetwork').setDisabled(!records.length);
                    }
                },
                setValue: function (valArr) {
                    me.networkStore.loadRawData(valArr);
                },
            }],
            listeners: {
                activate: function () {
                    var wiz = this.up('panel').up();
                    var wifis = wiz.getData('SupportedNetworks');
                    if (wifis && wifis.length) {
                        me.networkStore.loadData(wifis);
                    }
                },
            }
        });

        this.callParent(arguments);
    },
    saveChanges: function () {
        var me = this;

        var SupportedNetworks = [];

        this.networkStore.each(function (rec) {
            SupportedNetworks.push({
                "SSID": rec.get("SSID"),
                "Security": rec.get("Security"),
                "Passphrase": rec.get("Passphrase"),
            });
        });

        me.getRecord()['SupportedNetworks'] = SupportedNetworks;
    },
});