﻿Ext.define('MedwebCloud.view.LookingGlassSetupWizard.VoiceImages', {
    xtype: 'medwebcloud.lookingglasssetupwizard.voiceimages',
    extend: 'Ext.form.Panel',

    initComponent: function () {
        var me = this;

        Ext.define("VoiceCommand", {
            extend: 'Ext.data.Model',
            fields: ['Command', 'Sensitivity'],
        });

        var rowEditing = Ext.create('Ext.grid.plugin.RowEditing', {
            clicksToEdit: 1
        });
        var glasses = me.context.glasses;

        Ext.apply(this, {
            defaults: {
                margin: '0 0 20 0',
            },
            items: [{
                xtype: 'panel',
                title: false,
                border: false,
                cls: 'wiz-heading',
                width: '100%',
                html: 'Configure Image Settings:',
            }, {
                xtype: 'panel',
                title: false,
                border: false,
                cls: 'wiz-text',
                width: '100%',
                html: 'Shared pictures and saved snapshots may be viewed by head gesture or voice command.' +
                    '<br /><br />' +
                    'You can also manipulate the images by zooming and panning.',
            }, {
                xtype: 'medwebcloud.lookingglasssetupwizard.voiceinstance',
                title: 'Show Images',
                itemId: 'cmd-show-images',
                description: 'Show image list',
                glasses: glasses,
                phrases: [
                    ['show images', 70],
                ],
                listeners: {
                    'commandsChanged': function () {
                        me.raiseValuesChanged(this.getValues());
                    },
                },
            }, {
                xtype: 'medwebcloud.lookingglasssetupwizard.voiceinstance',
                title: 'Hide Images',
                itemId: 'cmd-hide-images',
                description: 'Hide image list',
                glasses: glasses,
                phrases: [
                    ['hide images', 70],
                ],
                listeners: {
                    'commandsChanged': function () {
                        me.raiseValuesChanged(this.getValues());
                    },
                },
            }, {
                xtype: 'medwebcloud.lookingglasssetupwizard.voiceinstance',
                title: 'Next Image',
                itemId: 'cmd-next-image',
                description: 'Next image',
                glasses: glasses,
                phrases: [
                    ['next image', 70],
                    ['right', 70],
                ],
                listeners: {
                    'commandsChanged': function () {
                        me.raiseValuesChanged(this.getValues());
                    },
                },
            }, {
                xtype: 'medwebcloud.lookingglasssetupwizard.voiceinstance',
                title: 'Previous Image',
                itemId: 'cmd-prev-image',
                description: 'Previous image',
                glasses: glasses,
                phrases: [
                    ['previous image', 70],
                    ['left', 70],
                ],
                listeners: {
                    'commandsChanged': function () {
                        me.raiseValuesChanged(this.getValues());
                    },
                },
            }, {
                xtype: 'medwebcloud.lookingglasssetupwizard.voiceinstance',
                title: 'Zoom In',
                itemId: 'cmd-zoom-in-image',
                description: 'Zoom in on the image (roughly 10% magnification change)',
                glasses: glasses,
                phrases: [
                    ['zoom in', 70],
                ],
                listeners: {
                    'commandsChanged': function () {
                        me.raiseValuesChanged(this.getValues());
                    },
                },
            }, {
                xtype: 'medwebcloud.lookingglasssetupwizard.voiceinstance',
                title: 'Zoom Out',
                itemId: 'cmd-zoom-out-image',
                description: 'Zoom out on the image (roughly 10% magnification change)',
                glasses: glasses,
                phrases: [
                    ['zoom out', 70],
                ],
                listeners: {
                    'commandsChanged': function () {
                        me.raiseValuesChanged(this.getValues());
                    },
                },
            }, {
                xtype: 'medwebcloud.lookingglasssetupwizard.voiceinstance',
                title: 'Pan Up',
                itemId: 'cmd-pan-up-image',
                description: 'Pan the image up',
                glasses: glasses,
                phrases: [
                    ['pan up', 70],
                ],
                listeners: {
                    'commandsChanged': function () {
                        me.raiseValuesChanged(this.getValues());
                    },
                },
            }, {
                xtype: 'medwebcloud.lookingglasssetupwizard.voiceinstance',
                title: 'Pan Left',
                itemId: 'cmd-pan-left-image',
                description: 'Pan the image left',
                glasses: glasses,
                phrases: [
                    ['pan left', 70],
                ],
                listeners: {
                    'commandsChanged': function () {
                        me.raiseValuesChanged(this.getValues());
                    },
                },
            }, {
                xtype: 'medwebcloud.lookingglasssetupwizard.voiceinstance',
                title: 'Pan Down',
                itemId: 'cmd-pan-down-image',
                description: 'Pan the image down',
                glasses: glasses,
                phrases: [
                    ['pan down', 70],
                ],
                listeners: {
                    'commandsChanged': function () {
                        me.raiseValuesChanged(this.getValues());
                    },
                },
            }, {
                xtype: 'medwebcloud.lookingglasssetupwizard.voiceinstance',
                title: 'Pan Right',
                itemId: 'cmd-pan-right-image',
                description: 'Pan the image right',
                glasses: glasses,
                phrases: [
                    ['pan right', 70],
                ],
                listeners: {
                    'commandsChanged': function () {
                        me.raiseValuesChanged(this.getValues());
                    },
                },
            }],
            listeners: {
                activate: function () {
                    var form = this;

                    var wiz = this.up('panel').up();
                    var voice = wiz.getData('VoiceCommands');

                    //Handle the actual form fields
                    this.getForm().getFields().each(function (itm) {
                        var data = wiz.getData(itm.getItemId());
                        if (data != null && data != undefined) {
                            itm.setValue(data);
                        }
                    });

                    //Handle the voice-instances
                    this.items.each(function (itm) {
                        var id = itm.getItemId();
                        if (id.substr(0, 4) === 'cmd-') {
                            if (voice && voice.length) {
                                var cmdId = id.substr(4);
                                var cmds = voice.filter(function (c) { return c.Key === cmdId; });
                                if (cmds && cmds.length && cmds[0].Phrases) {
                                    itm.setValues(cmds[0].Phrases);
                                    return;
                                }
                            }
                        }
                    });
                },
            }
        });

        this.callParent(arguments);
    },
    raiseValuesChanged: function () {
        var me = this;

        me.fireEvent('valuesChanged', me.getValues());
    },
    saveChanges: function () {
        var me = this;

        var items = (!!this.getFormItems) ? this.getFormItems() : this.items;
        items.each(function (itm) {
            if (itm.getValues) {
                me.saveVoiceCommand(itm.getItemId().substr(4));
            }
        });
    },
    saveVoiceCommand: function (cmd) {
        var me = this;
        if (!me.getRecord()['VoiceCommands']) me.getRecord()['VoiceCommands'] = [];

        var values = me.down('#cmd-' + cmd).getValues();

        var vc = me.getRecord()['VoiceCommands'].filter(function (vc) {
            return vc.Key === cmd;
        });
        if (!vc || !vc.length) {
            me.getRecord()['VoiceCommands'].push({
                Key: cmd,
                Phrases: values,
            });
        }
        else {
            vc[0].Phrases = values;
        }
    },
});