﻿Ext.define('MedwebCloud.view.LookingGlassSetupWizard.VoiceInstruments', {
    xtype: 'medwebcloud.lookingglasssetupwizard.voiceinstruments',
    extend: 'Ext.form.Panel',

    initComponent: function () {
        var me = this;

        Ext.define("VoiceCommand", {
            extend: 'Ext.data.Model',
            fields: ['Command', 'Sensitivity'],
        });

        var rowEditing = Ext.create('Ext.grid.plugin.RowEditing', {
            clicksToEdit: 1
        });
        var glasses = me.context.glasses;

        Ext.apply(this, {
            defaults: {
                margin: '0 0 20 0',
            },
            items: [{
                xtype: 'panel',
                title: false,
                border: false,
                cls: 'wiz-heading',
                width: '100%',
                html: 'Configure Device Stream Settings:',
            }, {
                xtype: 'panel',
                title: false,
                border: false,
                cls: 'wiz-text',
                width: '100%',
                html: 'While in a Medweb LIVE! session, other participants may share streaming medical data.',
            }, {
                xtype: 'medwebcloud.lookingglasssetupwizard.voiceinstance',
                title: 'Show Device Stream',
                itemId: 'cmd-show-stream',
                description: 'Show the currently shared device stream.',
                glasses: glasses,
                phrases: [
                    ['show stream', 70],
                ],
                listeners: {
                    'commandsChanged': function () {
                        me.raiseValuesChanged(this.getValues());
                    },
                },
            }, {
                xtype: 'medwebcloud.lookingglasssetupwizard.voiceinstance',
                title: 'Hide Device Stream',
                itemId: 'cmd-hide-stream',
                description: 'Hide the currently shared device stream.',
                glasses: glasses,
                phrases: [
                    ['hide stream', 70],
                ],
                listeners: {
                    'commandsChanged': function () {
                        me.raiseValuesChanged(this.getValues());
                    },
                },
            }],
            listeners: {
                activate: function () {
                    var form = this;

                    var wiz = this.up('panel').up();
                    var voice = wiz.getData('VoiceCommands');

                    //Handle the actual form fields
                    this.getForm().getFields().each(function (itm) {
                        var data = wiz.getData(itm.getItemId());
                        if (data != null && data != undefined) {
                            itm.setValue(data);
                        }
                    });

                    //Handle the voice-instances
                    this.items.each(function (itm) {
                        var id = itm.getItemId();
                        if (id.substr(0, 4) === 'cmd-') {
                            if (voice && voice.length) {
                                var cmdId = id.substr(4);
                                var cmds = voice.filter(function (c) { return c.Key === cmdId; });
                                if (cmds && cmds.length && cmds[0].Phrases) {
                                    itm.setValues(cmds[0].Phrases);
                                    return;
                                }
                            }
                        }
                    });
                },
            }
        });

        this.callParent(arguments);
    },
    raiseValuesChanged: function () {
        var me = this;

        me.fireEvent('valuesChanged', me.getValues());
    },
    saveChanges: function () {
        var me = this;

        var items = (!!this.getFormItems) ? this.getFormItems() : this.items;
        items.each(function (itm) {
            if (itm.getValues) {
                me.saveVoiceCommand(itm);
            }
        });
    },
    saveVoiceCommand: function (itm) {
        var me = this;
        if (!me.getRecord()['VoiceCommands']) me.getRecord()['VoiceCommands'] = [];

        var values = itm.getValues();
        var cmdId = itm.getItemId().substr(4);

        var vc = me.getRecord()['VoiceCommands'].filter(function (vc) {
            return vc.Key === cmdId;
        });
        if (!vc || !vc.length) {
            me.getRecord()['VoiceCommands'].push({
                Key: cmdId,
                Phrases: values,
            });
        }
        else {
            vc[0].Phrases = values;
        }
    },
});