﻿Ext.define('MedwebCloud.view.LookingGlassSetupWizard.VoiceVideos', {
    xtype: 'medwebcloud.lookingglasssetupwizard.voicevideos',
    extend: 'Ext.form.Panel',

    initComponent: function () {
        var me = this;

        Ext.define("VoiceCommand", {
            extend: 'Ext.data.Model',
            fields: ['Command', 'Sensitivity'],
        });

        var rowEditing = Ext.create('Ext.grid.plugin.RowEditing', {
            clicksToEdit: 1
        });
        var glasses = me.context.glasses;

        Ext.apply(this, {
            defaults: {
                margin: '0 0 20 0',
            },
            items: [{
                xtype: 'panel',
                title: false,
                border: false,
                cls: 'wiz-heading',
                width: '100%',
                html: 'Configure Local Video Settings:',
            }, {
                xtype: 'panel',
                title: false,
                border: false,
                cls: 'wiz-text',
                width: '100%',
                html: 'Looking Glass can access videos stored locally on the glasses and display them for you.',
            }, {
                xtype: 'medwebcloud.lookingglasssetupwizard.voiceinstance',
                title: 'Show Videos',
                itemId: 'cmd-show-videos',
                description: 'Show the list of local videos',
                glasses: glasses,
                phrases: [
                    ['show videos', 70],
                ],
                listeners: {
                    'commandsChanged': function () {
                        me.raiseValuesChanged(this.getValues());
                    },
                },
            }, {
                xtype: 'medwebcloud.lookingglasssetupwizard.voiceinstance',
                title: 'Hide Videos',
                itemId: 'cmd-hide-videos',
                description: 'Hide the list of local videos',
                glasses: glasses,
                phrases: [
                    ['hide list', 70],
                ],
                listeners: {
                    'commandsChanged': function () {
                        me.raiseValuesChanged(this.getValues());
                    },
                },
            }, {
                xtype: 'medwebcloud.lookingglasssetupwizard.voiceinstance',
                title: 'Next Video',
                itemId: 'cmd-next-video',
                description: 'Select the next video in the list',
                glasses: glasses,
                phrases: [
                    ['down', 70],
                ],
                listeners: {
                    'commandsChanged': function () {
                        me.raiseValuesChanged(this.getValues());
                    },
                },
            }, {
                xtype: 'medwebcloud.lookingglasssetupwizard.voiceinstance',
                title: 'Previous Video',
                itemId: 'cmd-prev-video',
                description: 'Select the previous video in the list',
                glasses: glasses,
                phrases: [
                    ['up', 70],
                ],
                listeners: {
                    'commandsChanged': function () {
                        me.raiseValuesChanged(this.getValues());
                    },
                },
            }, {
                xtype: 'medwebcloud.lookingglasssetupwizard.voiceinstance',
                title: 'Play Video',
                itemId: 'cmd-play-video',
                description: 'Start the currently selected video',
                glasses: glasses,
                phrases: [
                    ['play video', 70],
                    ['play movie', 70],
                    ['start video', 70],
                    ['start movie', 70],
                    ['select', 70],
                ],
                listeners: {
                    'commandsChanged': function () {
                        me.raiseValuesChanged(this.getValues());
                    },
                },
            }, {
                xtype: 'medwebcloud.lookingglasssetupwizard.voiceinstance',
                title: 'Stop Video',
                itemId: 'cmd-stop-video',
                description: 'Stop the currently playing video and return to the video list.',
                glasses: glasses,
                phrases: [
                    ['stop video', 70],
                    ['stop movie', 70],
                ],
                listeners: {
                    'commandsChanged': function () {
                        me.raiseValuesChanged(this.getValues());
                    },
                },
            }, {
                xtype: 'medwebcloud.lookingglasssetupwizard.voiceinstance',
                title: 'Pause Video',
                itemId: 'cmd-pause-video',
                description: 'Pause the currently playing video.',
                glasses: glasses,
                phrases: [
                    ['pause video', 70],
                    ['pause movie', 70],
                ],
                listeners: {
                    'commandsChanged': function () {
                        me.raiseValuesChanged(this.getValues());
                    },
                },
            }, {
                xtype: 'medwebcloud.lookingglasssetupwizard.voiceinstance',
                title: 'Resume Video',
                itemId: 'cmd-resume-video',
                description: 'Resume the currently shown (but paused) video.',
                glasses: glasses,
                phrases: [
                    ['resume video', 70],
                    ['resume movie', 70],
                ],
                listeners: {
                    'commandsChanged': function () {
                        me.raiseValuesChanged(this.getValues());
                    },
                },
            }],
            listeners: {
                activate: function () {
                    var form = this;

                    var wiz = this.up('panel').up();
                    var voice = wiz.getData('VoiceCommands');

                    //Handle the actual form fields
                    this.getForm().getFields().each(function (itm) {
                        var data = wiz.getData(itm.getItemId());
                        if (data != null && data != undefined) {
                            itm.setValue(data);
                        }
                    });

                    //Handle the voice-instances
                    this.items.each(function (itm) {
                        var id = itm.getItemId();
                        if (id.substr(0, 4) === 'cmd-') {
                            if (voice && voice.length) {
                                var cmdId = id.substr(4);
                                var cmds = voice.filter(function (c) { return c.Key === cmdId; });
                                if (cmds && cmds.length && cmds[0].Phrases) {
                                    itm.setValues(cmds[0].Phrases);
                                    return;
                                }
                            }
                        }
                    });
                },
            }
        });

        this.callParent(arguments);
    },
    raiseValuesChanged: function () {
        var me = this;

        me.fireEvent('valuesChanged', me.getValues());
    },
    saveChanges: function () {
        var me = this;

        var items = (!!this.getFormItems) ? this.getFormItems() : this.items;
        items.each(function (itm) {
            if (itm.getValues) {
                me.saveVoiceCommand(itm.getItemId().substr(4));
            }
        });
    },
    saveVoiceCommand: function (cmd) {
        var me = this;
        if (!me.getRecord()['VoiceCommands']) me.getRecord()['VoiceCommands'] = [];

        var values = me.down('#cmd-' + cmd).getValues();

        var vc = me.getRecord()['VoiceCommands'].filter(function (vc) {
            return vc.Key === cmd;
        });
        if (!vc || !vc.length) {
            me.getRecord()['VoiceCommands'].push({
                Key: cmd,
                Phrases: values,
            });
        }
        else {
            vc[0].Phrases = values;
        }
    },
});