﻿Ext.define('MedwebCloud.view.VoiceConfigWizardPage', {
    xtype: 'medwebcloud.voiceconfigwizardpage',
    extend: 'Ext.Panel',

    dftConfig: {},

    initComponent: function () {
        var me = this;

        var COLUMN_PLACEHOLDER = '';

        Ext.define("VoiceCommandPhrase", {
            extend: 'Ext.data.Model',
            fields: ['Key', 'Command', 'Phrase', 'Sensitivity', 'IsPlaceholder'],
        });

        me.phraseStore = Ext.create('Ext.data.ArrayStore', {
            model: 'VoiceCommandPhrase',
            groupers: [{ property: 'Command', direction: 'ASC' }],
        });

        /*
        var dftConfig = Ext.create('MedwebCloud.store.GlassSetups', {
            pageSize: 1,
            filters: [{
                property: 'CompanyId',
                comparison: 'eq',
                value: null
            }, {
                property: 'Name',
                comparison: 'eq',
                value: 'Default',
            }],
            expand: 'VoiceCommands',
        });*/

        var cfg;

        //dftConfig.load(function () {
        //    console.log("Glass Setups Loaded: ", this);
        // cfg = this.getAt(0).copy().raw;
        cfg = me.valueSet;

        //console.log("Loading default configuration: ", cfg);

        var list = [];
        var tree = [];

        for (var i = 0; i < cfg.VoiceCommands.length; i++) {
            var cmd = cfg.VoiceCommands[i];

            var cp = list.filter(function (e) { return e.Key === cmd.Key; });
            if (cp.length === 0) {
                list.push({
                    Key: cmd.Key,
                    Command: MedwebCloud.getMsg('medweb.view.lookingglass.voice.' + cmd.Key),
                    Phrase: 'placeholder',
                    Sensitivity: 0,
                    IsPlaceholder: true,
                });
                var cmdnode = {
                    payload: {
                        Key: cmd.Key,
                        Name: MedwebCloud.getMsg('medweb.view.lookingglass.voice.' + cmd.Key),
                    },
                    children: [],
                    expanded: false,
                };
                tree.push(cmdnode);
            }

            if (cmd.Phrases.length) {
                for (var j = 0; j < cmd.Phrases.length; j++) {
                    var phrase = cmd.Phrases[j];

                    var cp = list.filter(function (e) { return e.Key === cmd.Key && e.Phrase === phrase.Phrase; });
                    if (cp.length) continue;

                    list.push({
                        Key: cmd.Key,
                        Command: MedwebCloud.getMsg('medweb.view.lookingglass.voice.' + cmd.Key),
                        Phrase: phrase.Phrase,
                        Sensitivity: phrase.Sensitivity,
                        IsPlaceholder: false,
                    });
                    cmdnode.children.push({
                        payload: {
                            Key: cmd.Key,
                            Phrase: phrase.Phrase,
                            Sensitivity: phrase.Sensitivity,
                        },
                        leaf: true,
                    });
                }
            }
        }

        me.phraseStore.clearData();
        me.phraseStore.loadData(list);
        //});

        var checkWords = function (str, callback) {
            var srvArgs = new fm.websync.serviceArgs("/lookingglass/config/checkwords", str);
            //var lm = new Ext.LoadMask(me.up('form').down('#syncBtn'), { msg: "..." });

            srvArgs.setOnSuccess(function (ss) {
                var response = JSON.parse(ss.getDataJson());

                if (response && response.length) {
                    var errWords = '';

                    response.forEach(function (phrase) {
                        if (phrase.hasOwnProperty('Errors') && phrase.Errors.length === 0) {
                            //;
                            console.log("Voice phrases updated successfully.")
                        }
                        else if (phrase.hasOwnProperty('Errors')) {
                            phrase.Errors.forEach(function (w) {
                                errWords += '<li>' + w + '</li>';
                            });
                        }
                    });

                    if (errWords.length) {
                        Ext.Msg.alert('Error in voice sync', 'The following words were not found in the command dictionary.  Please check their spelling or replace them with more common words:<br /><ul>' + errWords + '</ul>');
                        //me.up('form').down('#syncBtn').setDisabled(true);
                    } else {
                        //me.up('form').down('#syncBtn').setDisabled(false);
                    }
                }
            });
            srvArgs.setOnComplete(function () {
                //lm.hide();
            })
            MedwebCloud.websyncClient.service(srvArgs);
        }

        var rowEditing = Ext.create('Ext.grid.plugin.RowEditing', {
            clicksToEdit: 2,
            listeners: {
                edit: function () {
                    checkWords(JSON.stringify(me.getValues().map(function (r) { return r.Phrase; })));
                }
            }
        });

        var CommandGrouper = Ext.create('Ext.grid.feature.CustomGrouping', {
            enableGroupingMenu: false,
            startCollapsed: true,
            groupHeaderTpl: '{name}',
            showSummaryRow: true,
        });

        Ext.apply(this, {
            xtype: 'form',
            layout: {
                type: 'vbox',
                align: 'center',
            },
            items: [{
                xtype: 'fieldset',
                cls: 'wiz-fieldset',
                width: '90%',
                padding: '5 10 5 10',
                margin: '10 0 0 0',
                items: [{
                    xtype: 'panel',
                    layout: 'vbox',
                    width: '100%',
                    border: false,
                    items: [{
                        xtype: 'medwebcloud.togglebutton',
                        onState: me.valueSet.AreVoiceCommandsEnabled,
                        labelText: 'Voice Control:',
                        itemId: 'vc-toggle',
                        listeners: {
                            'valueChanged': function (newValue) {
                                me.down('#vf-toggle').setDisabled(!newValue);
                            }
                        }
                    }, {
                        xtype: 'displayfield',
                        padding: '7 0 0 0',
                        width: '100%',
                        cls: 'wiz-subtext',
                        value: 'This feature enables you to control Looking Glass through the use of voice commands, which you can customize below.',
                    }, {
                        xtype: 'medwebcloud.togglebutton',
                        onState: me.valueSet.IsVoiceFeedbackEnabled,
                        labelText: 'Voice Feedback:',
                        itemId: 'vf-toggle',
                    }, {
                        xtype: 'displayfield',
                        padding: '7 0 0 15',
                        width: '100%',
                        cls: 'wiz-subtext',
                        value: 'If this feature is on, when a voice command is detected Looking Glass will show a brief message notifying you the command was received.',
                    }],
                }],
            }, {
                xtype: 'grid',
                width: '90%',
                margin: '10 0 0 0',
                viewConfig: {
                    deferEmptyText: false,
                    emptyText: '<div class="empty-grid">No commands defined.</div>',
                    stripeRows: true
                },
                cls: 'cgrid-row',
                sortableColumns: false,
                fixed: true,
                selType: 'rowmodel',
                plugins: [rowEditing],
                height: 170,
                rowLines: false,
                columns: [
                    {
                        dataIndex: 'Phrase', header: 'Phrase', flex: 0.6, sortable: false, draggable: false, menuDisabled: true, editor: { xtype: 'textfield', allowBlank: false }, summaryType: 'count', summaryRenderer: function (v) {
                            return '<div class="phrase-adder" style="display: inline-block;">Add Phrase</div>';
                        }
                    },
                    { dataIndex: 'Sensitivity', header: 'Recognition Rate', sortable: false, draggable: false, menuDisabled: true, flex: 0.2, editor: { xtype: 'numberfield', allowBlank: false, minValue: 0, maxValue: 100 } },
                    {
                        xtype: 'actioncolumn',
                        flex: 0.2,
                        sortable: false,
                        draggable: false,
                        menuDisabled: true,
                        items: [{
                            iconCls: 'sync-glasses-flat-icon contain-image fat-action',
                            handler: function (grid, rowIndex, colIndex, icon, clickEvt, record, row) {
                                //console.log(grid, rowIndex, colIndex, icon, clickEvt, record, row);
                                var channel = me.glass;
                                //console.log("Syncing over channel", channel);
                                if (!me.waitingSyncs || me.waitingSyncs <= 0) {
                                    me.waitingSyncs = 0;

                                    var confArgs = new fm.websync.subscribeArgs([channel]);
                                    confArgs.setOnReceive(function (r) {
                                        if (!r.getWasSentByMe() && r.getDataJson()) {
                                            var dta = r.getDataJson();
                                            var obj = JSON.parse(dta);
                                            if (obj && obj.glasses && obj.glasses === 'RCV-DATA') {
                                                Ext.Msg.toast({ body: 'Sync complete.' });
                                                me.waitingSyncs--;
                                                if (me.waitingSyncs <= 0) {
                                                    MedwebCloud.websyncClient.unsubscribe(new fm.websync.unsubscribeArgs([channel]));
                                                }
                                            }
                                        }
                                    });
                                    MedwebCloud.websyncClient.subscribe(confArgs);
                                }
                                me.waitingSyncs++;
                                MedwebCloud.websyncClient.publish(new fm.websync.publishArgs(channel, JSON.stringify({
                                    Description: 'voice-update',
                                    Data: {
                                        Key: record.get('Key'),
                                        Phrases: me.getValues(record.get('Key'))
                                    }
                                })));
                            },
                            tooltip: 'Sync phrase to Glasses',
                        }, {
                            iconCls: 'phrase-remove-icon contain-image',
                            handler: function (grid, rowIndex, colIndex, icon, clickEvt, record, row) {
                                var yesRow = me.phraseStore.getByInternalId(Ext.get(row).getAttribute('data-recordId'));
                                me.phraseStore.remove(yesRow);
                            },
                            tooltip: 'Remove Phrase',
                        }],
                    }
                ],
                features: [CommandGrouper],
                viewConfig: {
                    getRowClass: function (rec, idx) {
                        if (rec.get('IsPlaceholder')) {
                            return 'hidden';
                        }
                    }
                },
                minHeight: 100,
                store: me.phraseStore,
                listeners: {
                    'requestAdd': function (group) {
                        //console.log("Request received: ", group);

                        var newItem = me.phraseStore.model.create();
                        newItem.store = me.phraseStore;

                        me.editorWindow = Ext.create('MedwebCloud.view.EditorWindow', {
                            formType: 'medwebcloud.voicephraseform2',
                            autoShow: true,
                            autoSave: true,
                            title: "New phrase for: " + group.Name,
                            record: newItem,
                            listeners: {
                                editComplete: function (values) {
                                    values['Key'] = group.children[0].get('Key');
                                    values['Command'] = group.name;
                                    values['IsPlaceholder'] = false;
                                    newItem.set(values);
                                    newItem.store.insert(0, newItem);
                                    newItem.store.sort();
                                }
                            }
                        });
                    }
                },
            }],
            listeners: {
                beforedeactivate: function () {
                    me.valueSet.IsVoiceFeedbackEnabled = me.down('#vf-toggle').getValue();
                    me.valueSet.AreVoiceCommandsEnabled = me.down('#vc-toggle').getValue();

                    var key = '';
                    var cmd = {};
                    me.valueSet.VoiceCommands = [];
                    me.phraseStore.each(function (rec) {
                        var vc = me.valueSet.VoiceCommands.filter(function (e) { return e.Key === rec.get('Key'); });
                        if (vc.length) {
                            cmd = vc[0];
                        } else
                            //if (rec.get('Key') != key)
                        {
                            cmd = {};
                            key = rec.get('Key');
                            cmd.Key = key;
                            cmd.Command = rec.get('Command');
                            cmd.Phrases = [];

                            me.valueSet.VoiceCommands.push(cmd);
                        }

                        if (!rec.get('IsPlaceholder')) {
                            var cp = cmd.Phrases.filter(function (e) { return e.Phrase === rec.get('Phrase'); });
                            if (cp.length) return;

                            cmd.Phrases.push({ 'Phrase': rec.get('Phrase'), 'Sensitivity': rec.get('Sensitivity') });
                        }
                        //console.log("Added VC", JSON.stringify(rec.raw), JSON.stringify(me.valueSet.VoiceCommands));
                    });
                },
            },
        });

        me.callParent(arguments);
    },
    getValues: function (key) {
        var me = this;

        var phraseArray = [];

        me.phraseStore.each(function (rec) {
            if (!rec.get('IsPlaceholder')) {
                if (key && rec.get('Key') === key) {
                    phraseArray.push({
                        "Key": rec.get("Key"),
                        "Phrase": rec.get("Phrase"),
                        "Sensitivity": rec.get("Sensitivity"),
                    });
                }
            }
        });

        return phraseArray;
    },
});