﻿Ext.define('MedwebCloud.view.MapPanel', {
    extend: 'Ext.panel.Panel',
    requires: [],
    xtype: 'MedwebCloud.mappanel',
    autoLocate: true,
    map: null,
    layout: 'fit',
    initComponent: function () {
        var me = this;

        me.pointsCache = [];
        me.markersCache = [];

        var reset = function () {
            if (me.map) {
                var center = me.map.getCenter();
                if (center) {
                    google.maps.event.trigger(me.map, 'resize');
                    me.map.setCenter(center);
                }
            }
        };

        this.on('afterrender', function () {
            if (!window.google) {
                me.update('Mapping service is currently unavailable.');
                return;
            }
            var myOptions = {
                zoom: 15,
                //center: latlng,
                mapTypeControl: false,
                navigationControlOptions: { style: google.maps.NavigationControlStyle.SMALL },
                mapTypeId: google.maps.MapTypeId.ROADMAP
            };
            me.map = new google.maps.Map(me.getLayout().getRenderTarget().dom, myOptions);

            for (var i = 0; i < me.pointsCache.length; i++) {
                me.pointsCache[i]();
            }

            MedwebCloud.getLocation(function (position) {
                var latlng = new google.maps.LatLng(position.coords.latitude, position.coords.longitude);
                me.map.setCenter(latlng);

                if (me.autoLocate) {
                    me.addPoint(position.coords.latitude, position.coords.longitude, "Me", {
                        icon: MedwebCloud.getGravatarUrl(MedwebCloud.userProfile.Username, 32),
                        autoOpenInfoWindow: false
                    });
                }
            }, function (message) {
                //Ext.Msg.alert('Error', message);
            });

            window.setTimeout(reset, 100);
        });

        this.on('activate', reset);
        this.on('resize', reset);

        this.callParent(arguments);
    },
    addPoint: function (lat, lng, title, o) {
        var me = this;

        var latlng = new google.maps.LatLng(lat, lng);
        var infowindow = new google.maps.InfoWindow({
            content: '<div style="padding:10px">' + title + '</div>'
        });
        var marker = new google.maps.Marker(Ext.apply({
            position: latlng,
            map: me.map,
            title: title
        }, o || {}));

        google.maps.event.addListener(marker, 'click', function () {
            if (me._lastClickedMarker) {
                me._lastClickedMarker.setZIndex();
            }

            infowindow.open(me.map, marker);
            marker.setZIndex(google.maps.Marker.MAX_ZINDEX + 1);

            me._lastClickedMarker = marker;

            if (o && o.handler) {
                o.handler();
            }
        });

        if (o.moveToPoint && me.map) {
            me.map.setCenter(latlng);
        }

        return marker;
    }
});