﻿Axis = function (baseUrl, username, password) {
    this.baseUrl = baseUrl;
    this.username = username;
    this.password = password;
    if (username && password) {
        this.unauthenticatedBaseUrl = baseUrl;
        this.baseUrl = baseUrl.replace(/(https?):\/\/(.+?)/, '$1://' + this.username + ':' + this.password + '@$2');
    }
    if (this.baseUrl.charAt(this.baseUrl.length - 1) == '/') {
        this.baseUrl = this.baseUrl.substring(0, this.baseUrl.length - 1);
    }
    this._handlers = [];
};
Axis.prototype.connect = function (callback) {
    // create a frame to trigger the auth
    var me = this, failedTimer = 0;
    var frame = document.createElement('frame');

    $(frame).load(function () {
        window.clearTimeout(failedTimer);
        me.ready = true;

        frame.src = 'about:blank';
        document.body.removeChild(frame);
        me.fire('connect:success', {});

        me.queryTimer = window.setInterval(function () {
            me.query();
        }, 500); // no more than 2x a second

        if (callback) {
            callback();
        }
    });
    frame.src = this.baseUrl + '/view/viewer_index.shtml?_dt=' + (new Date().getTime());
    document.body.appendChild(frame);
    failedTimer = window.setTimeout(function () {
        me.fire('connect:failure');
    }, 10000);
};

Axis.prototype.disconnect = function () {
    window.clearInterval(this.queryTimer);
};
Axis.prototype.on = function (evt, fn) {
    this._handlers[evt] = this._handlers[evt] || [];
    this._handlers[evt].push(fn);
};
Axis.prototype.fire = function (evt, args) {
    var handlers = this._handlers[evt] || [];
    for (var i = 0; i < handlers.length; i++) {
        handlers[i](args);
    }
};
Axis.prototype.getVideoUrl = function (format) {
    switch (format) {
        case 'mjpg':
        case 'mjpeg':
        default:
            return this.baseUrl + '/mjpg/video.mjpg?_dt=' + (new Date()).getTime();
    }
};
Axis.prototype.moveTo = function (x, y, w, h) {
    var go = this.baseUrl + '/axis-cgi/com/ptz.cgi?camera=1&speed=100&center=' + x + ',' + y + '&imagewidth=' + w + '&imageheight=' + h + '&imagerotation=0&timestamp=' + (new Date()).getTime();
    var img = new Image();
    img.src = go;
};
Axis.prototype.getAppUrl = function () {
    return this.unauthenticatedBaseUrl + '/view/viewer_index.shtml?_dt=' + (new Date().getTime())
};
Axis.prototype.home = function () {
    var go = this.baseUrl + '/axis-cgi/com/ptz.cgi?camera=1&move=home&_dt=' + (new Date()).getTime();
    var img = new Image();
    img.src = go;
};
Axis.prototype.status = function () {
    //TODO: implement this one...somehow. cross domain, so maybe an iframe or something?
    //$.get('/axis-cgi/com/ptz.cgi?query=position,limits&camera=1&html=no&imagerotation=0&timestamp=1435954381187
}
Axis.prototype.focusIn = function () {
    this.focus(20);
}
Axis.prototype.focusOut = function () {
    this.focus(-20);
}
Axis.prototype.focus = function (x) {
    this.execute('continuousfocusmove', x);
}
Axis.prototype.zoomWide = function () {
    this.zoom(-1000);
};
Axis.prototype.zoomTele = function () {
    this.zoom(1000);
};

Axis.prototype.left = function () {
    this.pan(-25);
};
Axis.prototype.right = function () {
    this.pan(25);
};
Axis.prototype.up = function () {
    this.tilt(25);
};
Axis.prototype.down = function () {
    this.tilt(-25);
};
Axis.prototype.pan = function (x) {
    this._applyAction('pan', x, true);
};
Axis.prototype.tilt = function (x) {
    this._applyAction('tilt', x, true);
};
Axis.prototype.zoom = function (x) {
    this._applyAction('zoom', x, false);
};
Axis.prototype._applyAction = function (action, x, circular) {
    if (!this._data) {
        return;
    }

    var prop = action;
    var propCap = prop.charAt(0).toUpperCase() + prop.substring(1);

    var min = this._data['Min' + propCap];
    var max = this._data['Max' + propCap]

    var v = this._data[prop] + x;
    if (circular) {
        if (v < min) {
            v = max + (v % min);
        } else if (v > max) {
            v = min + (v % max);
        }
    } else {
        if (v < min) {
            v = min;
        } else if (v > max) {
            v = max;
        }
    }
    this.executeOne(prop, v);
};
Axis.prototype.panTilt = function (x, y) {
    this.execute('continuouspantiltmove', x, y);
};
Axis.prototype.query = function (x, y) {
    var url = MedwebCloud.proxy(this.baseUrl + '/axis-cgi/com/ptz.cgi?query=position,limits&camera=1&html=no&imagerotation=0');
    var me = this;
    if (me._querying) {
        return;
    }
    me._querying = true;
    /*
    Sample:

pan=-109.7906
tilt=-38.2875
zoom=1
iris=1
focus=9999
autofocus=off
MinPan=-180
MaxPan=180
MinTilt=-180
MaxTilt=0
MinZoom=1
MaxZoom=19999
MinIris=1
MaxIris=9999
MinFocus=1
MaxFocus=9999
MinFieldAngle=44
MaxFieldAngle=516
MinBrightness=1
MaxBrightness=9999
    */
    $.ajax({
        url: url,
        success: function (data) {
            var params = data.split('\n');
            me._data = {};
            for (var i = 0; i < params.length; i++) {
                var pairs = params[i].split('=');
                me._data[pairs[0]] = parseInt(pairs[1], 10);
            }
            me._querying = false;
        },
        failure: function () {
            // TODO
        }
    })
};
Axis.prototype.executeOne = function (action, x) {
    var go = this.baseUrl + '/axis-cgi/com/ptz.cgi?camera=1&' + action + '=' + x + '&imagerotation=0&_dt=' + (new Date()).getTime();
    var img = new Image();
    img.src = go;
};
Axis.prototype.execute = function (action, x, y) {
    var paramGo = '', paramStop = '';
    if (arguments.length == 3) {
        paramGo = x + ',' + y;
        paramStop = '0,0';
    } else if (arguments.length == 2) {
        paramGo = x;
        paramStop = '0';
    }
    var go = this.baseUrl + '/axis-cgi/com/ptz.cgi?camera=1&' + action + '=' + paramGo + '&imagerotation=0&_dt=' + (new Date()).getTime();
    var stop = this.baseUrl + '/axis-cgi/com/ptz.cgi?camera=1&' + action + '=' + paramStop + '&imagerotation=0&_dt=' + (new Date()).getTime();
    var img = new Image();
    img.src = go;
    window.setTimeout(function () {
        img.src = stop;
    }, 500);
};

Ext.define('MedwebCloud.view.RemoteCamerasPanel', {
    extend: 'Ext.panel.Panel',
    xtype: 'medwebcloud.remotecameraspanel',
    record: null,
    client: null,
    getRemoteCameraFilterer: function () {
        var companyId;
        if (!!this.record) {
            var creator = this.record.get('User');
            companyId = creator.CompanyId;
        } else {
            companyId = MedwebCloud.userProfile.CompanyId;
        }
        return [{
            property: 'CompanyId',
            comparison: 'eq',
            value: companyId
        }, {
            property: 'IsDeleted',
            comparison: 'ne',
            value: true
        }];
    },
    initComponent: function () {
        var me = this;
        var camerasLabel = MedwebCloud.getMsg('medweb.view.sessionpanel.button.remotecameras.label');
        var store = Ext.create('MedwebCloud.store.RemoteCameras', {
            autoLoad: true,
            filters: me.getRemoteCameraFilterer()
        });
        Ext.apply(this, {
            layout: 'fit',
            tbar: {
                items: [{
                    width: '100%',
                    xtype: 'combo',
                    fieldLabel: camerasLabel,
                    name: 'Camera',
                    queryMode: 'local',
                    store: store,
                    displayField: 'Name',
                    valueField: 'Id',
                    editable: false,
                    autoSelect: true,
                    forceSelection: true,
                    listeners: {
                        select: function (cbo, records, opts) {
                            if (records.length > 0) {
                                var record = records[0];
                                me.showCamera(record);
                            }
                        }
                    }
                }]
            }
        });
        this.callParent(arguments);

        this.on('show', function () {
            console.log('show')
            if (store.count() > 0 && !this._shownOnce) {
                console.log('connecting');
                this._shownOnce = true;
                var record = store.getAt(0);
                me.showCamera(record);
                window.setTimeout(function () {
                    me.down('combo').select(record);
                }, 100);
            }
        });
    },
    showCamera: function (record) {
        var me = this;

        this.removeAll();

        var axis = new Axis(record.get('Url'), record.get('Username'), record.get('Password'));
        window.axis = axis;
        var controls = me.buildControls(axis)

        this.mask('Connecting to camera...');

        axis.on('connect:failure', function () {
            me.unmask();
        });

        axis.on('connect:success', function () {
            me.unmask();

            var cameraCmp = me.add(new Ext.Component({
                layout: 'fit',
                cls: 'ip-camera-container',
                html: '<img class="camera ip-camera axis-camera" style="cursor:crosshair" src="' + axis.getVideoUrl('mjpg') + '" />',
                listeners: {
                    'afterrender': function () {
                        window.setTimeout(function () {
                            console.log('showing controls', controls);
                            controls.show();
                            controls.anchorTo(cameraCmp, 'br-br', [-5, -5]);

                            var img = $(cameraCmp.el.dom).find('img');

                            $(img).on('click', function (e) {
                                var offset = $(this).offset();
                                var x = e.pageX - offset.left;
                                var y = e.pageY - offset.top;
                                axis.moveTo(x, y, $(this).width(), $(this).height());
                            });

                            // keep controls in sync
                            me.on('hide', function () {
                                controls.hide();
                            });
                            me.on('show', function () {
                                controls.show();
                            });

                            var wasShowing = false;
                            Ext.eventBus.on('section:collage:deactivated', function () {
                                // if collage is deactivated, hide the controls if they're showing
                                if (controls.isVisible()) {
                                    wasShowing = true;
                                    controls.hide();
                                } else {
                                    wasShowing = false;
                                }
                            });

                            Ext.eventBus.on('section:collage:activated', function () {
                                // if collage is deactivated, hide the controls if they're showing
                                if (wasShowing) {
                                    controls.show();
                                }
                            });

                            // kill the camera controls if the camera goes away
                            cameraCmp.on('destroy', function () {
                                console.log('destroying controls');
                                controls.destroy();
                                axis.disconnect();
                            });
                        }, 10);
                    }
                }
            }));
        });

        axis.connect();
    },

    buildControls: function (axis) {
        var btnZoomIn = MedwebCloud.getMsg('medweb.view.videopanel.button.zoomin.label');
        var btnZoomOut = MedwebCloud.getMsg('medweb.view.videopanel.button.zoomout.label');
        var btnZoomReset = MedwebCloud.getMsg('medweb.view.videopanel.button.zoomreset.label');

        return new Ext.Window({
            //title: 'Controls',
            header: false,
            closable: false,
            resizable: false,
            layout: {
                type: 'table',
                columns: 3
            },
            defaults: {
                width: 24,
                border: false,
                scale: 'medium',
                style: 'background:none',
            },
            items: [{
                // nada
            }, {
                // nada
            }, {
                xtype: 'button',
                icon: '/images/bullet_go.png',
                handler: function () {
                    window.open(axis.getAppUrl(), '_blank');
                }
            }, {
                // nothing
            }, {
                xtype: 'button',
                //text: 'Up'
                iconCls: 'arrow-up',
                handler: function () {
                    axis.up();
                }
            }, {
                // nothing
            }, {
                xtype: 'button',
                //text: 'Left'
                iconCls: 'arrow-left',
                handler: function () {
                    axis.left();
                }
            }, {
                xtype: 'button',
                //text: 'H'
                iconCls: 'home',
                handler: function () {
                    axis.home();
                }
            }, {
                region: 'west',
                xtype: 'button',
                //text: 'Right',
                iconCls: 'arrow-right',
                handler: function () {
                    axis.right();
                }
            }, {
                // nothing
            }, {
                region: 'south',
                xtype: 'button',
                iconCls: 'arrow-down',
                handler: function () {
                    axis.down();
                }
            }, {
                // nothing
            }, {
                xtype: 'button',
                //text: btnZoomOut,
                iconCls: 'zoom-out',
                handler: function () {
                    axis.zoomWide();
                }
            }, {
                // nothing
            }, {
                xtype: 'button',
                //text: btnZoomIn,
                iconCls: 'zoom-in',
                handler: function () {
                    axis.zoomTele();
                }
            }]
        });
    }
});