﻿Ext.define('MedwebCloud.view.ScheduleSessionPanel', {
    extend: 'Ext.Panel',
    xtype: 'MedwebCloud.schedulesessionpanel',
    requires: ['MedwebCloud.view.ContactsGrid'],
    record: null,
    initComponent: function () {
        var me = this;

        //trying to get the default to be the NEXT 5 minutes appoximaly
        var ms5Min = 300000;
        var intIn5Min = ((new Date()).valueOf() + 1.5 * ms5Min);
        var in5Min = new Date(intIn5Min - (intIn5Min % ms5Min));

        var startDateValue = in5Min, startTimeValue = in5Min;
        me.startDateTime = function () {
            return new Date(startDateValue.getFullYear(), startDateValue.getMonth(), startDateValue.getDate(), startTimeValue.getHours(), startTimeValue.getMinutes(), 0, 0);
        };

        var lblStartDate = MedwebCloud.getMsg('medweb.view.schedulesessionpanel.startdate.label');
        var lblStartTime = MedwebCloud.getMsg('medweb.view.schedulesessionpanel.starttime.label');

        var schedule = new Ext.FormPanel({
            frame: false,
            height: 50,
            region: 'north',
            labelAlign: 'left',
            labelStyle: 'font-weight:bold;',
            items: [{
                layout: 'column',
                border: false,
                items: [{ // column #1
                    columnWidth: .5,
                    padding: '10 5 10 10',
                    layout: 'form',
                    border: false,
                    items: [{
                        xtype: 'datefield',
                        fieldLabel: lblStartDate,
                        name: 'startDate',
                        minValue: new Date(),
                        value: in5Min,
                        listeners: {
                            change: function (el, newDate, oldDate) {
                                startDateValue = newDate;
                            }
                        }
                    }] // close items for first column
                },
                {
                    columnWidth: .5,
                    layout: 'form',
                    border: false,
                    padding: '10 10 10 5',
                    items: [{
                        xtype: 'timefield',
                        fieldLabel: lblStartTime,
                        name: 'startTime',
                        value: in5Min,
                        increment: 15,
                        listeners: {
                            change: function (el, newDate, oldDate) {
                                startTimeValue = newDate;
                            }
                        }
                    }] // close items for second column
                }]
            }]
        });

        var inviteSelections = {};
        var inviteSelectionCount = 0;

        var setButtonEnable = function () {
            var win = me.up(),
                toolbar = win.dockedItems.last(),
                button = toolbar.getComponent('scheduleButton');
            if (inviteSelectionCount > 0) {
                button.enable();
            } else {
                button.disable();
            }
        };
        var selectListener = function (model, record, index) {
            var id = record.get("Id");
            inviteSelections[id] = record;
            inviteSelectionCount += 1;
            setButtonEnable()
        };
        var deselectListener = function (model, record, index) {
            var id = record.get("Id");
            delete inviteSelections[id];
            inviteSelectionCount -= 1;
            setButtonEnable();
        };

        var personalSelectModel = Ext.create('Ext.selection.CheckboxModel', {
            listeners: {
                deselect: deselectListener,
                select: selectListener
            }
        });
        var patientSelectModel = Ext.create('Ext.selection.CheckboxModel', {
            listeners: {
                deselect: deselectListener,
                select: selectListener
            }
        });
        var networkSelectModel = Ext.create('Ext.selection.CheckboxModel', {
            listeners: {
                deselect: deselectListener,
                select: selectListener
            }
        });
        var globalSelectModel = Ext.create('Ext.selection.CheckboxModel', {
            listeners: {
                deselect: deselectListener,
                select: selectListener
            }
        });

        var tabPatient = MedwebCloud.getMsg('medweb.view.schedulesessionpanel.tab.patient.title');
        var tabPersonal = MedwebCloud.getMsg('medweb.view.schedulesessionpanel.tab.personal.title');
        var tabInstitutional = MedwebCloud.getMsg('medweb.view.schedulesessionpanel.tab.institutional.title');
        var tabGeneral = MedwebCloud.getMsg('medweb.view.schedulesessionpanel.tab.global.title');

        var btnAdd = MedwebCloud.getMsg('medweb.view.schedulesessionpanel.personal.button.add.title');

        var invites = new Ext.TabPanel({
            border: false,
            region: 'center',
            defaults: {
                autoScroll: true
            },
            items: [{
                title: tabPatient,
                layout: 'fit',
                xtype: 'MedwebCloud.contactsgrid',
                selModel: patientSelectModel,
                hidden: !this.patientEmail,
                patientData: MedwebCloud.PatientData
            }, {
                title: tabPersonal,
                layout: 'fit',
                xtype: 'MedwebCloud.contactsgrid',
                itemId: 'personalGrid',
                selModel: personalSelectModel,
                contactType: 1,
                canEdit: true,
                tbar: {
                    items: [{
                        text: btnAdd,
                        iconCls: 'addContactIcon',
                        handler: function () {
                            MedwebCloud.createContact(function (values) {
                                inviteSelections[values.Email] = values;
                            });
                        }
                    }]
                }
            },
            {
                title: tabInstitutional,
                layout: 'fit',
                xtype: 'MedwebCloud.contactsgrid',
                selModel: networkSelectModel,
                contactType: 2
            }]
        });

        Ext.apply(this, {
            border: false,
            layout: 'border',
            items: [schedule, invites]
        });

        Ext.apply(this, {
            populatePatient: function (name, email) {
            },
            createSession: function () {
                if (inviteSelectionCount <= 0) {
                    return;
                }
                var session = Ext.create('MedwebCloud.model.CreateSessionModel', {
                    StartTime: me.startDateTime(),
                });
                var sessionInvites = session.Invitations();

                for (key in inviteSelections) {
                    if (inviteSelections.hasOwnProperty(key)) {
                        var invite = inviteSelections[key];
                        if (!!invite.Email) {
                            console.log("using invite");
                            console.log(invite);
                            sessionInvites.add(invite);
                        } else {
                            console.log("constructing invite");
                            console.log(invite);
                            var details = invite.get('ContactDetails');
                            //email only gets set from incoming patient othewise fetch from details
                            var contact = {
                                DisplayName: invite.get('DisplayName'),
                                Email: invite.get('Email')
                            }
                            if (details) {
                                for (var i = details.length - 1; i >= 0; i--) {
                                    var detail = details[i];
                                    if (detail.TypeName.toLowerCase() == "email") {
                                        contact.Email = detail.Value;
                                        break;
                                    }
                                }
                            }
                            sessionInvites.add(contact);
                        }
                    }
                }

                session.save({
                    failure: function () {
                        Ext.Msg.alert(
                            MedwebCloud.getMsg('medweb.view.schedulesessionpanel.action.add.failure.title'),
                            MedwebCloud.getMsg('medweb.view.schedulesessionpanel.action.add.failure.message'));
                    },
                    success: function (s) {
                        Ext.Msg.toast({
                            body: MedwebCloud.getMsg('medweb.view.schedulesessionpanel.action.add.success.message')
                        });
                    }
                });
            },
        }
       )

        this.callParent(arguments);
    },

    inviteSelected: function () {
        if (!this.selectedRecord) {
            return;
        }
        this.inviteContact(this.selectedRecord);
    },
    inviteContact: function (contactRecord) {
        var me = this;
        this.showEmails(contactRecord, function (displayName, email) {
            var invitation = Ext.create('MedwebCloud.model.InvitationModel', {
                SessionId: me.record.get('Id'),
                DisplayName: displayName,
                Email: email
            });
            invitation.save({
                failure: function () {
                    Ext.Msg.alert(
                        MedwebCloud.getMsg('medweb.view.schedulesessionpanel.action.add.failure.title'),
                        MedwebCloud.getMsg('medweb.view.schedulesessionpanel.action.add.failure.message'));
                },
                success: function (s) {
                    Ext.Msg.toast({
                        body: MedwebCloud.getMsg('medweb.view.schedulesessionpanel.action.add.success.message')
                    });
                }
            });
            me.fireEvent('invite');
        });
    },

    showEmails: function (contactRecord, callback) {
        var details = contactRecord.data.ContactDetails;
        if (!details) {
            Ext.Msg.alert(MedwebCloud.getMsg('medweb.view.schedulesessionpanel.action.show.failure.message'));
            return;
        }

        var detailsStore = Ext.create('Ext.data.Store', {
            filters: [{
                property: 'TypeName',
                value: 'email'
            }],
            fields: ['TypeName', 'Value', 'Level1Name', 'Level2Name'],
            data: {
                'items': details
            },
            proxy: {
                type: 'memory',
                reader: {
                    type: 'json',
                    root: 'items'
                }
            }
        });

        if (detailsStore.getCount() > 1) {
            var btnOk = MedwebCloud.getMsg('medweb.view.schedulesessionpanel.action.show.button.positive.label');
            var btnCancel = MedwebCloud.getMsg('medweb.view.schedulesessionpanel.action.show.button.negative.label');
            var title = MedwebCloud.getMsg('medweb.view.schedulesessionpanel.action.show.window.title');
            var email = MedwebCloud.getMsg('medweb.user.email.label')

            var w = new Ext.Window({
                modal: true,
                title: title,
                height: 300,
                width: 300,
                layout: 'fit',
                border: false,
                items: [{
                    xtype: 'grid',
                    store: detailsStore,
                    columns: [{
                        text: email,
                        dataIndex: 'Value',
                        flex: 1
                    }],
                    listeners: {
                        celldblclick: function (grid, td, cellIndex, record, tr, rowIndex, e, eOpts) {
                            callback(contactRecord.get('DisplayName'), record.get('Value'));
                            w.close();
                        }
                    }
                }],
                buttons: [{
                    text: btnOk,
                    handler: function () {
                        var records = w.down('.grid').getSelectionModel().getSelection();
                        if (!records || records.length == 0) {
                            return;
                        }
                        w.close();
                        callback(contactRecord.get('DisplayName'), records[0].get('Value'));
                    }
                },
                {
                    text: btnCancel,
                    handler: function () {
                        w.close();
                    }
                }]
            });
            w.show();
        } else {
            callback(contactRecord.get('DisplayName'), detailsStore.getAt(0).get('Value'));
        }
    }
});