﻿Ext.define('MedwebCloud.view.ScreenSharePanel', {
    extend: 'Ext.panel.Panel',
    xtype: 'medwebcloud.screensharepanel',
    record: null,
    client: null,

    initComponent: function () {
        var me = this;

        var supported = fm.icelink.webrtc.screenshare.supported;
        var installed = fm.icelink.webrtc.screenshare.extensionInstalled;
        var message = '';
        var disabled = false;

        var labels = {
            sharescreen: MedwebCloud.getMsg('medweb.view.waitingroom.sharescreen.label'),
            sharescreenadd: MedwebCloud.getMsg('medweb.view.waitingroom.sharescreenadd.label'),
            sharescreenunsupported: MedwebCloud.getMsg('medweb.view.waitingroom.sharescreenunsupported.label'),
            sharescreeninstalled: MedwebCloud.getMsg('medweb.view.waitingroom.sharescreeninstalled.label'),
        };

        this.notifyChannel = '/collage/' + this.record.get('Id') + '/screenshare/nofify';

        if (supported && !installed) {
            // almost
            message = labels.sharescreenadd;
        } else if (supported && installed) {
            // go!
            message = labels.sharescreen
        } else {
            // no dice
            message = labels.sharescreenadd;
            disabled = true;
        }

        Ext.apply(this, {
            layout: 'card',
            items: [{
                itemId: 'noShare',
                xtype: 'panel',
                layout: {
                    type: 'vbox',
                    align: 'center',
                    pack: 'center'
                },
                items: [{
                    xtype: 'button',
                    text: message,
                    disabled: disabled,
                    scale: 'large',
                    icon: 'images/monitor32.png',
                    handler: function (btn) {
                        if (supported && !installed) {
                            // almost, do the install first
                            /*me.performInstall(function () {
                                // TODO: figure out why the state is bad on the initial install....reloading works...
                                // me.showScreenShare();

                                // force a refresh for now
                                window.location.reload();
                            });*/
                            window.open('https://chrome.google.com/webstore/detail/bkfnkdicnbiiaojkichfodjjbmielbnm/');
                            fm.icelink.webrtc.screenshare.extensionCallback = function () {
                                supported = true;
                                installed = true;
                            }
                            btn.setText(labels.sharescreeninstalled);
                        } else if (supported && installed) {
                            // go!
                            me.showScreenShare();
                        } else {
                            // no dice
                            window.open('http://www.google.com/chrome');
                        }
                    }
                }]
            }, {
                xtype: 'panel',
                layout: 'fit',
                itemId: 'shareContainer'
            }]
        });
        this.callParent(arguments);

        this.listen();
    },
    performInstall: function (success, failure) {
        var links = document.getElementsByName('link');
        var url = $('link[rel=chrome-webstore-item]').attr('href')
        chrome.webstore.install(url, function () {
            // success
            if (success) {
                success();
            }
        }, function () {
            // failure
            if (failure) {
                failure();
            }
        });
    },
    showScreenShare: function () {
        var me = this, vid = this.getVideoPanel(false);

        var container = me.down('#shareContainer');
        container.removeAll();
        container.add(vid);
        me.getLayout().setActiveItem(1);

        this.client.publish({
            channel: this.notifyChannel,
            data: {
                action: 'start'
            }
        });
        this.sharing = true;
    },
    getVideoPanel: function (readOnly) {
        var me = this,
            vid = null,
            stopSharing = function () {
                me.getLayout().setActiveItem(0);
                vid.destroy();

                me.client.publish({
                    channel: me.notifyChannel,
                    data: {
                        action: 'stop'
                    }
                });
                me.sharing = true;
            };

        vid = new MedwebCloud.view.VideoPanel({
            client: this.client,
            record: this.record,
            labelRemote: false,
            uniqueId: this.record.get('Id'),
            screen: !readOnly,
            showLocal: !readOnly,
            showRemote: readOnly,
            border: false,
            audio: false,
            options: { audio: false },
            waitingText: 'Waiting for screen...',
            altWaitingText: 'Waiting for screen...',
            subchannel: 'screenshare',
            actionButtons: [{
                text: 'Stop',
                itemId: 'stop',
                iconCls: 'endIcon',
                handler: stopSharing
            }]
        });

        fm.icelink.webrtc.screenshare.screenShareCancelCallback = function () {
            stopSharing();
        }

        return vid;
    },
    listen: function () {
        var me = this;

        this.client.subscribe({
            channel: this.notifyChannel,
            onReceive: function (args) {
                if (args.getWasSentByMe()) {
                    return;
                }
                if (args.getData().action == 'start') {
                    if (!me.listening) {
                        // join the conference and disable screen sharing options
                        console.log('remote started')

                        var vid = me.getVideoPanel(true);
                        var container = me.down('#shareContainer');
                        container.removeAll();
                        container.add(vid);
                        me.getLayout().setActiveItem(1);
                        me.listening = true;

                        // hide the stop button
                        vid.down('#stop').hide();
                    }
                } else if (args.getData().action == 'stop') {
                    if (me.listening) {
                        // leave the conference
                        console.log('remote stopped');

                        var container = me.down('#shareContainer');
                        container.removeAll();
                        me.getLayout().setActiveItem(0);
                        me.listening = false;
                    }
                } else if (args.getData().action == 'query') {
                    if (me.sharing) {
                        me.client.publish({
                            channel: me.notifyChannel,
                            data: { action: 'start' }
                        });
                    }
                }
            }
        });

        // publish out a query to ask if anyone else is using the screen share
        me.client.publish({
            channel: this.notifyChannel,
            data: { action: 'query' }
        });
    }
});