﻿Ext.define('MedwebCloud.view.BaseEditorWrapper', {
    extend: 'Ext.panel.Panel',
    xtype: 'medwebcloud.baseeditorwrapper',
    
    itemName: 'Record',
    itemsName:null, //Plural Name for Records
    gridTitle: 'Records',
    storeType: null,

    formType: null,
    editForm: null,
    addForm: null,

    filters: [],
    expands: null,
    sorters: [],

    addEnabled: true,
    editEnabled: true,
    removeEnabled: true,
    restoreEnabled: false,

    beforeAdd: null,

    customArgs: null,
    
    initComponent: function () {

        var me = this;

        if (!me.itemsName) me.itemName; 

        if (!me.addTitle) me.addTitle = me.getMsg('titles.add', me.itemName);
        if (!me.editTitle) me.editTitle = me.getMsg('titles.edit', me.itemName);

        if (!me.showdeletedTitle) me.showdeletedTitle = me.getMsg('titles.showdeleted', me.itemName);
        if (!me.restoreTitle) me.restoreTitle = me.getMsg('titles.restore', me.itemsName);

        

        if (me.storeType == null) {
            console.log("A store type is required.");
            throw "InvalidArgumentException";
        }

        if (me.addEnabled && (me.addForm == null && me.formType == null)) {
            console.log("If adding is enabled, an appropriate form must be defined.");
            throw new "InvalidArgumentException";

        }

        if (me.editEnabled && (me.editForm == null && me.formType == null)) {
            console.log("If editing is enabled, an appropriate form must be defined.");
            throw new "InvalidArgumentException";
        }

        this.callParent(arguments);
    },
    getFormValues: function (form) {
        var me = this;

        var values = {};

        if (form.isValid()) {
            appendValues(form, values);
        }

        return values;

        //Tail recursive function to handle stacked fieldsets
        // Both fieldsets AND forms allow each()ing over their items list
        function appendValues(fieldset, valueList)
        {
            fieldset.items.each(function (f) {
                switch (f.getXType())
                {
                    case 'fieldset':
                        appendValues(f, valueList);
                        break;
                    case 'tabpanel':
                    case 'grid':
                        //Blacklist of types to skip
                        break;

                        //A little oddly, default is the expected case.
                        // Might be a good idea to swap this to a whitelist later, 
                        //   but the list of possible good (and bad!) inputs is theoretically infinite.
                    default:
                        valueList[f.getName()] = f.getValue();
                        break;
                }
            });
        }
    },
    getMsg: function (subkey) {
        var fullKey = 'medweb.view.shared.editors.' + subkey;
        arguments[0] = fullKey;
        return window.MedwebCloud.getMsg.apply(this, arguments);
    },
    onOperationComplete: function (onSuccess) {
        var me = this;

        me.getEl().mask(me.getMsg('messages.saving'));
        me.store.sync({
            success: function () {
                if (typeof (onSuccess) === 'function') {
                    onSuccess();
                }
                me.reportSaveSuccess();
                me.store.load();
            },
            failure: function (batch) {
                me.store.rejectChanges();
                me.reportSaveFailure(batch);
            },
            callback: function () {
                me.getEl().unmask();
            }
        });
    },
    onOperationFailed: function () {
        me.reportSaveFailure({});
    },
    reportSaveSuccess: function (batch) {
        var me = this;
        Ext.Msg.toast({ body: me.getMsg('messages.saved') });
    },
    reportSaveFailure: function (batch) {
        var me = this;

        var error = me.getMsg('errors.unknown');
        var errHelp = me.getMsg('errors.unknown.helptext');
        try{
            var respObj = JSON.parse(batch.exceptions[0].error.statusText);
            error = respObj.message;
        }
        catch (ex) {
            console.log("Could not parse error [Excpn]: ", ex);
            console.log("Could not parse error [batch]: ", batch);
        }

        switch (error) {
            case 'JSONP request timed out.':
                error = me.getMsg('errors.jsonp_request_timed_out');
                errHelp = me.getMsg('errors.jsonp_request_timed_out.helptext');
                break;
        }

        Ext.MessageBox.show({
            buttons: Ext.MessageBox.OK,
            width: 400,
            progress: false,
            draggable: true,
            closable: true,
            title: me.getMsg('titles.savefailed'),
            msg: me.getMsg('messages.savefailed') + '<br /><br />&nbsp;' + error + '<br /><br />' + errHelp,
            icon: Ext.MessageBox.ERROR,
        });

    },
    addItem: function () {
        var me = this;

        if (!me.addEnabled) return false;

        if (typeof (me.innerAddCheck) === 'function') {
            var mayContinue = me.innerAddCheck();
            if (mayContinue === false) return false; //Hard test for false, so we don't cancel on undefined
        }

        var newItem = me.store.model.create();
        newItem.store = me.store;

        if (typeof (me.beforeAdd) === 'function') {
            var mayContinue = me.beforeAdd(newItem);
            if (mayContinue === false) return false; //Hard test for false, so we don't cancel on undefined
        }

        me.startCreate(newItem);
    },
    startAdd: function(){
        throw "NotImplementedException";
    },
    editItem: function (record) {
        var me = this;

        if (!me.editEnabled) return false;

        if (typeof (me.innerEditCheck) === 'function') {
            var mayContinue = me.innerEditCheck();
            if (mayContinue === false) return false; //Hard test for false, so we don't cancel on undefined
        }

        me.startEdit(record);
    },
    startEdit: function () {
        throw "NotImplementedException";
    },
    removeItem: function () {
        var me = this;

        if (!me.removeEnabled) return false;

        if (typeof (me.innerRemoveCheck) === 'function') {
            var mayContinue = me.innerRemoveCheck();
            if (mayContinue === false) return false; //Hard test for false, so we don't cancel on undefined
        }

        me.startRemove();
    },
    startRemove: function () {
        throw "NotImplementedException";
    },
    
    restoreItem: function () {
        var me = this;
        if (!me.restoreEnabled) return false;

        me.startRestore();
    },
    startRestore: function () {
        throw "NotImplementedException";
    },
    showDeletedItems:  function (showDeleted){
        var me = this;
        me.store.clearFilter(true);
        me.showingDeleted = showDeleted;
        if (showDeleted) {
            this.down('#restoreButton').show();
            var filters = [{ property: 'IsDeleted', value: 'ALL' }];
            
            for (var i = 0, l=me.filters.length; i < l; i++) {
                fltr = me.filters[i];
                if(fltr.property != 'IsDeleted'){
                    filters.push(fltr);
                }
            }
            me.store.filter(filters);
        } else {
            this.down('#restoreButton').hide();
            me.store.filter(me.filters)
        }


    },
    initToolbar: function () {
        var me = this;

        me.toolbar = { items: [] };
        if (me.addEnabled) {
            var buttonProps = me.addButton || me.prefabs().TBtnAdd;
            me.toolbar.items.push(Ext.apply(buttonProps, {
                itemId: 'addButton',
                handler: function (e) {
                    me.addItem();
                },
            }));
        }
        if (me.removeEnabled) {
            var buttonProps = me.removeButton || me.prefabs().TBtnRemove;
            me.toolbar.items.push(Ext.apply(buttonProps, {
                itemId: 'removeButton',
                handler: function (e) {
                    me.removeItem();
                },
                disabled: true,
            }));
        }
        if (me.restoreEnabled) {
            var buttonProps = me.restoreButton || me.prefabs().TBtnShowDeleted;
            me.toolbar.items.push(Ext.apply(buttonProps, {
                itemId: 'showDeletedButton',
                handler: function (e, checked) {
                    me.showDeletedItems(checked);
                },
                checked:false
            }));
            buttonProps = me.restoreButton || me.prefabs().TBtnRestore;
            me.toolbar.items.push(Ext.apply(buttonProps, {
                itemId: 'restoreButton',
                handler: function (e) {
                    me.restoreItem();
                },
                disabled: true,
            }));
        }

        return me.toolbar;
    },
    prefabs: function () {
        var me = this;
        if (!me._prefabs) {
            me._prefabs = {
                TBtnSave: {
                    text: me.getMsg('buttons.savechanges'),
                    iconCls: 'saveIcon',
                },
                TBtnCancel: {
                    text: me.getMsg('buttons.undochanges'),
                    iconCls: 'endIcon',
                },
                TBtnAdd: {
                    text: me.getMsg('buttons.add', me.itemName),
                    iconCls: 'addIcon',
                },
                TBtnRemove: {
                    text: me.getMsg('buttons.remove', me.itemName),
                    iconCls: 'deleteIcon',
                },
                TBtnRestore: {
                    text: me.getMsg('buttons.restore', me.itemName),
                    hidden:true,
                    iconCls: 'restoreIcon',
                },
                TBtnShowDeleted: {
                    xtype:'checkbox',
                    boxLabel: me.getMsg('buttons.showdeleted', me.itemsName),
                },
            };
        }
        return me._prefabs;
    },
});