﻿Ext.define('MedwebCloud.util.ViewUtils', {
    singleton: true,
    constructor: function () {
        var self = this;
        function allModules() {
            return MedwebCloud.UIConfig.AvailableModules;
        }
        //Create Modules Store:
        var moduleData = Ext.Array.map(allModules(), function (module, position, allMods) {
            return { value: module, name: MedwebCloud.getMsg('medweb.modules.' + module.toLowerCase() + '.label') };
        });

        self.getAllModules = allModules;
        self.getModuleStore = function () {
            return Ext.create('Ext.data.Store', {
                fields: ['value', 'name'],
                proxy: {
                    type: 'memory',
                    reader: {
                        type: 'json'
                    }
                },
                data: moduleData.slice()
            });
        }
        self.themeStore = Ext.create('Ext.data.Store', {
            fields: ['value', 'name'],
            data: [
                { "value": "ext-all", "name": "Classic" },
                { "value": "ext-all-gray", "name": "Gray" },
                { "value": "ext-all-access", "name": "High Contrast" },
                { "value": "ext-all-neptune", "name": "Neptune" }
            ]
        });

        function allSections() {
            return MedwebCloud.UIConfig.CollageSections;
        }
        self.getAllCollageSections = allSections;
        self.getCollageSectionStore = function () {
            Ext.create('Ext.data.Store', {
                fields: ['Name', 'DisplayName'],
                proxy: {
                    type: 'memory',
                    reader: {
                        type: 'json'
                    }
                },
                data: allSections().slice()
            });
        }

        function allTransferModes() {
            return MedwebCloud.UIConfig.TransferModes;
        }

        //Create TransferMode data:
        var transferModesData = Ext.Array.map(allTransferModes(), function (module, position, allMods) {
            return { Name: module, DisplayName: MedwebCloud.getMsg('medweb.transfermode.' + module.toLowerCase() + '.label') };
        });

        self.getAllTransferModes = allTransferModes;
        self.getTransferModeStore = function () {
            Ext.create('Ext.data.Store', {
                fields: ['Name', 'DisplayName'],
                proxy: {
                    type: 'memory',
                    reader: {
                        type: 'json'
                    }
                },
                data: transferModesData.slice()
            });
        }
    },

    getFormLayout: function () {
        return {
            type: 'vbox',
            align: 'stretch'
        };
    },
    getFormDefaults: function () {
        return {
            anchor: '100%',
            labelWidth: 150
        };
    },
    getField: function (fieldName, opts, labelPrefix, labelSuffix) {
        return Ext.apply({
            fieldLabel: MedwebCloud.getMsg(labelPrefix + fieldName.toLowerCase() + labelSuffix),
            name: fieldName,
            itemId: fieldName
        }, opts);
    }
});