﻿Ext.define('MedwebCloud.view.WindowEditor', {
    extend: 'MedwebCloud.view.BaseEditorWrapper',
    xtype: 'medwebcloud.windoweditor',
    
    columns: null,

    initComponent: function () {

        var me = this;

        me.store = Ext.create(me.storeType, {
            autoLoad: true,
            remoteSort: true,
            remoteFilter: true,
            filters: me.filters,
            expand: me.expands,
            sorters: me.sorters,
        });

        var gridProps = {
            xtype: 'gridpanel',
            store: me.store,
            region: 'center',
            layout: 'fit',
            border: false,
            plugins: [Ext.create('Ext.ux.grid.plugin.HeaderFilters')],
            dockedItems: [{
                xtype: 'pagingtoolbar',
                store: me.store,
                dock: 'bottom',
                displayInfo: true
            }],
            viewConfig: {
                getRowClass: function (rec, rowIdx, params, store) {
                    if (!!rec && !!rec.get) {
                        return rec.get('IsDeleted') ? 'deleted-record' : '';
                    }
                }
            },
            listeners: {
                itemdblclick: function (view, record, item, index, e, eOpts) {
                    me.editItem(record);
                },
                selectionchange: function (selModel) {
                    var recs = selModel.getSelection();
                    if (recs.length == 1) {
                        var recIsDeleted = recs[0].get('IsDeleted');
                        me.setAllowRemove(!recIsDeleted);
                        me.setAllowRestore(recIsDeleted);
                    } else {
                        me.setAllowRemove(false);
                        me.setAllowRestore(false);
                    }                 
                }
            },
        };

        if (me.height && me.width)
        {
            gridProps.layout = null;
            gridProps.width = me.width;
            gridProps.height = me.height - 54; //Magic height of bottom bar
        }

        if (me.columns)
        {
            gridProps.columns = Ext.apply({}, me.columns);

            if (me.columns.defaults) {
                if (me.columns.defaults.menuDisabled !== false) {
                    gridProps.columns.defaults.menuDisabled = true;
                }
            }
            else if (me.columns.items)
            {
                gridProps.columns.defaults = { menuDisabled: true };
            }
            else //Assume we were just passed an array of columns
            {
                gridProps.columns.items = me.columns;
                gridProps.columns.defaults = { menuDisabled: true };
            }
        }
        else
        {
            gridProps.columns = [{
                dataIndex: 'Id',
                text: 'Id',
                flex: 1,
            }];
        }

        me.tbar = me.initToolbar();

        this.callParent(arguments);
        
        me.grid = me.add(gridProps);

    },
    addItem: function () {
        this.callParent(arguments);
    },
    startCreate: function (record) {
        var me = this;

        var editorWindow = Ext.create('MedwebCloud.view.EditorWindow', {
            formType: me.addForm || me.formType,
            autoShow: true,
            title: (typeof me.addTitle === 'function') ? me.addTitle(record) : me.addTitle,
            record: record,
            customArgs: me.customArgs,
            listeners: {
                editComplete: function (values) {
                    record.set(values);
                    me.store.insert(0, record);
                    me.onOperationComplete();
                },
                forceComplete: function (success) {
                    if (success) {
                        me.onOperationComplete();
                    } else {
                        me.onOperationFailed();
                    }
                }
            }
        });
    },
    editItem: function (record) {
        this.callParent(arguments);
    },
    startEdit: function (record) {
        var me = this;

        var editorWindow = Ext.create('MedwebCloud.view.EditorWindow', {
            formType: me.editForm || me.formType,
            autoShow: true,
            title: (typeof me.editTitle === 'function') ? me.editTitle(record) : me.editTitle,
            record: record,
            customArgs: me.customArgs,
            listeners: {
                editComplete: function (values) {
                    record.set(values);
                    me.onOperationComplete();
                },
                forceComplete: function (success) {
                    if (success) {
                        me.onOperationComplete();
                    } else {
                        me.onOperationFailed();
                    }
                }
            }
        });
    },
    removeItem: function () {
        this.callParent(arguments);
    },
    startRemove: function () {
        var me = this;

        var selModel = me.grid.getSelectionModel();
        if (selModel.hasSelection()) {
            me.openDeleteWindow(selModel.getSelection());
        }
    },
    openDeleteWindow: function (records) {
        var me = this;

        Ext.MessageBox.show({
            title: me.getMsg('titles.confirmdelete'),
            msg: me.getMsg('messages.confirmdelete'),
            modal: true,
            buttons: Ext.Msg.YESNO,
            buttonText: {
                yes: me.getMsg('buttons.deleteyes'),
                no: me.getMsg('buttons.deleteno'),
            },
            icon: Ext.Msg.QUESTION,
            fn: function (btnId) {
                if (btnId === 'yes') {
                    for (var i = 0; i < records.length; i++)
                    {
                        records[i].set('IsDeleted', true);
                        if (me.showingDeleted) {
                            me.store.save();
                        } else {
                            me.store.remove(records[i]);
                        }
                    }
                    me.onOperationComplete(function () {
                        var gridSel = me.grid.getSelectionModel();
                        if (gridSel) {
                            gridSel.clearSelections();
                        }
                    });
                }
            }
        });

    },

    startRestore: function () {
        var me = this;

        var selModel = me.grid.getSelectionModel();
        if (selModel.hasSelection()) {
            me.openRestoreWindow(selModel.getSelection());
        }
    },
    openRestoreWindow: function (records) {
        var me = this;

        Ext.MessageBox.show({
            title: me.getMsg('titles.confirmrestore'),
            msg: me.getMsg('messages.confirmrestore'),
            modal: true,
            buttons: Ext.Msg.YESNO,
            buttonText: {
                yes: me.getMsg('buttons.restoreyes'),
                no: me.getMsg('buttons.restoreno'),
            },
            icon: Ext.Msg.QUESTION,
            fn: function (btnId) {
                if (btnId === 'yes') {
                    for (var i = 0; i < records.length; i++)
                    {
                        records[i].set('IsDeleted', false);
                        me.store.save();
                    }
                    me.onOperationComplete(function () {
                        var gridSel = me.grid.getSelectionModel();
                        if (gridSel) {
                            gridSel.clearSelections();
                        }
                    });
                }
            }
        });
    },

    onOperationComplete: function (onSuccess) {
        this.callParent(arguments);
    },
    onOperationFailed: function () {
        this.callParent(arguments);
    },
    setAllowRemove: function (allowRemove) {
        if (this.removeEnabled) {
            this.down('#removeButton').setDisabled(!allowRemove);
        }
    },
    setAllowRestore: function (allowRestore) {
        if (this.restoreEnabled) {
            this.down('#restoreButton').setDisabled(!allowRestore);
        }
    },
    initToolbar: function () {
        return this.callParent(arguments);
    },
});