﻿Ext.define('MedwebCloud.view.UserManagementPanel', {
    extend: 'Ext.panel.Panel',
    xtype: 'MedwebCloud.usermanagementpanel',
    requires: [
        'MedwebCloud.view.UsersGrid',
        'MedwebCloud.view.UserEditor',
        'MedwebCloud.view.ChangePasswordForm'
    ],
    initComponent: function () {
        var me = this;
        var btnAdd = MedwebCloud.getMsg('medweb.view.usermanagementpanel.button.add.label');
        var btnChange = MedwebCloud.getMsg('medweb.view.usermanagementpanel.button.changepassword.label');
        var btnUpdates = MedwebCloud.getMsg('medweb.view.usermanagementpanel.button.update.label');

        Ext.apply(me, {
            layout: 'border',
            items: [{
                region: 'west',
                layout: 'fit',
                border: false,
                width: 350,
                split: true,
                tbar: [{
                    xtype: 'button',
                    text: btnAdd,
                    iconCls: 'addIcon',
                    handler: Ext.Function.bind(me.addUserHandler, me)
                }],
                items: [{
                    border: false,
                    xtype: 'MedwebCloud.usersgrid',
                    itemId: 'usersGrid',
                    tbar: [{
                        xtype: 'gridsearchbar',
                        dataIndex: 'DisplayName',
                        border: false,
                        autoSearch: true,
                        anyMatch: true
                    }],
                    listeners: {
                        itemclick: function (grid, record) {
                            var form = me.down('#userEditor');
                            if (record) {
                                form.loadRecord(record);
                                form.setDisabled(!(MedwebCloud.userProfile.IsSuperAdmin || record.get("CompanyId") == MedwebCloud.userProfile.CompanyId));
                                //me.down('#deleteButton').setDisabled(false);
                            }
                        }
                    }
                }]
            }, {
                region: 'center',
                xtype: 'MedwebCloud.usereditor',
                hidePasswordFields: true,
                itemId: 'userEditor',
                frame: true,
                margin: 5,
                padding: 15,
                disabled: true,
                buttons: [{
                    xtype: 'button',
                    hidden: true,
                    text: btnChange,
                    iconCls: 'lockIcon',
                    handler: Ext.Function.bind(me.changePasswordHandler, me)
                }, {
                    text: btnUpdates,
                    iconCls: 'saveIcon',
                    handler: function (btn) {
                        btn.setDisabled(true);
                        var data = btn.up('#userEditor').getForm().getFieldValues();
                        if (data.Password != data.ConfirmPassword) {
                            Ext.Msg.alert(
                                MedwebCloud.getMsg('medweb.view.usermanagementpanel.action.update.validationerror.title'),
                                MedwebCloud.getMsg('medweb.view.usermanagementpanel.action.update.validationerror.message'));
                            return;
                        }
                        var selectedUser = me.getSelectedUser();
                        data.Id = selectedUser.get('Id');

                        Ext.Ajax.request({
                            url: MedwebCloud.api('/v1/users/' + selectedUser.get('Id')),
                            method: 'POST',
                            jsonData: data,
                            success: function (response) {
                                btn.setDisabled(false);
                                var form = me.down('#userEditor');
                                form.getForm().reset();
                                form.setDisabled(true);
                                me.down('grid').loadGrid();
                            },
                            failure: function (response) {
                                btn.setDisabled(false);
                                Ext.Msg.alert(
                                    MedwebCloud.getMsg('medweb.view.usermanagementpanel.action.update.failure.title'), response.responseText);
                            }
                        });
                    }
                }]
            }]
        });

        me.callParent(arguments);
    },
    addUserHandler: function () {
        var me = this;
        var title = MedwebCloud.getMsg('medweb.view.usermanagementpanel.action.add.title');
        var btnOk = MedwebCloud.getMsg('medweb.view.usermanagementpanel.action.add.button.positive.label');
        var btnCancel = MedwebCloud.getMsg('medweb.view.usermanagementpanel.action.add.button.negative.label');
        var popup = Ext.create('Ext.Panel', {
            title: title,
            modal: true,
            centered: true,
            floating: true,
            closable: true,
            draggable: true,
            width: 400,
            height: 300,
            layout: 'fit',
            items: [{
                xtype: 'MedwebCloud.usereditor',
                itemId: 'addUserForm',
                border: false,
                //frame: true,
                //padding: 15,
                buttons: [{
                    text: btnOk,
                    iconCls: 'saveIcon',
                    handler: function (btn) {
                        btn.setDisabled(true);
                        var data = btn.up('#addUserForm').getValues();
                        if (data.Password != data.ConfirmPassword) {
                            Ext.Msg.alert(
                                MedwebCloud.getMsg('medweb.view.usermanagementpanel.action.update.validationerror.title'),
                                MedwebCloud.getMsg('medweb.view.usermanagementpanel.action.update.validationerror.message'));
                            return;
                        }
                        Ext.Ajax.request({
                            url: MedwebCloud.api('/v1/users'),
                            method: 'POST',
                            jsonData: data,
                            success: function (response) {
                                popup.close();
                                me.down('#usersGrid').loadGrid();
                            },
                            failure: function (response) {
                                btn.setDisabled(false);
                                Ext.Msg.alert(MedwebCloud.getMsg('medweb.view.usermanagementpanel.action.update.failure.title'), response.responseText);
                            }
                        });
                    }
                }, {
                    text: btnCancel,
                    handler: function () {
                        popup.close();
                    }
                }]
            }]
        });

        popup.show();
    },

    deleteUserHandler: function () {
        var me = this;
        var ttl = MedwebCloud.getMsg('medweb.view.usermanagementpanel.action.delete.confirm.title');
        var msg = MedwebCloud.getMsg('medweb.view.usermanagementpanel.action.delete.confirm.message');

        Ext.MessageBox.confirm(ttl, msg, function (btn) {
            if (btn === 'yes') {
                var selectedUser = me.getSelectedUser();

                Ext.Ajax.request({
                    method: 'POST',
                    url: MedwebCloud.api('/v1/users/delete/' + selectedUser.get('Id')),
                    success: function (response) {
                        //me.down('#deleteButton').setDisabled(true);
                        //clear and disable form
                        var form = me.down('#userEditor');
                        form.getForm().reset();
                        form.setDisabled(true);
                        me.down('#usersGrid').loadGrid();
                    },
                    failure: function (response) {
                        Ext.Msg.alert(MedwebCloud.getMsg('medweb.view.usermanagementpanel.action.delete.failure.title'), response.responseText);
                    }
                });
            }
        });
    },

    changePasswordHandler: function () {
        var me = this;

        var selectedUser = me.getSelectedUser();
        var title = MedwebCloud.getMsg('medweb.view.usermanagementpanel.action.changepassword.title');
        var btnOK = MedwebCloud.getMsg('medweb.view.usermanagementpanel.action.changepassword.button.positive.label');
        var btnCancel = MedwebCollage.getMsg('medweb.view.usermanagementpanel.action.changepassword.button.negative.label');
        var popup = Ext.create('Ext.Panel', {
            title: title,
            modal: true,
            centered: true,
            floating: true,
            closable: true,
            draggable: true,
            width: 400,
            height: 200,
            layout: 'fit',
            items: [{
                xtype: 'MedwebCloud.changepasswordform',
                itemId: 'changePasswordForm',
                frame: true,
                padding: 15,
                buttons: [{
                    text: btnOK,
                    handler: function (btn) {
                        btn.setDisabled(true);
                        var data = btn.up('#changePasswordForm').getValues();
                        if (data.NewPassword != data.ConfirmPassword) {
                            Ext.Msg.alert(
                                MedwebCloud.getMsg('medweb.view.usermanagementpanel.action.update.validationerror.title'),
                                MedwebCloud.getMsg('medweb.view.usermanagementpanel.action.update.validationerror.message'));
                            return;
                        }
                        data.UserId = selectedUser.get('Id');
                        Ext.Ajax.request({
                            url: MedwebCloud.api('/v1/passwordChange'),
                            jsonData: data,
                            success: function (response) {
                                popup.close();
                                //me.down('#usersGrid').loadGrid();
                            },
                            failure: function (response) {
                                btn.setDisabled(false);
                                Ext.Msg.alert(MedwebCloud.getMsg('medweb.view.usermanagementpanel.action.changepassword.failure.title'), response.responseText);
                            }
                        });
                    }
                }, {
                    text: btnCancel,
                    handler: function () {
                        popup.close();
                    }
                }]
            }]
        });

        popup.show();
    },

    getSelectedUser: function () {
        var me = this;

        var grid = me.down('#usersGrid');
        var selections = grid.getSelectionModel().getSelection();
        if (selections.length < 1) { return null; }
        var selectedUser = selections[0];

        return selectedUser;
    }
});