﻿Ext.define('MedwebCloud.view.VitalsPlusGrid', {
    extend: 'Ext.grid.Panel',
    xtype: 'medwebcloud.vitalsplusgrid',
    client: null,    
    initComponent: function () {
        var panel;
        var me = this;
        Ext.apply(me, {
            store: Ext.create('MedwebCloud.store.Patients', {

            }),
            columns: [{
                dataIndex: 'Id',
                text: 'Id',
                flex: 0.25
            }, {
                dataIndex: 'Name',
                text: 'Name',
                flex: 1
            }]
        });

        this.callParent(arguments);

        
        var userProfile = MedwebCloud.userProfile;

        this.client.join({
            userId: userProfile.Id.toString(),
            userNickname: userProfile.DisplayName,
            channel: '/VitalsPlus',
            onSuccess: function (args) {
                var users = args.getUsers();
                for (var i = 0; i < users.length; i++) {
                    var user = users[i];
                    me.addPatient({
                        Id: user.getUserId(),
                        Name: user.getUserNickname()
                    });
                }

                //TEST DATA
                if (window.location.hostname.indexOf('medweb.dev') > -1 || window.location.hostname.indexOf('localhost') > -1) {
                    me.addPatient({
                        Id: '1234',
                        Name: 'Doe, John'
                    });
                }
            },
            onReceive: function (args) {
                console.log('r', args)
            },
            onUserJoin: function (args) {
                var user = args.getJoinedUser();

                me.addPatient({
                    Id: user.getUserId(),
                    Name: user.getUserNickname()
                });
            },
            onUserLeave: function (args) {
                var user = args.getLeftUser();
                me.removePatient(user.getUserId());
                if(me.store.data.getCount()=='0'){
                    $("#noPatientWarning").show();
                }
            }
        })
    },
    addPatient: function (config) {
        var idx = this.store.findBy(function (r) {
            return r.get('Id') == config.Id;
        });
        if (idx == -1) {
            this.store.add(new MedwebCloud.model.Patient({
                Id: config.Id,
                Name: config.Name
            }));
            $("#noPatientWarning").hide();
            var id = config.Id.replace('@', '').replace('.', '');
            var dashboardPanelId = 'dashboardPanel-' + id;
            var patientPanelId = 'patientPanel' + id;            
            var title = config.Name + ' (PID: ' + config.Id + ')';

            panel = Ext.create('Ext.panel.Panel', {
                id: dashboardPanelId,
                width: 400,
                height: 225,
                closable: false,
                cls: 'float-tabs ',
                title:'<div style="color:white">' + title + '</div>',
                renderTo: Ext.get('tabpanel-1115-body'),
                items: [{
                    id: patientPanelId,
                    height: 225,
                    xtype: 'medwebcloud.vitalspluspanel',
                    client: this.client,
                    record: config,
                    recordId: id
                   
                }]
            });
            
            panel.header.addClass("headerColor");
        }
    },
    removePatient: function (id) {
        var idx = this.store.findBy(function (r) {
            return r.get('Id') == id;
        });
        this.store.removeAt(idx);
        var id = id.replace('@', '').replace('.', '');
        $('#dashboardPanel-' + id).remove();
        $('#patientPanel' + id).remove();
        // TODO: close the tab?
    }
});

