﻿/** SCREEN SHARING **/
if (!window.fm.icelink.webrtc.screenshare) { window.fm.icelink.webrtc.screenshare = {}; }

window.fm.icelink.webrtc.screenshare.active = false;
window.fm.icelink.webrtc.screenshare.extensionInstalled = false;

var screenShareId = "";

window.addEventListener('message', function (event) {
    if (event.origin != window.location.origin) return;

    // content-script will send a 'SS_PING' msg if extension is installed
    if (event.data.type && (event.data.type === 'FM_SS_PING')) {
        window.fm.icelink.webrtc.screenshare.extensionInstalled = true;
        if (fm.icelink.webrtc.screenshare.extensionCallback) {
            fm.icelink.webrtc.screenshare.extensionCallback();
        }
    }

    // user chose a stream
    if (event.data.type && (event.data.type === 'FM_SS_DIALOG_SUCCESS') && fm.icelink.webrtc.screenshare.active) {
        window.fm.icelink.webrtc.screenshare.screenShareId = event.data.streamId;
        window.fm.icelink.webrtc.screenshare.screenShareCallback(window.fm.icelink.webrtc.screenshare.screenShareCallbackArg);
        console.log('streamId = ' + screenShareId);
    }

    // user clicked on 'cancel' in choose media dialog
    if (event.data.type && (event.data.type === 'FM_SS_DIALOG_CANCEL')) {
        console.log('User cancelled!');
        if (window.fm.icelink.webrtc.screenshare.screenShareCancelCallback) {
            window.fm.icelink.webrtc.screenshare.screenShareCancelCallback();
        }
    }
});


fm.icelink.webrtc.screenshare.activate = function () {
    if ("https:" != location.protocol) {
        console.log('screen sharing will only work if you connect via https');
        window.fm.icelink.webrtc.screenshare.active = false;
    } else {
        window.fm.icelink.webrtc.screenshare.active = true;
    }
}

fm.icelink.webrtc.screenshare.deactivate = function () {
    window.fm.icelink.webrtc.screenshare.active = false;
}

/* icelink overrides */
fm.icelink.webrtc.localMediaStream.prototype.initialize = function (callback) {
    this._initialized = true;
    if (fm.icelink.webrtc.screenshare.active && fm.icelink.webrtc.screenshare.extensionInstalled) {
        window.fm.icelink.webrtc.screenshare.screenShareCallback = callback;
        window.fm.icelink.webrtc.screenshare.screenShareCallbackArg = this;
        window.postMessage({ type: 'FM_SS_UI_REQUEST', text: 'start' }, '*');
    } else {
        if (callback) {
            return callback(this);
        }
    }
};

fm.icelink.webrtc.localMediaStream.prototype.start = function (startArgs) {
    var audio, getUserMediaFailure, getUserMediaSuccess, video,
      _this = this;
    if (!this._initialized) {
        throw new Error('Local media stream must be initialized using "initialize" before calling "start".');
    }
    getUserMediaSuccess = function (localStream) {
        var acp, vcp;
        _this.setBackingStream(localStream);
        acp = _this.getAudioCaptureProvider();
        vcp = _this.getVideoCaptureProvider();
        if (acp) {
            acp.setLocalStream(_this);
        }
        if (vcp) {
            vcp.setLocalStream(_this);
        }
        return _this.raiseStartSuccess(startArgs, localStream);
    };
    getUserMediaFailure = function (error) {
        if (typeof error === 'string' || error instanceof String) {
            error = new Error(error);
        }
        if (!error.message) {
            error = new Error('Permission denied. (The camera may be locked or you may need to access this page through a webserver instead of the file system.)');
        }
        return _this.raiseStartFailure(startArgs, error);
    };
    if (window.fm.icelink.webrtc.screenshare.active) {
        return navigator.webkitGetUserMedia({
            audio: false,
            video: {
                mandatory: {
                    chromeMediaSource: 'desktop',
                    chromeMediaSourceId: window.fm.icelink.webrtc.screenshare.screenShareId,
                    maxWidth: window.screen.width > 1920 ? window.screen.width : 1920,
                    maxHeight: window.screen.height > 1080 ? window.screen.height : 1080
                }
            }
        }, getUserMediaSuccess, getUserMediaFailure);
    } else {
        audio = false;
        if (startArgs.getAudio()) {
            audio = true;
        }
        video = false;
        if (startArgs.getVideo()) {
            video = {
                optional: [],
                mandatory: {
                    maxWidth: startArgs.getVideoWidth(),
                    maxHeight: startArgs.getVideoHeight(),
                    maxFrameRate: startArgs.getVideoFrameRate()
                }
            };
        }
        if (audio && video) {
            return navigator.fmGetUserMedia({
                audio: audio,
                video: video
            }, getUserMediaSuccess, getUserMediaFailure);
        } else if (audio) {
            return navigator.fmGetUserMedia({
                audio: audio
            }, getUserMediaSuccess, getUserMediaFailure);
        } else if (video) {
            return navigator.fmGetUserMedia({
                video: video
            }, getUserMediaSuccess, getUserMediaFailure);
        } else {
            return getUserMediaSuccess(null);
        }
    }
};

if (window.chrome && window.chrome.webstore && window.chrome.webstore.install) {
    fm.icelink.webrtc.screenshare.supported = true;
}